@@ -43,7 +43,6 @@ import java.util.Random;
  * 
  * Histogram
  * @author Modest von Korff
- * @version 1.0
  * 29 Feb 2008 MvK: Start implementation
  * 06 Jun 2016 MvK: added cumulative histogram to output.
  * 03.12.2018 improvements toString()
@@ -68,6 +67,15 @@ public class Histogram {
 
 	private double [] arrRaw;
 
+	private int digits;
+
+	/**
+	 *
+	 * @param arrRaw data
+	 * @param min minimum value considered in the histogram
+	 * @param max maximum value considered in the histogram
+	 * @param bins
+	 */
 	public Histogram(double [] arrRaw, double min, double max, int bins) {
 		initialize(arrRaw,min,max,bins);
 	}
@@ -106,6 +114,8 @@ public class Histogram {
 
 		arrBins = new float [bins];
 
+		digits = -1;
+
 		calcBins();
 
 		calcHistogram();
@@ -207,6 +217,10 @@ public class Histogram {
 		return maximumX;
 	}
 
+	public void setDigits(int digits) {
+		this.digits = digits;
+	}
+
 	/**
      * First row: bins, upper border.
      * Sec row: frequencies
@@ -218,8 +232,9 @@ public class Histogram {
     	int logRange = (int)Math.log10(range);
     	int bins = arrFrequencies.length;
     	int logBins = (int)Math.log10(bins);
-    	
-    	int digits = logBins-logRange+1;
+
+		if(digits<0)
+    		digits = logBins-logRange+1;
     	
     	String sFormatDigits="";
     	for (int i = 0; i < digits; i++) {
@@ -243,30 +258,20 @@ public class Histogram {
 		int sumFreq = 0;
 
     	for (int i = 0; i < arrStrY.length; i++) {
-    		
     		String sX = nfX.format(arrBins[i]);
-
     		String sY = nfY.format(arrFrequencies[i]);
-
 			sumFreq += arrFrequencies[i];
-
 			double fractionCumulative = (double)sumFreq / arrRaw.length;
-
     		String sFractionCumulative = nfFractionCumulative.format(fractionCumulative);
-
 			int maxLen = sX.length();
-
 			if(sY.length() > maxLen) {
-
 				maxLen = sY.length();
 			}
 
 			if(sFractionCumulative.length() > maxLen) {
-
 				maxLen = sFractionCumulative.length();
 			}
 
-
     		while(sX.length() < maxLen){
     			sX = " " + sX ;
     		}
@@ -311,7 +316,18 @@ public class Histogram {
     	return sb.toString();
     }
 
-	public double getBinWidth() {
+	/**
+	 * Replaces the spaces with tab. So, the output can be used in Excel by copy/paste.
+	 * @return
+	 */
+	public String toStringWithTabs() {
+		String s = toString();
+		String sTab = s.replaceAll("[ ]{1,}", "\t");
+		return sTab;
+	}
+
+
+		public double getBinWidth() {
 		return binwidth;
 	}
 
@@ -321,14 +337,14 @@ public class Histogram {
 		double min=0;
 		double max=0.1;
 		double minBin=0;
-		double maxBin=0.1;
+		double maxBin=1;
 		int bins=20;
 
 		double [] arr = new double [n];
 		Random rnd = new Random();
 		for (int i = 0; i < n; i++) {
 			// arr[i]= min + rnd.nextInt((int)(max-min));
-			arr[i]= min + rnd.nextDouble()*0.1;
+			arr[i]= min + rnd.nextDouble();
 		}
 
 
