@@ -38,9 +38,9 @@ import com.actelion.research.chem.reaction.Reaction;
 import com.actelion.research.chem.reaction.ReactionArrow;
 import com.actelion.research.gui.generic.GenericDepictor;
 import com.actelion.research.gui.generic.GenericDrawContext;
+import com.actelion.research.gui.generic.GenericRectangle;
 
 import java.awt.*;
-import java.awt.geom.Rectangle2D;
 
 
 
@@ -56,7 +56,7 @@ public class ExtendedDepictor {
     private int						mDisplayMode,mReactantCount,mMarkushCoreCount,mChemistryType;
     private boolean					mReactionLayoutNeeded;
     private DepictorTransformation	mTransformation;
-    private Color                   mFragmentNoColor;
+    private int                     mFragmentNoColor,mDefaultAVBL;
 
     public ExtendedDepictor(StereoMolecule mol, DrawingObjectList drawingObjectList) {
         if (mol != null) {
@@ -139,15 +139,26 @@ public class ExtendedDepictor {
 			for (int i=0; i<mCatalyst.length; i++)
 				mCatalystDepictor[i] = new GenericDepictor(mCatalyst[i]);
 			}
+		mDefaultAVBL = AbstractDepictor.cOptAvBondLen;
         }
 
     public void setDisplayMode(int displayMode) {
         mDisplayMode = displayMode;
         }
 
-    public void setFragmentNoColor(Color c) {
+    public void setDefaultAVBL(int avbl) {
+    	mDefaultAVBL = avbl;
+    }
+
+    public void setFactorTextSize(double factor) {
+    	if (mDepictor != null)
+	    	for (GenericDepictor d:mDepictor)
+	    		d.setFactorTextSize(factor);
+        }
+
+    public void setFragmentNoColor(int argb) {
         // use setFragmentNoColor(null) if you don't want fragment numbers to be shown
-        mFragmentNoColor = c;
+        mFragmentNoColor = argb;
         }
 
     public int getMoleculeCount() {
@@ -167,43 +178,61 @@ public class ExtendedDepictor {
         return mDepictor[i];
         }
 
-    public void setForegroundColor(Color foreGround, Color background) {
-        if (mDepictor != null)
-            for (GenericDepictor d:mDepictor)
-                d.setForegroundColor(foreGround, background);
-
-		if (mCatalystDepictor != null)
-			for (GenericDepictor d:mCatalystDepictor)
-				d.setForegroundColor(foreGround, background);
-        }
-
-	public void setOverruleColor(Color foreGround, Color background) {
-		if (mDepictor != null)
-			for (GenericDepictor d:mDepictor)
-				d.setOverruleColor(foreGround, background);
-
-		if (mCatalystDepictor != null)
-			for (GenericDepictor d:mCatalystDepictor)
-				d.setOverruleColor(foreGround, background);
+    @Deprecated
+    // Use rgb version of this method instead
+	public void setForegroundColor(Color foreGround, Color background) {
+	    setForegroundColor(foreGround.getRGB(), background.getRGB());
 		}
 
-    public void paint(GenericDrawContext context) {
-        Color saveColor = context.getColor();
+	public void setForegroundColor(int foreground, int background) {
+        if (mDepictor != null)
+            for (GenericDepictor d:mDepictor)
+                d.setForegroundColor(foreground, background);
+
+		if (mCatalystDepictor != null)
+			for (GenericDepictor d:mCatalystDepictor)
+				d.setForegroundColor(foreground, background);
+        }
+
+	@Deprecated
+	// Use rgb version of this method instead
+	public void setOverruleColor(Color foreground, Color background) {
+		if (mDepictor != null)
+			for (GenericDepictor d:mDepictor)
+				d.setOverruleColor(foreground, background);
+
+		if (mCatalystDepictor != null)
+			for (GenericDepictor d:mCatalystDepictor)
+				d.setOverruleColor(foreground, background);
+		}
+
+	public void setOverruleColor(int foreground, int background) {
+		if (mDepictor != null)
+			for (GenericDepictor d:mDepictor)
+				d.setOverruleColor(foreground, background);
+
+		if (mCatalystDepictor != null)
+			for (GenericDepictor d:mCatalystDepictor)
+				d.setOverruleColor(foreground, background);
+	}
+
+	public void paint(GenericDrawContext context) {
+        int saveRGB = context.getRGB();
         int fontSize = context.getFontSize();
         try {
             paintFragmentNumbers(context);
             paintStructures(context);
             paintDrawingObjects(context);
         } finally {
-            context.setColor(saveColor);
+            context.setRGB(saveRGB);
             context.setFont(fontSize, false, false);
         }
     }
 
     public void paintFragmentNumbers(GenericDrawContext context) {
-        if (mFragmentNoColor != null && mMolecule != null) {
+        if (mFragmentNoColor != 0 && mMolecule != null) {
             double averageBondLength = calculateAverageBondLength();
-            context.setColor(mFragmentNoColor);
+            context.setRGB(mFragmentNoColor);
             context.setFont((int)(1.6*averageBondLength), true, false);
             for (int i=0; i<mMolecule.length; i++) {
                 if (mMolecule[i].getAllAtoms() != 0) {
@@ -218,7 +247,7 @@ public class ExtendedDepictor {
                     cog.y = (int)mDepictor[i].getTransformation().transformY(cog.y);
 
 
-                    String str = (mChemistryType == TYPE_MOLECULES) ? "F"+(i+1)
+                    String str = (mChemistryType == TYPE_MOLECULES) ? ""+(i+1)
                                : (mChemistryType == TYPE_MARKUSH) ? ((i < mMarkushCoreCount) ? ""+(char)('A'+i) : "R"+(i+1-mMarkushCoreCount))
                                : (mChemistryType == TYPE_REACTION) ? ((i < mReactantCount) ? ""+(char)('A'+i) : "P"+(i+1-mReactantCount)) : "?"+(i+1);
                     context.drawCenteredString(cog.x, cog.y, str);
@@ -238,9 +267,9 @@ public class ExtendedDepictor {
 
     public void paintStructures(GenericDrawContext context) {
         if (mDepictor != null) {
-            for (int i=0; i<mDepictor.length; i++) {
-                mDepictor[i].setDisplayMode(mDisplayMode);
-                mDepictor[i].paint(context);
+            for (GenericDepictor d:mDepictor) {
+                d.setDisplayMode(mDisplayMode);
+                d.paint(context);
 /*
 Rectangle2D.Float r = mDepictor[i].getBoundingRect();
 if (r != null) {
@@ -250,9 +279,10 @@ g.drawRect((int)r.x, (int)r.y, (int)r.width, (int)r.height);
                 }
             }
 		if (mCatalystDepictor != null) {
-			for (int i=0; i<mCatalystDepictor.length; i++) {
-//				mCatalystDepictor[i].setDisplayMode(mDisplayMode);
-				mCatalystDepictor[i].paint(context);
+			for (GenericDepictor d:mCatalystDepictor) {
+//				d.setDisplayMode(mDisplayMode);
+//				d.setFactorTextSize(mFactorTextSize);
+				d.paint(context);
 /*
 Rectangle2D.Float r = mCatalystDepictor[i].getBoundingRect();
 if (r != null) {
@@ -276,7 +306,7 @@ g.drawRect((int)r.x, (int)r.y, (int)r.width, (int)r.height);*/
             }
         }
 
-    public DepictorTransformation updateCoords(GenericDrawContext context, Rectangle2D.Double viewRect, int mode) {
+	public DepictorTransformation updateCoords(GenericDrawContext context, GenericRectangle viewRect, int mode) {
     // returns full transformation that moves/scales original molecules/objects into viewRect
         validateView(context, viewRect, mode);
 
@@ -285,20 +315,20 @@ g.drawRect((int)r.x, (int)r.y, (int)r.width, (int)r.height);*/
             }
         else {
             if (mMolecule != null)
-                for (int i=0; i<mMolecule.length; i++)
-                    mTransformation.applyTo(mMolecule[i]);
+                for (StereoMolecule mol:mMolecule)
+                    mTransformation.applyTo(mol);
 
             if (mDrawingObjectList != null)
-                for (int i=0; i<mDrawingObjectList.size(); i++)
-                    mTransformation.applyTo(mDrawingObjectList.get(i));
+                for (AbstractDrawingObject o:mDrawingObjectList)
+                    mTransformation.applyTo(o);
 
             if (mDepictor != null)
-                for (int i=0; i<mDepictor.length; i++)
-                    mDepictor[i].getTransformation().clear();
+                for (GenericDepictor d:mDepictor)
+                    d.getTransformation().clear();
 
 			if (mCatalystDepictor != null)
-				for (int i=0; i<mCatalystDepictor.length; i++)
-					mCatalystDepictor[i].getTransformation().clear();
+				for (GenericDepictor d:mCatalystDepictor)
+					d.getTransformation().clear();
 
 			DepictorTransformation t = mTransformation;
             mTransformation = new DepictorTransformation();
@@ -306,32 +336,29 @@ g.drawRect((int)r.x, (int)r.y, (int)r.width, (int)r.height);*/
             }
         }
 
-    public DepictorTransformation validateView(GenericDrawContext context, Rectangle2D.Double viewRect, int mode) {
+    public DepictorTransformation validateView(GenericDrawContext context, GenericRectangle viewRect, int mode) {
     // returns incremental transformation that moves/scales already transformed molecules/objects into viewRect
         if (mReactionLayoutNeeded)
             layoutReaction(context);
 
-        Rectangle2D.Double boundingRect = null;
+        GenericRectangle boundingRect = null;
         if (mDepictor != null) {
-            for (int i=0; i<mDepictor.length; i++) {
-                mDepictor[i].validateView(context, null, 0);
-                boundingRect = (boundingRect == null) ? mDepictor[i].getBoundingRect()
-                            : (Rectangle2D.Double)boundingRect.createUnion(mDepictor[i].getBoundingRect());
+            for (GenericDepictor d:mDepictor) {
+                d.validateView(context, null, 0);
+                boundingRect = (boundingRect == null) ? d.getBoundingRect() : boundingRect.union(d.getBoundingRect());
                 }
             }
 		if (mCatalystDepictor != null) {
-			for (int i=0; i<mCatalystDepictor.length; i++) {
-				mCatalystDepictor[i].validateView(context, null, 0);
-				boundingRect = (boundingRect == null) ? mCatalystDepictor[i].getBoundingRect()
-						: (Rectangle2D.Double)boundingRect.createUnion(mCatalystDepictor[i].getBoundingRect());
+			for (GenericDepictor d:mCatalystDepictor) {
+				d.validateView(context, null, 0);
+				boundingRect = (boundingRect == null) ? d.getBoundingRect() : boundingRect.union(d.getBoundingRect());
 				}
 			}
         if (mDrawingObjectList != null) {
-            for (int i=0; i<mDrawingObjectList.size(); i++) {
-                Rectangle2D.Double objectBounds = mDrawingObjectList.get(i).getBoundingRect(context);
+            for (AbstractDrawingObject o:mDrawingObjectList) {
+	            GenericRectangle objectBounds = o.getBoundingRect(context);
                 mTransformation.applyTo(objectBounds);
-                boundingRect = (boundingRect == null) ? objectBounds
-                        : (Rectangle2D.Double)boundingRect.createUnion(objectBounds);
+                boundingRect = (boundingRect == null) ? objectBounds : boundingRect.union(objectBounds);
                 }
             }
 
@@ -346,12 +373,12 @@ g.drawRect((int)r.x, (int)r.y, (int)r.width, (int)r.height);*/
             t.applyTo(mTransformation);
 
             if (mDepictor != null)
-                for (int i=0; i<mDepictor.length; i++)
-                    mDepictor[i].applyTransformation(t);
+                for (GenericDepictor d:mDepictor)
+                    d.applyTransformation(t);
 
 			if (mCatalystDepictor != null)
-				for (int i=0; i<mCatalystDepictor.length; i++)
-					mCatalystDepictor[i].applyTransformation(t);
+				for (GenericDepictor d:mCatalystDepictor)
+					d.applyTransformation(t);
 
 			return t;
             }
@@ -378,12 +405,12 @@ g.drawRect((int)r.x, (int)r.y, (int)r.width, (int)r.height);*/
                     }
                 }
             }
-        return (bondCount == 0) ? AbstractDepictor.cOptAvBondLen
+        return (bondCount == 0) ? mDefaultAVBL
                                 : mTransformation.getScaling() * averageBondLength / bondCount;
         }
 
     private void layoutReaction(GenericDrawContext context) {
-        Rectangle2D.Double[] boundingRect = new Rectangle2D.Double[mMolecule.length];
+        GenericRectangle[] boundingRect = new GenericRectangle[mMolecule.length];
         double totalWidth = 0.0;
         double totalHeight = 0.0;
         for (int i=0; i<mMolecule.length; i++) {
@@ -395,7 +422,7 @@ g.drawRect((int)r.x, (int)r.y, (int)r.width, (int)r.height);*/
 
         final double catalystScale = 0.7;
 		double catalystSpacing = 0.5 * AbstractDepictor.cOptAvBondLen;
-		Rectangle2D.Double[] catalystBoundingRect = new Rectangle2D.Double[mCatalyst.length];
+	    GenericRectangle[] catalystBoundingRect = new GenericRectangle[mCatalyst.length];
 		double totalCatalystWidth = 0.0;
 		double totalCatalystHeight = 0.0;
 		for (int i=0; i<mCatalyst.length; i++) {
