@@ -66,7 +66,9 @@
 package com.actelion.research.chem;
 
 import com.actelion.research.calc.ArrayUtilsCalc;
+import com.actelion.research.chem.descriptor.DescriptorEncoder;
 import com.actelion.research.chem.descriptor.DescriptorHandler;
+import com.actelion.research.util.BurtleHasher;
 
 import java.util.*;
 
@@ -91,9 +93,6 @@ public class ExtendedMoleculeFunctions {
 	public static final String [] arrRGroupsSymbol = {"R1","R2","R3","R4","R5","R6","R7","R8","R9","R10","R11","R12","R13","R14","R15","R16"};
 
 
-
-
-
 	public static void makeSkeleton(StereoMolecule mol) {
 		for (int bond=0; bond<mol.getAllBonds(); bond++)
 			mol.setBondType(bond, Molecule.cBondTypeSingle);
@@ -236,6 +235,31 @@ public class ExtendedMoleculeFunctions {
 		return hetero;
 	}
 
+	public static int getNumBondsNoHydrogen(ExtendedMolecule mol) {
+		int bnds = 0;
+		for (int i = 0; i < mol.getBonds(); i++) {
+			int at1 = mol.getBondAtom(0,i);
+			int atNo1 = mol.getAtomicNo(at1);
+			int at2 = mol.getBondAtom(1,i);
+			int atNo2 = mol.getAtomicNo(at2);
+			if(atNo1!=1 && atNo2!=1) {
+				bnds++;
+			}
+		}
+		return bnds;
+	}
+	public static int getNumNonHydrogenAtoms(ExtendedMolecule mol) {
+		int non = 0;
+
+		for (int i = 0; i < mol.getAtoms(); i++) {
+			int atomicNo = mol.getAtomicNo(i);
+			if(atomicNo!=1) {
+				non++;
+			}
+		}
+
+		return non;
+	}
 	public static int getNumCarbonAtoms(ExtendedMolecule mol) {
 		int carbon = 0;
 
@@ -760,9 +784,7 @@ public class ExtendedMoleculeFunctions {
 	}
 
 	public final static int [][] getTopologicalDistanceMatrix(StereoMolecule mol) {
-
 		return getNumberOfBondsBetweenAtoms(mol, mol.getBonds(), null);
-
 	}
 
 	/**
@@ -1018,6 +1040,23 @@ public class ExtendedMoleculeFunctions {
 		return ring;
 	}
 
+	public static boolean containsFiveBindingCarbon(StereoMolecule mol) {
+
+		boolean five=false;
+		for (int i = 0; i < mol.getAtoms(); i++) {
+			if(mol.getAtomicNo(i)==6){
+				int sumBO=0;
+				for (int j = 0; j < mol.getConnAtoms(i); j++) {
+					int bo = mol.getConnBondOrder(i,j);
+					sumBO+=bo;
+				}
+				five=(sumBO>4)?true:false;
+			}
+		}
+
+		return five;
+	}
+
 	/**
 	 *
 	 * @param mol
@@ -1673,6 +1712,56 @@ public class ExtendedMoleculeFunctions {
 		return dh.getSimilarity(d1, d2);
 	}
 
+	/**
+	 * Taken from Thomas Sander SkeletonSpheres descriptor
+	 * @param mol
+	 * @param rootAtom atom index to start
+	 * @param depth so many spheres are taken
+	 * @return Fragment containing the spheres started at rootAtom
+	 */
+	public static StereoMolecule getSphere(StereoMolecule mol, int rootAtom, int depth){
+
+		mol.ensureHelperArrays(Molecule.cHelperRings);
+		StereoMolecule fragment = new StereoMolecule(mol.getAtoms(), mol.getBonds());
+
+		int[] atomList = new int[mol.getAtoms()];
+		boolean[] atomMask = new boolean[mol.getAtoms()];
+		if (rootAtom != 0)
+			Arrays.fill(atomMask, false);
+
+		int min = 0;
+		int max = 0;
+
+		for (int sphere=0; sphere<depth && max<mol.getAtoms(); sphere++) {
+			if (max == 0) {
+				atomList[0] = rootAtom;
+				atomMask[rootAtom] = true;
+				max = 1;
+			}
+			else {
+				int newMax = max;
+				for (int i=min; i<max; i++) {
+					int atom = atomList[i];
+					for (int j=0; j<mol.getConnAtoms(atom); j++) {
+						int connAtom = mol.getConnAtom(atom, j);
+						if (!atomMask[connAtom]) {
+							atomMask[connAtom] = true;
+							atomList[newMax++] = connAtom;
+						}
+					}
+				}
+				min = max;
+				max = newMax;
+			}
+
+			mol.copyMoleculeByAtoms(fragment, atomMask, true, null);
+		}
+
+		return fragment;
+
+	}
+
+
 	/**
 	 * Starts with 1 and goes until 16
 	 * @return
