@@ -84,6 +84,13 @@ public class IDCodeParserWithoutCoordinateInvention {
 	 * @return
 	 */
 	public StereoMolecule getCompactMolecule(byte[] idcode) {
+		if (idcode == null || idcode.length == 0)
+			return null;
+
+		for (int i=2; i<idcode.length-2; i++)
+			if (idcode[i] == ' ')
+				return getCompactMolecule(idcode, idcode, 0, i+1);
+
 		return getCompactMolecule(idcode, null);
 		}
 
@@ -211,13 +218,13 @@ public class IDCodeParserWithoutCoordinateInvention {
 	public void parse(StereoMolecule mol, byte[] idcode, byte[] coordinates, int idcodeStart, int coordsStart) {
 		mol.clear();
 
-		if (idcode==null || idcodeStart >= idcode.length)
+		if (idcode==null || idcodeStart < 0 || idcodeStart >= idcode.length)
 			return;
 
 		mMol = mol;
 		int version = Canonizer.cIDCodeVersion2;
 
-		if (coordinates != null && coordsStart >= coordinates.length)
+		if (coordinates != null && (coordsStart < 0 || coordsStart >= coordinates.length))
 			coordinates = null;
 
 		decodeBitsStart(idcode, idcodeStart);
@@ -261,7 +268,7 @@ public class IDCodeParserWithoutCoordinateInvention {
 		mMol.setAtomY(0, 0.0);
 		mMol.setAtomZ(0, 0.0);
 
-		boolean decodeOldCoordinates = (coordinates != null && coordinates[0] >= '\'');
+		boolean decodeOldCoordinates = (coordinates != null && coordinates[coordsStart] >= '\'');
 		double targetAVBL = 0.0;
 		double xOffset = 0.0;
 		double yOffset = 0.0;
@@ -466,7 +473,7 @@ public class IDCodeParserWithoutCoordinateInvention {
 				no = decodeBits(abits);
 				for (int i=0; i<no; i++) {
 					int atom = decodeBits(abits);
-					int ringState = decodeBits(Molecule.cAtomQFRingStateBits) << Molecule.cAtomQFRingStateShift;
+					long ringState = (long)decodeBits(Molecule.cAtomQFRingStateBits) << Molecule.cAtomQFRingStateShift;
 					mMol.setAtomQueryFeature(atom, ringState, true);
 					}
 				break;
@@ -474,7 +481,7 @@ public class IDCodeParserWithoutCoordinateInvention {
 				no = decodeBits(abits);
 				for (int i=0; i<no; i++) {
 					int atom = decodeBits(abits);
-					int aromState = decodeBits(Molecule.cAtomQFAromStateBits) << Molecule.cAtomQFAromStateShift;
+					long aromState = (long)decodeBits(Molecule.cAtomQFAromStateBits) << Molecule.cAtomQFAromStateShift;
 					mMol.setAtomQueryFeature(atom, aromState, true);
 					}
 				break;
@@ -489,7 +496,7 @@ public class IDCodeParserWithoutCoordinateInvention {
 				no = decodeBits(abits);
 				for (int i=0; i<no; i++) {
 					int atom = decodeBits(abits);
-					int hydrogen = decodeBits(Molecule.cAtomQFHydrogenBits) << Molecule.cAtomQFHydrogenShift;
+					long hydrogen = (long)decodeBits(Molecule.cAtomQFHydrogenBits) << Molecule.cAtomQFHydrogenShift;
 					mMol.setAtomQueryFeature(atom, hydrogen, true);
 					}
 				break;
@@ -541,7 +548,7 @@ public class IDCodeParserWithoutCoordinateInvention {
 				no = decodeBits(abits);
 				for (int i=0; i<no; i++) {
 					int atom = decodeBits(abits);
-					int piElectrons = decodeBits(Molecule.cAtomQFPiElectronBits) << Molecule.cAtomQFPiElectronShift;
+					long piElectrons = (long)decodeBits(Molecule.cAtomQFPiElectronBits) << Molecule.cAtomQFPiElectronShift;
 					mMol.setAtomQueryFeature(atom, piElectrons, true);
 					}
 				break;
@@ -549,18 +556,19 @@ public class IDCodeParserWithoutCoordinateInvention {
 				no = decodeBits(abits);
 				for (int i=0; i<no; i++) {
 					int atom = decodeBits(abits);
-					int neighbours = decodeBits(Molecule.cAtomQFNeighbourBits) << Molecule.cAtomQFNeighbourShift;
+					long neighbours = (long)decodeBits(Molecule.cAtomQFNeighbourBits) << Molecule.cAtomQFNeighbourShift;
 					mMol.setAtomQueryFeature(atom, neighbours, true);
 					}
 				break;
-			case 15: //  datatype 'start second feature set'
-				offset = 16;
+			case 15: //  datatype 'start next feature set'
+			case 31:
+				offset += 16;
 				break;
-			case 16: //  datatype 'AtomQFRingSize'
+			case 16: //  datatype 'AtomQFSmallRingSize'
 				no = decodeBits(abits);
 				for (int i=0; i<no; i++) {
 					int atom = decodeBits(abits);
-					int ringSize = decodeBits(Molecule.cAtomQFRingSizeBits) << Molecule.cAtomQFRingSizeShift;
+					long ringSize = (long)decodeBits(Molecule.cAtomQFSmallRingSizeBits) << Molecule.cAtomQFSmallRingSizeShift;
 					mMol.setAtomQueryFeature(atom, ringSize, true);
 					}
 				break;
@@ -587,7 +595,7 @@ public class IDCodeParserWithoutCoordinateInvention {
 				no = decodeBits(abits);
 				for (int i=0; i<no; i++) {
 					int atom = decodeBits(abits);
-					int charge = decodeBits(Molecule.cAtomQFChargeBits) << Molecule.cAtomQFChargeShift;
+					long charge = (long)decodeBits(Molecule.cAtomQFChargeBits) << Molecule.cAtomQFChargeShift;
 					mMol.setAtomQueryFeature(atom, charge, true);
 					}
 				break;
@@ -655,10 +663,64 @@ public class IDCodeParserWithoutCoordinateInvention {
 				no = decodeBits(abits);
 				for (int i=0; i<no; i++) {
 					int atom = decodeBits(abits);
-					int hint = decodeBits(Molecule.cAtomQFRxnParityBits) << Molecule.cAtomQFRxnParityShift;
+					long hint = (long)decodeBits(Molecule.cAtomQFRxnParityBits) << Molecule.cAtomQFRxnParityShift;
 					mMol.setAtomQueryFeature(atom, hint, true);
 					}
 				break;
+			case 30: //  datatype 'AtomQFNewRingSize'
+				no = decodeBits(abits);
+				for (int i=0; i<no; i++) {
+					int atom = decodeBits(abits);
+					long ringSize = (long)decodeBits(Molecule.cAtomQFNewRingSizeBits) << Molecule.cAtomQFNewRingSizeShift;
+					mMol.setAtomQueryFeature(atom, ringSize, true);
+					}
+				break;
+			case 32: //  datatype 'AtomQFNewRingSize'
+				no = decodeBits(abits);
+				for (int i=0; i<no; i++) {
+					int atom = decodeBits(abits);
+					long stereoState = (long)decodeBits(Molecule.cAtomQFStereoStateBits) << Molecule.cAtomQFStereoStateShift;
+					mMol.setAtomQueryFeature(atom, stereoState, true);
+					}
+				break;
+			case 33: //  datatype 'AtomQFENeighbours'
+				no = decodeBits(abits);
+				for (int i=0; i<no; i++) {
+					int atom = decodeBits(abits);
+					long eNeighbours = (long)decodeBits(Molecule.cAtomQFENeighbourBits) << Molecule.cAtomQFENeighbourShift;
+					mMol.setAtomQueryFeature(atom, eNeighbours, true);
+					}
+				break;
+			case 34:	//	datatype 'AtomQFHetereoAromatic'
+				no = decodeBits(abits);
+				for (int i=0; i<no; i++) {
+					int atom = decodeBits(abits);
+					mMol.setAtomQueryFeature(atom, Molecule.cAtomQFHeteroAromatic, true);
+					}
+				break;
+			case 35:	//	datatype 'BondQFMatchFormalOrder'
+				no = decodeBits(bbits);
+				for (int i=0; i<no; i++) {
+					int bond = decodeBits(bbits);
+					mMol.setBondQueryFeature(bond, Molecule.cBondQFMatchFormalOrder, true);
+					}
+				break;
+			case 36:	//	datatype 'cBondQFRareBondType'
+				no = decodeBits(bbits);
+				for (int i=0; i<no; i++) {
+					int bond = decodeBits(bbits);
+					int bondType = decodeBits(Molecule.cBondQFRareBondTypesBits) << Molecule.cBondQFRareBondTypesShift;
+					mMol.setBondQueryFeature(bond, bondType, true);
+					}
+				break;
+			case 37:	//	datatype 'rare order bond'
+				no = decodeBits(bbits);
+				for (int i=0; i<no; i++) {
+					int bond = decodeBits(bbits);
+					int bondType = decodeBits(1) == 0 ? Molecule.cBondTypeQuadruple : Molecule.cBondTypeQuintuple;
+					mMol.setBondType(bond, bondType);
+					}
+				break;
 				}
 			}
 
@@ -794,7 +856,7 @@ public class IDCodeParserWithoutCoordinateInvention {
 		// - have distinguishable substituents on both ends, i.e. is a stereo double bond
 		// - are not in a small ring
 		// Here we don't know, whether a double bond without E/Z parity is a stereo bond with unknown
-		// configuration or not a stereo bond. Therefore we need to set a flag, that causes the Canonizer
+		// configuration or not a stereo bond. Therefore, we need to set a flag, that causes the Canonizer
 		// during the next stereo recognition with atom coordinates to assign an unknown configuration rather
 		// than E or Z based on created or given coordinates.
 		// In a next step these double bonds are converted into cross bonds by
@@ -807,8 +869,9 @@ public class IDCodeParserWithoutCoordinateInvention {
 					mMol.setBondParityUnknownOrNone(bond);
 			}
 
+		mMol.setParitiesValid(0);
+
 		if (!coords2DAvailable && ensure2DCoordinates()) {
-			mMol.setParitiesValid(0);
 			try {
 				inventCoordinates(mMol);
 				coords2DAvailable = true;
@@ -1417,14 +1480,15 @@ public class IDCodeParserWithoutCoordinateInvention {
 						System.out.println();
 						break;
 					case 15: //  datatype 'start second feature set'
-						offset = 16;
-						System.out.println("<start second feature set>");
+					case 31:
+						offset += 16;
+						System.out.println("<start next feature set>");
 						break;
-					case 16: //  datatype 'AtomQFRingSize'
+					case 16: //  datatype 'AtomQFSmallRingSize'
 						no = decodeBits(abits);
-						System.out.print("AtomQFRingSize:");
+						System.out.print("AtomQFSmallRingSize:");
 						for (int i = 0; i < no; i++)
-							System.out.print(" " + decodeBits(abits) + ":" + decodeBits(Molecule.cAtomQFRingSizeBits));
+							System.out.print(" " + decodeBits(abits) + ":" + decodeBits(Molecule.cAtomQFSmallRingSizeBits));
 						System.out.println();
 						break;
 					case 17: //  datatype 'AtomAbnormalValence'
@@ -1473,14 +1537,14 @@ public class IDCodeParserWithoutCoordinateInvention {
 						no = decodeBits(abits);
 						System.out.print("AtomQFFlatNitrogen:");
 						for (int i = 0; i < no; i++)
-							System.out.print(" " + decodeBits(abits) + ":true");
+							System.out.print(" " + decodeBits(abits));
 						System.out.println();
 						break;
 					case 23:    //	datatype 'cBondQFMatchStereo'
 						no = decodeBits(bbits);
 						System.out.print("cBondQFMatchStereo:");
 						for (int i = 0; i < no; i++)
-							System.out.print(" " + decodeBits(abits) + ":true");
+							System.out.print(" " + decodeBits(abits));
 						System.out.println();
 						break;
 					case 24:    //	datatype 'cBondQFAromatic'
@@ -1507,7 +1571,7 @@ public class IDCodeParserWithoutCoordinateInvention {
 						no = decodeBits(abits);
 						System.out.print("AtomQFExcludeGroup:");
 						for (int i = 0; i < no; i++)
-							System.out.print(" " + decodeBits(abits) + ":true");
+							System.out.print(" " + decodeBits(abits));
 						System.out.println();
 						break;
 					case 28:    //	datatype 'coordinate bond'
@@ -1524,8 +1588,56 @@ public class IDCodeParserWithoutCoordinateInvention {
 							System.out.print(" " + decodeBits(abits) + ":" + decodeBits(Molecule.cAtomQFRxnParityBits));
 						System.out.println();
 						break;
+					case 30: //  datatype 'AtomQFNewRingSize'
+						no = decodeBits(abits);
+						System.out.print("AtomQFNewRingSize:");
+						for (int i = 0; i < no; i++)
+							System.out.print(" " + decodeBits(abits) + ":" + decodeBits(Molecule.cAtomQFNewRingSizeBits));
+						System.out.println();
+						break;
+					case 32: //  datatype 'AtomQFStereoStateBits'
+						no = decodeBits(abits);
+						System.out.print("AtomQFStereoState:");
+						for (int i = 0; i < no; i++)
+							System.out.print(" " + decodeBits(abits) + ":" + decodeBits(Molecule.cAtomQFStereoStateBits));
+						System.out.println();
+						break;
+					case 33: //  datatype 'AtomQFENeighbours'
+						no = decodeBits(abits);
+						System.out.print("AtomQFENeighbours:");
+						for (int i = 0; i < no; i++)
+							System.out.print(" " + decodeBits(abits) + ":" + decodeBits(Molecule.cAtomQFENeighbourBits));
+						System.out.println();
+						break;
+					case 34:    //	datatype 'in hetero aromatic ring'
+						no = decodeBits(abits);
+						System.out.print("AtomQFHeteroAromatic:");
+						for (int i = 0; i < no; i++)
+							System.out.print(" " + decodeBits(abits));
+						System.out.println();
+						break;
+					case 35:    //	datatype 'cBondQFMatchFormalOrder'
+						no = decodeBits(bbits);
+						System.out.print("BondQFMatchFormalOrder:");
+						for (int i = 0; i < no; i++)
+							System.out.print(" " + decodeBits(abits));
+						System.out.println();
+						break;
+					case 36:    //	datatype 'cBondQFRareBondType'
+						no = decodeBits(bbits);
+						System.out.print("BondQFRareBondType:");
+						for (int i = 0; i < no; i++)
+							System.out.print(" " + decodeBits(bbits) + ":" + decodeBits(Molecule.cBondQFRareBondTypesBits));
+						System.out.println();
+						break;
+					case 37:    // datatype 'rare bond type'
+						no = decodeBits(bbits);
+						System.out.print("Rare Bond Type:");
+						for (int i=0; i<no; i++)
+							System.out.print(" " + decodeBits(bbits) + ":" + (decodeBits(1) == 0 ? "quadruple" : "quintuple"));
+						break;
+					}
 				}
-			}
 
 			if (coordinates != null) {
 				if (coordinates[0] == '!' || coordinates[0] == '#') {    // new coordinate format
