@@ -562,7 +562,7 @@ public class IsomericSmilesCreator {
 		if (charge == 0 && (mMode & MODE_CREATE_SMARTS) != 0) {
 			// Because SMARTS don't know a charge query feature, we set the atom charge in case of neg/pos charge required.
 			// There is no way to require an uncharged atom (Molecule.cAtomQFNotChargeNeg | Molecule.cAtomQFNotChargePos).
-			int chargeFeatures = mMol.getAtomQueryFeatures(atom) & Molecule.cAtomQFCharge;
+			long chargeFeatures = mMol.getAtomQueryFeatures(atom) & Molecule.cAtomQFCharge;
 			if (chargeFeatures == (Molecule.cAtomQFNotCharge0 | Molecule.cAtomQFNotChargePos))
 				charge = -1;    // 'require negative charge' is translated into charge := -1
 			else if (chargeFeatures == (Molecule.cAtomQFNotCharge0 | Molecule.cAtomQFNotChargeNeg))
@@ -645,31 +645,31 @@ public class IsomericSmilesCreator {
 	private String getAtomSMARTSFeatures(int atom, StringBuilder buffer) {
 		buffer.setLength(0);
 
-		int queryFeatures = mMol.getAtomQueryFeatures(atom);
+		long queryFeatures = mMol.getAtomQueryFeatures(atom);
 
 		// SMARTS don't distinguish between a charged atom and atom charge query features
-		int chargeFeatures = queryFeatures & Molecule.cAtomQFCharge;
+		int chargeFeatures = (int)((queryFeatures & Molecule.cAtomQFCharge) >> Molecule.cAtomQFChargeBits);
 		switch (chargeFeatures) {
-			case Molecule.cAtomQFNotChargeNeg | Molecule.cAtomQFNotChargePos:
+			case (int)((Molecule.cAtomQFNotChargeNeg | Molecule.cAtomQFNotChargePos) >> Molecule.cAtomQFChargeBits):
 				buffer.append("+0");   // 'require negative charge' is translated into charge := -1
 				break;
-			case Molecule.cAtomQFNotCharge0 | Molecule.cAtomQFNotChargePos:
+			case (int)((Molecule.cAtomQFNotCharge0 | Molecule.cAtomQFNotChargePos) >> Molecule.cAtomQFChargeBits):
 				if (mMol.getAtomCharge(atom) == 0)
 					buffer.append("-");   // 'require negative charge' is translated into charge := -1
 				break;
-			case Molecule.cAtomQFNotCharge0 | Molecule.cAtomQFNotChargeNeg:
+			case (int)((Molecule.cAtomQFNotCharge0 | Molecule.cAtomQFNotChargeNeg) >> Molecule.cAtomQFChargeBits):
 				if (mMol.getAtomCharge(atom) == 0)
 					buffer.append("+");   // 'require positive charge' is translated into charge := +1
 				break;
 			}
 
-		int aromState = queryFeatures & Molecule.cAtomQFAromState;
+		long aromState = queryFeatures & Molecule.cAtomQFAromState;
 		if (aromState == Molecule.cAtomQFAromatic)
 			buffer.append(";a");
 		else if (aromState == Molecule.cAtomQFNotAromatic)
 			buffer.append(";A");
 
-		int hydrogenQueryFeatures = queryFeatures & Molecule.cAtomQFHydrogen;
+		long hydrogenQueryFeatures = queryFeatures & Molecule.cAtomQFHydrogen;
 		if (hydrogenQueryFeatures != 0) {
 			if (hydrogenQueryFeatures == (Molecule.cAtomQFNot1Hydrogen | Molecule.cAtomQFNot2Hydrogen | Molecule.cAtomQFNot3Hydrogen))
 				buffer.append(";H0");
@@ -691,66 +691,86 @@ public class IsomericSmilesCreator {
 
 		// Atom membership of SSSR rings cannot be directly translated into number of ring bonds.
 		// We try to get close...
-		int ringState = queryFeatures & Molecule.cAtomQFRingState;
-		switch (ringState) {
-			case Molecule.cAtomQFNotChain:
-				buffer.append(";!R0");
-				break;
-			case Molecule.cAtomQFNot2RingBonds:
-				buffer.append(";!R1");
-				break;
-			case Molecule.cAtomQFNot3RingBonds:
-				buffer.append(";!R2");
-				break;
-			case Molecule.cAtomQFNot4RingBonds:
-				buffer.append(";!R3");
-				break;
-			case Molecule.cAtomQFNot2RingBonds | Molecule.cAtomQFNot3RingBonds | Molecule.cAtomQFNot4RingBonds:
-				buffer.append(";R0");
-				break;
-			case Molecule.cAtomQFNotChain | Molecule.cAtomQFNot3RingBonds | Molecule.cAtomQFNot4RingBonds:
-				buffer.append(";R1");
-				break;
-			case Molecule.cAtomQFNotChain | Molecule.cAtomQFNot2RingBonds | Molecule.cAtomQFNot4RingBonds:
-				buffer.append(";R2");
-				break;
-			case Molecule.cAtomQFNotChain | Molecule.cAtomQFNot2RingBonds | Molecule.cAtomQFNot3RingBonds:
-				buffer.append(";R3");
-				break;
+		long ringState = queryFeatures & Molecule.cAtomQFRingState;
+		if (ringState == Molecule.cAtomQFNotChain)
+			buffer.append(";!R0");
+		else if (ringState == Molecule.cAtomQFNot2RingBonds)
+			buffer.append(";!R1");
+		else if (ringState == Molecule.cAtomQFNot3RingBonds)
+			buffer.append(";!R2");
+		else if (ringState == Molecule.cAtomQFNot4RingBonds)
+			buffer.append(";!R3");
+		else if (ringState == (Molecule.cAtomQFNot2RingBonds | Molecule.cAtomQFNot3RingBonds | Molecule.cAtomQFNot4RingBonds))
+			buffer.append(";R0");
+		else if (ringState == (Molecule.cAtomQFNotChain | Molecule.cAtomQFNot3RingBonds | Molecule.cAtomQFNot4RingBonds))
+			buffer.append(";R1");
+		else if (ringState == (Molecule.cAtomQFNotChain | Molecule.cAtomQFNot2RingBonds | Molecule.cAtomQFNot4RingBonds))
+			buffer.append(";R2");
+		else if (ringState == (Molecule.cAtomQFNotChain | Molecule.cAtomQFNot2RingBonds | Molecule.cAtomQFNot3RingBonds))
+			buffer.append(";R3");
+
+		long ringSize = queryFeatures & Molecule.cAtomQFNewRingSize;
+		if (ringSize == Molecule.cAtomQFRingSize0)
+			buffer.append(";!r" + ringSize);
+		else if (ringSize == (Molecule.cAtomQFNewRingSize & ~Molecule.cAtomQFRingSize0))
+			buffer.append(";r" + ringSize);
+		else if (ringSize != 0) {
+			if ((ringSize & Molecule.cAtomQFRingSizeLarge) != 0) {  // negative logic
+				if ((ringSize & Molecule.cAtomQFRingSize0) == 0)
+					buffer.append(";!r0" + ringSize);
+				if ((ringSize & Molecule.cAtomQFRingSize3) == 0)
+					buffer.append(";!r3" + ringSize);
+				if ((ringSize & Molecule.cAtomQFRingSize4) == 0)
+					buffer.append(";!r4" + ringSize);
+				if ((ringSize & Molecule.cAtomQFRingSize5) == 0)
+					buffer.append(";!r5" + ringSize);
+				if ((ringSize & Molecule.cAtomQFRingSize6) == 0)
+					buffer.append(";!r6" + ringSize);
+				if ((ringSize & Molecule.cAtomQFRingSize7) == 0)
+					buffer.append(";!r7" + ringSize);
+			}
+			else {
+				buffer.append(";");
+				if ((ringSize & Molecule.cAtomQFRingSize0) != 0)
+					buffer.append("r0," + ringSize);
+				if ((ringSize & Molecule.cAtomQFRingSize3) != 0)
+					buffer.append("r3," + ringSize);
+				if ((ringSize & Molecule.cAtomQFRingSize4) != 0)
+					buffer.append("r4," + ringSize);
+				if ((ringSize & Molecule.cAtomQFRingSize5) != 0)
+					buffer.append("r5," + ringSize);
+				if ((ringSize & Molecule.cAtomQFRingSize6) != 0)
+					buffer.append("r6," + ringSize);
+				if ((ringSize & Molecule.cAtomQFRingSize7) != 0)
+					buffer.append("r7," + ringSize);
+				buffer.setLength(buffer.length()-1);
+			}
 		}
 
-		int ringSize = (queryFeatures & Molecule.cAtomQFRingSize) >> Molecule.cAtomQFRingSizeShift;
-		if (ringSize != 0)
-			buffer.append(";r"+ringSize);
-
-		int neighbourFeatures = queryFeatures & Molecule.cAtomQFNeighbours;
-		switch (neighbourFeatures) {
-			case Molecule.cAtomQFNeighbours & ~Molecule.cAtomQFNot1Neighbour:
-				buffer.append(";D1");   // exactly 1
-				break;
-			case Molecule.cAtomQFNeighbours & ~Molecule.cAtomQFNot2Neighbours:
-				buffer.append(";D2");   // exactly 2
-				break;
-			case Molecule.cAtomQFNeighbours & ~Molecule.cAtomQFNot3Neighbours:
-				buffer.append(";D3");   // exactly 3
-				break;
-			case Molecule.cAtomQFNot3Neighbours | Molecule.cAtomQFNot4Neighbours:
-				buffer.append(";!D3;!D4");   // less than 3
-				break;
-			case Molecule.cAtomQFNot4Neighbours:
-				buffer.append(";!D4");   // less than 4
-				break;
-			case Molecule.cAtomQFNot0Neighbours | Molecule.cAtomQFNot1Neighbour:
-				buffer.append(";!D0;!D1");   // more than 1
-				break;
-			case Molecule.cAtomQFNot0Neighbours | Molecule.cAtomQFNot1Neighbour | Molecule.cAtomQFNot2Neighbours:
-				buffer.append(";!D0;!D1;!D2");   // more than 2
-				break;
-			case Molecule.cAtomQFNeighbours & ~Molecule.cAtomQFNot4Neighbours:
-				buffer.append(";!D0;!D1;!D2;!D3");   // more than 3
-				break;
+		if (ringSize == 0) {
+			ringSize = (queryFeatures & Molecule.cAtomQFSmallRingSize) >> Molecule.cAtomQFSmallRingSizeShift;
+			if (ringSize != 0)
+				buffer.append(";r" + ringSize);
 		}
 
+		long neighbourFeatures = queryFeatures & Molecule.cAtomQFNeighbours;
+		if (neighbourFeatures == (Molecule.cAtomQFNeighbours & ~Molecule.cAtomQFNot1Neighbour))
+			buffer.append(";D1");   // exactly 1
+		if (neighbourFeatures == (Molecule.cAtomQFNeighbours & ~Molecule.cAtomQFNot2Neighbours))
+			buffer.append(";D2");   // exactly 2
+		if (neighbourFeatures == (Molecule.cAtomQFNeighbours & ~Molecule.cAtomQFNot3Neighbours))
+			buffer.append(";D3");   // exactly 3
+		if (neighbourFeatures == (Molecule.cAtomQFNot3Neighbours | Molecule.cAtomQFNot4Neighbours))
+			buffer.append(";!D3;!D4");   // less than 3
+		if (neighbourFeatures == Molecule.cAtomQFNot4Neighbours)
+			buffer.append(";!D4");   // less than 4
+		if (neighbourFeatures == (Molecule.cAtomQFNot0Neighbours | Molecule.cAtomQFNot1Neighbour))
+			buffer.append(";!D0;!D1");   // more than 1
+		if (neighbourFeatures == (Molecule.cAtomQFNot0Neighbours | Molecule.cAtomQFNot1Neighbour | Molecule.cAtomQFNot2Neighbours))
+			buffer.append(";!D0;!D1;!D2");   // more than 2
+		if (neighbourFeatures == (Molecule.cAtomQFNeighbours & ~Molecule.cAtomQFNot4Neighbours))
+			buffer.append(";!D0;!D1;!D2;!D3");   // more than 3
+
 		if ((queryFeatures & Molecule.cAtomQFNoMoreNeighbours) != 0)
 			buffer.append(";D"+mMol.getConnAtoms(atom));   // Convert into exact explicit neighbour count 'D'
 
@@ -800,7 +820,7 @@ public class IsomericSmilesCreator {
 		if (mEZHalfParity[bond] != 0)
 			builder.append(mEZHalfParity[bond] == 1 ? '/' : '\\');
 		if (mMode == MODE_CREATE_SMARTS) {
-			int bondTypes = mMol.getBondQueryFeatures(Molecule.cBondQFBondTypes);
+			int bondTypes = mMol.getBondQueryFeatures(Molecule.cBondQFBondTypes | Molecule.cBondQFRareBondTypes);
 			if (bondTypes != 0) {
 				if ((bondTypes & Molecule.cBondTypeSingle) != 0 && mEZHalfParity[bond] == 0) {
 					builder.append('-');
@@ -815,6 +835,16 @@ public class IsomericSmilesCreator {
 						builder.append(',');
 					builder.append('#');
 					}
+				if ((bondTypes & Molecule.cBondTypeQuadruple) != 0) {
+					if (builder.length() != startLength)
+						builder.append(',');
+					builder.append('$');
+					}
+				if ((bondTypes & Molecule.cBondTypeQuintuple) != 0) {   // SMILES doesn't support quintuple bonds, thus we use quadruple
+					if (builder.length() != startLength)
+						builder.append(',');
+					builder.append('$');
+					}
 				if ((bondTypes & Molecule.cBondTypeDelocalized) != 0) {
 					if (builder.length() != startLength)
 						builder.append(',');
@@ -839,8 +869,10 @@ public class IsomericSmilesCreator {
 				builder.append('=');
 			else if (bondType == Molecule.cBondTypeTriple)
 				builder.append('#');
-//				else if (order == Molecule.cBondTypeQuadruple)	// not supported by Molecule
-//					builder.append('$');
+			else if (bondType == Molecule.cBondTypeQuadruple)
+				builder.append('$');
+			else if (bondType == Molecule.cBondTypeQuintuple)	// not supported by SMILES, we use quadruple instead
+				builder.append('$');
 			else if (bondType == Molecule.cBondTypeDelocalized)
 				builder.append(':');
 			else if (bondType == Molecule.cBondTypeMetalLigand)
