@@ -1,17 +1,33 @@
 /*
- * Copyright 2017 Idorsia Pharmaceuticals Ltd., Hegenheimermattweg 91, CH-4123 Allschwil, Switzerland
+ * Copyright (c) 1997 - 2016
+ * Actelion Pharmaceuticals Ltd.
+ * Gewerbestrasse 16
+ * CH-4123 Allschwil, Switzerland
  *
- * This file is part of DataWarrior.
- * 
- * DataWarrior is free software: you can redistribute it and/or modify it under the terms of the
- * GNU General Public License as published by the Free Software Foundation, either version 3 of
- * the License, or (at your option) any later version.
- * 
- * DataWarrior is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
- * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU General Public License for more details.
- * You should have received a copy of the GNU General Public License along with DataWarrior.
- * If not, see http://www.gnu.org/licenses/.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright notice, this
+ *    list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright notice,
+ *    this list of conditions and the following disclaimer in the documentation
+ *    and/or other materials provided with the distribution.
+ * 3. Neither the name of the the copyright holder nor the
+ *    names of its contributors may be used to endorse or promote products
+ *    derived from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+ * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
+ * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
+ * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  * @author Thomas Sander
  */
@@ -1521,20 +1537,20 @@ public class Mutator {
 		}
 
 	private void repairStereoChemistry(StereoMolecule mol) {
+		for (int bond=0; bond<mol.getAllBonds(); bond++)
+			if (mol.isStereoBond(bond))
+				mol.setBondType(bond, Molecule.cBondTypeSingle);
+
         for (int atom=0; atom<mol.getAtoms(); atom++) {
-            switch (mol.getAtomParity(atom)) {
-            case Molecule.cAtomParityUnknown:
-                int parity = (mRandom.nextDouble() < 0.5) ? Molecule.cAtomParity1 : Molecule.cAtomParity2;
-                boolean isPseudo = mol.isAtomParityPseudo(atom);
-                mol.setAtomParity(atom, parity, isPseudo);
-            case Molecule.cAtomParity1:
-            case Molecule.cAtomParity2:
+        	int parity = mol.getAtomParity(atom);
+        	if (parity == Molecule.cAtomParityUnknown) {
+		        parity = (mRandom.nextDouble()<0.5) ? Molecule.cAtomParity1 : Molecule.cAtomParity2;
+		        boolean isPseudo = mol.isAtomParityPseudo(atom);
+		        mol.setAtomParity(atom, parity, isPseudo);
+		        mol.setAtomESR(atom, Molecule.cESRTypeAbs, 0);
+	            }
+        	if (parity != Molecule.cAtomParityNone)
                 mol.setStereoBondFromAtomParity(atom);
-                break;
-            case Molecule.cAtomParityNone:
-                mol.convertStereoBondsToSingleBonds(atom);
-                break;
-                }
             }
         for (int bond=0; bond<mol.getBonds(); bond++) {
         	if (mol.isBINAPChiralityBond(bond)) {
@@ -1547,10 +1563,6 @@ public class Mutator {
 	            case Molecule.cBondParityZor2:
 	                mol.setStereoBondFromBondParity(bond);
 	                break;
-	            case Molecule.cBondParityNone:
-	                mol.convertStereoBondsToSingleBonds(mol.getBondAtom(0, bond));
-	                mol.convertStereoBondsToSingleBonds(mol.getBondAtom(1, bond));
-	                break;
 	                }
         		}
             }
