@@ -36,7 +36,9 @@ package com.actelion.research.chem;
 
 import com.actelion.research.util.IntArrayComparator;
 
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.TreeSet;
 
 public class SSSearcher {
 	// CONSTANTS TO DEFINE KIND OF SIMILARITY BETWEEN ATOMS AND BONDS
@@ -54,7 +56,7 @@ public class SSSearcher {
 	index creation). Example: key C=C-N-C=C, query pyrol, molecule indole, key would match
 	pyrol but not indol!!!
   - match modes used for the actual atom by atom check must be more or equally restrictive
-	than the match mode used for index creation. Otherwise index keys may filter out
+	than the match mode used for index creation. Otherwise, index keys may filter out
 	molecules which would be considered a match with the less strict matching conditions
 	of the atom by atom check.
 */
@@ -63,7 +65,7 @@ public class SSSearcher {
 	public static final int cIndexMatchMode = cMatchDBondToDelocalized;
 	public static final int cDefaultMatchMode = cMatchAromDBondToDelocalized;
 
-	public final static int cCountModeExistance		= 1;    // check only, don't create matchList
+	public final static int cCountModeExistence     = 1;    // check only, don't create matchList
 	public final static int cCountModeFirstMatch	= 2;    // create matchList with just one match
 	public final static int cCountModeSeparated		= 3;    // create list of all non-overlapping matches / not optimized for maximum match count
 	public final static int cCountModeOverlapping	= 4;    // create list not containing multiple matches sharing exactly the same atoms
@@ -85,8 +87,10 @@ public class SSSearcher {
 
 	private int[] mMoleculeAtomType;	// atom features required to match
 	private int[] mFragmentAtomType;
-	private int[] mMoleculeAtomFeatures;	// flags defining given/required atom features
-	private int[] mFragmentAtomFeatures;
+	private long[] mMoleculeAtomFeatures;	// flags defining given/required atom features
+	private long[] mFragmentAtomFeatures;
+	private long[] mMoleculeRingFeatures;	// flags defining given/required atom ring size features
+	private long[] mFragmentRingFeatures;
 	private int[] mMoleculeBondFeatures;	// flags defining given/required bond features
 	private int[] mFragmentBondFeatures;
 
@@ -98,6 +102,7 @@ public class SSSearcher {
 	private int[] mFragmentGraphParentBond;
 	private boolean[] mFragmentGraphIsRingClosure;
 	private boolean[] mIsExcludeAtom;
+	private boolean[] mIsBridgeBondAtom;
 	private int[] mFragmentConnAtoms;	// in case of exclude atoms, these are not part of this
 	private int[] mMatchTable;
 	private int[] mExcludeGroupNo;
@@ -109,6 +114,7 @@ public class SSSearcher {
 	private TreeSet<int[]> mSortedMatchSet;
 	private ArrayList<int[]> mMatchList;
 	private ArrayList<BridgeBond> mBridgeBondList;
+	private ArrayList<boolean[]> mBridgeBondAtomList;
 
 	private boolean mMoleculeFeaturesValid;
 	private boolean mFragmentFeaturesValid;
@@ -127,8 +133,7 @@ public class SSSearcher {
 	 */
 	public SSSearcher() {
 		mDefaultMatchMode = cDefaultMatchMode;
-		mMatchList = new ArrayList<int[]>();
-		mSortedMatchSet = new TreeSet<int[]>(new IntArrayComparator());
+		mSortedMatchSet = new TreeSet<>(new IntArrayComparator());
 		}
 
 
@@ -143,8 +148,7 @@ public class SSSearcher {
 	 */
 	public SSSearcher(int matchMode) {
 		mDefaultMatchMode = matchMode;
-		mMatchList = new ArrayList<int[]>();
-		mSortedMatchSet = new TreeSet<int[]>(new IntArrayComparator());
+		mSortedMatchSet = new TreeSet<>(new IntArrayComparator());
 		}
 
 
@@ -200,7 +204,7 @@ public class SSSearcher {
 
 		mRequiredHelperLevel = Molecule.cHelperRings;
 		for (int atom=0; atom<mFragment.getAtoms(); atom++)
-			if ((mFragment.getAtomQueryFeatures(atom) & Molecule.cAtomQFMatchStereo) != 0)
+			if ((mFragment.getAtomQueryFeatures(atom) & (Molecule.cAtomQFStereoState | Molecule.cAtomQFMatchStereo)) != 0)
 				mRequiredHelperLevel = Molecule.cHelperParities;
 		for (int bond=0; bond<mFragment.getBonds(); bond++)
 			if ((mFragment.getBondQueryFeatures(bond) & Molecule.cBondQFMatchStereo) != 0)
@@ -379,7 +383,8 @@ System.out.println();
 	 * If the match count mode is one of cCountModeFirstMatch, cCountModeOverlapping,
 	 * cCountModeRigorous then this method returns an arraylist of all counted matches,
 	 * i.e. int arrays mapping fragment atoms to molecule atoms. Atoms being part of a
-	 * matched bridge bond are naturally not covered by the mapping.<br>
+	 * matched bridge bond are naturally not covered by the mapping. Atoms being part of a
+	 * matching bridge bond are available with getBridgeBondAtomList().<br>
 	 * Note: If some query fragment atoms are marked as exclude group, then the respective
 	 * matchlist values are -1.
 	 * @return list of distinct counted matches.
@@ -388,6 +393,16 @@ System.out.println();
 		return mMatchList;
 		}
 
+	/**
+	 * getMatchList() doesn't include information about atoms, which are part of a matching bridge bond.
+	 * This method returns an atom mask for a given matchNo, where all atoms are flagged that are part of a
+	 * matching bridge bond within that match.
+	 * @param matchNo index of corresponding match from getMatchList()
+	 * @return null or atom mask in target atom space
+	 */
+	public boolean[] getMatchingBridgeBondAtoms(int matchNo) {
+		return mBridgeBondAtomList.size() <= matchNo ? null : mBridgeBondAtomList.get(matchNo);
+		}
 
 	/**
 	 * Fastest check, whether the molecule contains the fragment.
@@ -398,7 +413,7 @@ System.out.println();
 	 * @return whether fragment was found as sub-structure in molecule
 	 */
 	public boolean isFragmentInMolecule() {
-		return (findFragmentInMolecule(cCountModeExistance, mDefaultMatchMode) > 0);
+		return (findFragmentInMolecule(cCountModeExistence, mDefaultMatchMode) > 0);
 		}
 
 
@@ -410,7 +425,7 @@ System.out.println();
 	 * @return whether fragment was found as sub-structure in molecule
 	 */
 	public boolean isFragmentInMolecule(int matchMode) {
-		return (findFragmentInMolecule(cCountModeExistance, matchMode) > 0);
+		return (findFragmentInMolecule(cCountModeExistence, matchMode) > 0);
 		}
 
 	/**
@@ -453,7 +468,8 @@ System.out.println();
 	 */
 	public int findFragmentInMolecule(int countMode, int matchMode, final boolean[] atomExcluded) {
 		mStop = false;
-		mMatchList = new ArrayList<int[]>();
+		mMatchList = new ArrayList<>();
+		mBridgeBondAtomList = new ArrayList<>();
 		mSortedMatchSet.clear();
 
 		if (mMolecule == null
@@ -568,7 +584,7 @@ System.out.println();
 							}
 						}
 
-					if (countMode == cCountModeExistance && !isExcludedMatch)
+					if (countMode == cCountModeExistence && !isExcludedMatch)
 						return 1;
 
 					if (!isExcludedMatch) {
@@ -596,13 +612,13 @@ System.out.println();
 		if (countMode == cCountModeFirstMatch
 		 || countMode == cCountModeRigorous) {
 			// count every match (even permutations of same atoms)
-			mMatchList.add(copyOf(mMatchTable, mMatchTable.length));
+			addMatchAtoms();
 			}
 		else if (countMode == cCountModeOverlapping) {
 			int[] sortedMatch = getSortedMatch(copyOf(mMatchTable, mMatchTable.length));
 			if (!mSortedMatchSet.contains(sortedMatch)) {
 				mSortedMatchSet.add(sortedMatch);
-				mMatchList.add(copyOf(mMatchTable, mMatchTable.length));
+				addMatchAtoms();
 				}
 			}
 		else if (countMode == cCountModeSeparated) {
@@ -626,7 +642,7 @@ System.out.println();
 					}
 				if (!found) {
 					mSortedMatchSet.add(sortedMatch);
-					mMatchList.add(copyOf(mMatchTable, mMatchTable.length));
+					addMatchAtoms();
 					}
 				}
 			}
@@ -634,11 +650,16 @@ System.out.println();
 			int[] sortedMatch = getSortedSymmetryMatch(copyOf(mMatchTable, mMatchTable.length));
 			if (!mSortedMatchSet.contains(sortedMatch)) {
 				mSortedMatchSet.add(sortedMatch);
-				mMatchList.add(copyOf(mMatchTable, mMatchTable.length));
+				addMatchAtoms();
 				}
 			}
 		}
 
+	private void addMatchAtoms() {
+		mMatchList.add(copyOf(mMatchTable, mMatchTable.length));
+		if (mBridgeBondList != null)
+			mBridgeBondAtomList.add(copyOf(mIsBridgeBondAtom, mIsBridgeBondAtom.length));
+		}
 
 	/**
 	 * @return sorted match atoms without excluded atoms
@@ -693,8 +714,8 @@ System.out.println();
 		if (fragmentConnAtoms > moleculeConnAtoms)
 			return false;
 
-		int moleculeQF = mMolecule.getAtomQueryFeatures(moleculeAtom);
-		int fragmentQF = mFragment.getAtomQueryFeatures(fragmentAtom);
+		long moleculeQF = mMolecule.getAtomQueryFeatures(moleculeAtom);
+		long fragmentQF = mFragment.getAtomQueryFeatures(fragmentAtom);
 
 		int[] fragmentList = mFragment.getAtomList(fragmentAtom);
 		int[] moleculeList = mMolecule.getAtomList(moleculeAtom);
@@ -761,9 +782,29 @@ System.out.println();
 				}
 			}
 
+		// molecule features must be a subset of the fragment features
 		if ((mMoleculeAtomFeatures[moleculeAtom] & ~mFragmentAtomFeatures[fragmentAtom]) != 0)
 			return false;
 
+		// all ring sizes found for fragment atom must all exist for molecule atom
+		if ((mFragmentRingFeatures[fragmentAtom] & ~mMoleculeRingFeatures[moleculeAtom]) != 0)
+			return false;
+
+		long fragmentRingQF = fragmentQF & Molecule.cAtomQFNewRingSize;
+		if (mMolecule.isFragment()) {
+			// For a fragment in fragment search, the query fragment must not be more restrictive than the target.
+			// Thus, if we have molecule ring features and no restriction on the query or more allowed features
+			// on the query then don't consider the atom a match.
+			long moleculeRingQF = fragmentQF & Molecule.cAtomQFNewRingSize;
+			if (moleculeRingQF != 0 && (fragmentRingQF == 0 || (fragmentRingQF & ~moleculeRingQF) != 0))
+				return false;
+			}
+		else {
+			// at least one of the ring sizes defined in ring query features must match one of the ring sizes found in molecule atom
+			if (fragmentRingQF != 0 && (fragmentRingQF & mMoleculeRingFeatures[moleculeAtom]) == 0)
+				return false;
+			}
+
 		if (mFragment.getAtomCharge(fragmentAtom) != 0
 		 && mFragment.getAtomCharge(fragmentAtom) != mMolecule.getAtomCharge(moleculeAtom))
 			return false;
@@ -773,24 +814,18 @@ System.out.println();
 		if (mFragment.getAtomRadical(fragmentAtom) != 0
 		 && mFragment.getAtomRadical(fragmentAtom) != mMolecule.getAtomRadical(moleculeAtom))
 			return false;
-		int ringSize = (mFragment.getAtomQueryFeatures(fragmentAtom) & Molecule.cAtomQFRingSize) >> Molecule.cAtomQFRingSizeShift;
-		if (ringSize != 0) {
-			if (mMolecule.isFragment()
-			 && ringSize == (mMolecule.getAtomQueryFeatures(fragmentAtom) & Molecule.cAtomQFRingSize) >> Molecule.cAtomQFRingSizeShift)
-				return true;
 
-			boolean found = false;
-			RingCollection ringSet = mMolecule.getRingSet();
-			for (int i=0; i<ringSet.getSize(); i++) {
-				if (ringSet.getRingSize(i) == ringSize) {
-					if (ringSet.isAtomMember(i, moleculeAtom)) {
-						found = true;
-						break;
-						}
-					}
+		int smallestRingSize = (int)((mFragment.getAtomQueryFeatures(fragmentAtom) & Molecule.cAtomQFSmallRingSize) >> Molecule.cAtomQFSmallRingSizeShift);
+		if (smallestRingSize != 0) {
+			if (!mMolecule.isFragment()) {
+				if (mMolecule.getAtomRingSize(moleculeAtom) != smallestRingSize)
+					return false;
+				}
+			else {
+				int targetRingSize = (int)((mMolecule.getAtomQueryFeatures(moleculeAtom) & Molecule.cAtomQFSmallRingSize) >> Molecule.cAtomQFSmallRingSizeShift);
+				if (smallestRingSize != targetRingSize)
+					return false;
 				}
-			if (!found)
-				return false;
 			}
 
 		return true;
@@ -816,24 +851,33 @@ System.out.println();
 				int fragmentParity = mFragment.getAtomParity(fragmentAtom);
 				int moleculeParity = mMolecule.getAtomParity(moleculeAtom);
 
-					// always consider as match if fragment atom is no stereo center
+				// always consider as match if fragment atom is no stereo center
 				if (fragmentParity == Molecule.cAtomParityNone)
 			   		continue;
 
-				// consider as match if asymetrical fragment atom matches on non-stereo-center
-				if (moleculeParity == Molecule.cAtomParityNone)
-			   		continue;
-
-				// unknown molecule centers need to match everything because
-				// parities from idcodes don't include them, i.e. depending
-				// on the source of the parities, unknown centers may look as
-				// no centers. Both must retrieve the same results.
+				// unknown fragment centers match everything
 				if (fragmentParity == Molecule.cAtomParityUnknown)
 			   		continue;
 
-				if (moleculeParity == Molecule.cAtomParityUnknown)
+				// Here the fragment center is clearly specified as either 1 or 2.
+				// Thus, unknown molecule centers should not be considered a match.
+				// A molecule may not have a stereo center here for symmetry reasons,
+				// but match 100% anyway. In this case we interpret: the user wants a
+				// stereo center. Therefore, we don't consider no stereo centers a match.
+				// Into the bargain: parities within idcodes don't include 'unknown',
+				// because the information is implicit: if a stereo center within an idcode
+				// has no 1 or 2 parity, then it is automatically treated to be unknown.
+				// If idcodes are parsed with given or created coordinates, then implicit
+				// unknowns are converted to explicit ones. However, if idcodes are intentionally
+				// parsed without giving coordinates, then an unknown stereo center looks
+				// like a no-stereo-center, because parities were taken from the idcode and
+				// never calculated by ensureHelperArrays().
+				if (moleculeParity == Molecule.cAtomParityNone
+				 || moleculeParity == Molecule.cAtomParityUnknown)
 			   		return false;
 
+				// From here both, fragment and molecule, have a defined parity: 1 or 2.
+
 				if (mFragment.getAtomESRType(fragmentAtom) == Molecule.cESRTypeAnd) {
 					esrGroupAtomCount++;
 					continue;
@@ -1158,7 +1202,7 @@ System.out.println();
 				if (!mFragmentGraphIsRingClosure[current]) {	// current graph position is not an anchor
 					if (!atomUsed[candidate]) {
 						if (areAtomsSimilar(candidate, mFragmentGraphAtom[current])
-								&& areBondsSimilar(mMolecule.getConnBond(mMatchTable[mFragmentGraphParentAtom[current]], index[current]), mFragmentGraphParentBond[current])) {
+						 && areBondsSimilar(mMolecule.getConnBond(mMatchTable[mFragmentGraphParentAtom[current]], index[current]), mFragmentGraphParentBond[current])) {
 							atomUsed[candidate] = true;
 							mMatchTable[mFragmentGraphAtom[current]] = candidate;
 							current++;
@@ -1167,7 +1211,7 @@ System.out.println();
 					}
 				else {	// current graph position is ringClosure
 					if (candidate == mMatchTable[mFragmentGraphAtom[current]]
-							&& areBondsSimilar(mMolecule.getConnBond(mMatchTable[mFragmentGraphParentAtom[current]], index[current]), mFragmentGraphParentBond[current])) {
+					 && areBondsSimilar(mMolecule.getConnBond(mMatchTable[mFragmentGraphParentAtom[current]], index[current]), mFragmentGraphParentBond[current])) {
 						current++;
 						}
 					}
@@ -1219,15 +1263,19 @@ System.out.println();
 	 */
 	private boolean doBridgeBondsMatch(boolean[] moleculeAtomUsed, int excludeGroupNo) {
 		if (mBridgeBondList != null) {
+			mIsBridgeBondAtom = new boolean[moleculeAtomUsed.length];
 			for (BridgeBond bb:mBridgeBondList) {
 				if (mExcludeGroupNo == null
 				 ||	(excludeGroupNo == -1 && mExcludeGroupNo[bb.atom1] == -1 && mExcludeGroupNo[bb.atom2] == -1)
 				 || (excludeGroupNo != -1 && (mExcludeGroupNo[bb.atom1] == excludeGroupNo || mExcludeGroupNo[bb.atom2] == excludeGroupNo))) {
-//				if ((mIsExcludeAtom[bb.atom1] || mIsExcludeAtom[bb.atom2]) == isExcludeFragment) {
-					int bridgeSize = mMolecule.getPathLength(mMatchTable[bb.atom1], mMatchTable[bb.atom2], bb.maxBridgeSize+1, moleculeAtomUsed) - 1;
+					int[] pathAtom = new int[bb.maxBridgeSize+2];
+					int bridgeSize = mMolecule.getPath(pathAtom, mMatchTable[bb.atom1], mMatchTable[bb.atom2], bb.maxBridgeSize+1, moleculeAtomUsed, null) - 1;
 					if (bridgeSize < bb.minBridgeSize
 					 || bridgeSize > bb.maxBridgeSize)
 						return false;
+
+					for (int i=1; i<=bridgeSize; i++)
+						mIsBridgeBondAtom[pathAtom[i]] = true;
 					}
 				}
 			}
@@ -1242,7 +1290,28 @@ System.out.println();
 	 * @return true if all molecule bond features are present in fragment bond
 	 */
 	public boolean areBondsSimilar(int moleculeBond, int fragmentBond) {
-		if ((mMoleculeBondFeatures[moleculeBond] & ~mFragmentBondFeatures[fragmentBond]) != 0)
+		int molDefaults = mMoleculeBondFeatures[moleculeBond];
+		int frgDefaults = mFragmentBondFeatures[fragmentBond];
+
+		if ((mFragment.getBondQueryFeatures(fragmentBond) & Molecule.cBondQFMatchFormalOrder) != 0) {
+			int molBondType = mMolecule.getBondTypeSimple(moleculeBond);
+			int frgBondType = mFragment.getBondTypeSimple(fragmentBond);
+			int frgBondTypes = mFragment.getBondQueryFeatures(fragmentBond) & Molecule.cBondQFBondTypes;
+			if (molBondType != frgBondType
+			 && !(molBondType == Molecule.cBondTypeSingle && (frgBondTypes & Molecule.cBondQFSingle) != 0)
+			 && !(molBondType == Molecule.cBondTypeDouble && (frgBondTypes & Molecule.cBondQFDouble) != 0)
+			 && !(molBondType == Molecule.cBondTypeTriple && (frgBondTypes & Molecule.cBondQFTriple) != 0)
+			 && !(molBondType == Molecule.cBondTypeQuadruple && (frgBondTypes & Molecule.cBondQFQuadruple) != 0)
+			 && !(molBondType == Molecule.cBondTypeQuintuple && (frgBondTypes & Molecule.cBondQFQuintuple) != 0)
+			 && !(molBondType == Molecule.cBondTypeMetalLigand && (frgBondTypes & Molecule.cBondQFMetalLigand) != 0)
+			 && !(molBondType == Molecule.cBondTypeDelocalized && (frgBondTypes & Molecule.cBondQFDelocalized) != 0))
+				return false;
+
+			molDefaults &= ~Molecule.cBondQFBondTypes;
+			frgDefaults &= ~Molecule.cBondQFBondTypes;
+			}
+
+		if ((molDefaults & ~frgDefaults) != 0)
 			return false;
 
 		int ringSize = (mFragment.getBondQueryFeatures(fragmentBond) & Molecule.cBondQFRingSize) >> Molecule.cBondQFRingSizeShift;
@@ -1339,11 +1408,11 @@ System.out.println();
 		int nTotalMoleculeAtoms = mMolecule.getAtoms();
 
 		mMoleculeAtomType = new int[nTotalMoleculeAtoms];
-		mMoleculeAtomFeatures = new int[nTotalMoleculeAtoms];
+		mMoleculeAtomFeatures = new long[nTotalMoleculeAtoms];
 
 		for (int atom=0; atom<nTotalMoleculeAtoms; atom++) {
 			mMoleculeAtomFeatures[atom] = ((getAtomQueryDefaults(mMolecule, atom)
-					| mMolecule.getAtomQueryFeatures(atom))
+				| mMolecule.getAtomQueryFeatures(atom))
 					& Molecule.cAtomQFSimpleFeatures)
 					^ Molecule.cAtomQFNarrowing;
 
@@ -1356,6 +1425,31 @@ System.out.println();
 				mMoleculeAtomType[atom] += mMolecule.getAtomMass(atom) << 16;
 			}
 
+		mMoleculeRingFeatures = new long[nTotalMoleculeAtoms];
+		RingCollection ringSet = mMolecule.getRingSet();
+		for (int i=0; i<ringSet.getSize(); i++) {
+			int ringSize = ringSet.getRingSize(i);
+			for (int atom:ringSet.getRingAtoms(i)) {
+				if (ringSize == 3)
+					mMoleculeRingFeatures[atom] |= Molecule.cAtomQFRingSize3;
+				else if (ringSize == 4)
+					mMoleculeRingFeatures[atom] |= Molecule.cAtomQFRingSize4;
+				else if (ringSize == 5)
+					mMoleculeRingFeatures[atom] |= Molecule.cAtomQFRingSize5;
+				else if (ringSize == 6)
+					mMoleculeRingFeatures[atom] |= Molecule.cAtomQFRingSize6;
+				else if (ringSize == 7)
+					mMoleculeRingFeatures[atom] |= Molecule.cAtomQFRingSize7;
+				}
+			}
+		for (int atom=0; atom<nTotalMoleculeAtoms; atom++) {
+			int ringSize = mMolecule.getAtomRingSize(atom);
+			if (ringSize == 0)
+				mMoleculeRingFeatures[atom] |= Molecule.cAtomQFRingSize0;
+			else if (ringSize > 7)
+				mMoleculeRingFeatures[atom] |= Molecule.cAtomQFRingSizeLarge;
+			}
+
 		int nTotalMoleculeBonds = mMolecule.getBonds();
 
 		mMoleculeBondFeatures = new int[nTotalMoleculeBonds];
@@ -1369,7 +1463,7 @@ System.out.println();
 			}
 
 	private void setupFragmentFeatures(int matchMode) {
-		int[] atomFeaturesWithoutExcludeAtoms = null;
+		long[] atomFeaturesWithoutExcludeAtoms = null;
 		int[] bondFeaturesWithoutExcludeAtoms = null;
 		int[] atomTypeWithoutExcludeAtoms = null;
 
@@ -1378,6 +1472,8 @@ System.out.println();
 		for (int atom=0; atom<mFragment.getAtoms(); atom++)
 			mFragmentConnAtoms[atom] = mFragment.getConnAtoms(atom);
 
+		// If we have exclude groups, we need to determine atom and bond features without the exclude atoms and
+		// map them to the original fragment atom and bond indexes.
 		if (mFragmentExcludeAtoms != 0) {
 			StereoMolecule fragmentWithoutExcludeGroups = new StereoMolecule(mFragment.getAllAtoms(), mFragment.getAllBonds());
 			boolean[] isNonExcludeAtom = new boolean[mFragment.getAllAtoms()];
@@ -1397,6 +1493,8 @@ System.out.println();
 					mFragmentConnAtoms[atom] = fragmentWithoutExcludeGroups.getConnAtoms(index++);
 			}
 
+		// We cannot skip this if we have exclude groups:
+		// We need to determine exclude groups atom & bond features for the exclude group matching phase.
 		setupFragmentFeatures(mFragment, matchMode);
 
 		if (mFragmentExcludeAtoms != 0) {
@@ -1419,12 +1517,12 @@ System.out.println();
 	private void setupFragmentFeatures(StereoMolecule fragment, int matchMode) {
 		int nTotalFragmentAtoms = fragment.getAtoms();
 
-		mFragmentAtomFeatures = new int[fragment.getAtoms()];
+		mFragmentAtomFeatures = new long[fragment.getAtoms()];
 		mFragmentAtomType = new int[fragment.getAtoms()];
 
 		for (int atom=0; atom<nTotalFragmentAtoms; atom++) {
 			mFragmentAtomFeatures[atom] = ((getAtomQueryDefaults(fragment, atom)
-					| fragment.getAtomQueryFeatures(atom))
+				| fragment.getAtomQueryFeatures(atom))
 					& Molecule.cAtomQFSimpleFeatures)
 					^ Molecule.cAtomQFNarrowing;
 			mFragmentAtomType[atom] = fragment.getAtomicNo(atom);
@@ -1436,6 +1534,38 @@ System.out.println();
 				mFragmentAtomType[atom] += fragment.getAtomMass(atom) << 16;
 			}
 
+		mFragmentRingFeatures = new long[fragment.getAtoms()];
+		RingCollection ringSet = fragment.getRingSet();
+		for (int i=0; i<ringSet.getSize(); i++) {
+			boolean containsBridgeBond = false;
+			for (int bond:ringSet.getRingBonds(i)) {
+				if (fragment.isBondBridge(bond)) {
+					containsBridgeBond = true;
+					break;
+					}
+				}
+			if (!containsBridgeBond) {
+				int ringSize = ringSet.getRingSize(i);
+				for (int atom:ringSet.getRingAtoms(i)) {
+					if (ringSize == 3)
+						mFragmentRingFeatures[atom] |= Molecule.cAtomQFRingSize3;
+					else if (ringSize == 4)
+						mFragmentRingFeatures[atom] |= Molecule.cAtomQFRingSize4;
+					else if (ringSize == 5)
+						mFragmentRingFeatures[atom] |= Molecule.cAtomQFRingSize5;
+					else if (ringSize == 6)
+						mFragmentRingFeatures[atom] |= Molecule.cAtomQFRingSize6;
+					else if (ringSize == 7)
+						mFragmentRingFeatures[atom] |= Molecule.cAtomQFRingSize7;
+					}
+				}
+			}
+// Cannot require that, because if a molecule atom is also part of a small ring,
+// then the large ring membership is not known anymore
+//		for (int atom=0; atom<nTotalFragmentAtoms; atom++)
+//			if (fragment.getAtomRingSize(atom) > 7)
+//				mFragmentRingFeatures[atom] |= Molecule.cAtomQFRingSizeLarge;
+
 		int nTotalFragmentBonds = fragment.getBonds();
 
 		mFragmentBondFeatures = new int[fragment.getBonds()];
@@ -1465,15 +1595,23 @@ System.out.println();
 	 * @param atom the atom of which to generate feature flags
 	 * @return atom features independent of query features
 	 */
-	private int getAtomQueryDefaults(StereoMolecule mol, int atom) {
-		int queryDefaults = 0;
+	private long getAtomQueryDefaults(StereoMolecule mol, int atom) {
+		long queryDefaults = 0;
 
 		if (!mol.isFragment()) {
-			if (mol.isAromaticAtom(atom))
+			if (mol.isHeteroAromaticAtom(atom))
+				queryDefaults |= (Molecule.cAtomQFAromatic
+								| Molecule.cAtomQFHeteroAromatic);
+			else if (mol.isAromaticAtom(atom))
 				queryDefaults |= Molecule.cAtomQFAromatic;
 			else
 				queryDefaults |= Molecule.cAtomQFNotAromatic;
 
+			if (mol.isAtomStereoCenter(atom))
+				queryDefaults |= Molecule.cAtomQFIsStereo;
+			else
+				queryDefaults |= Molecule.cAtomQFIsNotStereo;
+
 			int ringBonds = mol.getAtomRingBondCount(atom);
 			if (ringBonds == 0)
 				queryDefaults |= (Molecule.cAtomQFNot2RingBonds
@@ -1535,6 +1673,25 @@ System.out.println();
 				break;
 				}
 
+			int eValue = mol.getAtomElectronegativeNeighbours(atom);
+			switch (eValue) {
+				case 0:
+					queryDefaults |= (Molecule.cAtomQFENeighbours & ~Molecule.cAtomQFNot0ENeighbours);
+					break;
+				case 1:
+					queryDefaults |= (Molecule.cAtomQFENeighbours & ~Molecule.cAtomQFNot1ENeighbour);
+					break;
+				case 2:
+					queryDefaults |= (Molecule.cAtomQFENeighbours & ~Molecule.cAtomQFNot2ENeighbours);
+					break;
+				case 3:
+					queryDefaults |= (Molecule.cAtomQFENeighbours & ~Molecule.cAtomQFNot3ENeighbours);
+					break;
+				default:
+					queryDefaults |= (Molecule.cAtomQFENeighbours & ~Molecule.cAtomQFNot4ENeighbours);
+					break;
+				}
+
 			int piElectrons = mol.getAtomPi(atom);
 			switch (piElectrons) {
 			case 0:
@@ -1553,7 +1710,10 @@ System.out.println();
 			}
 		else {	// The fragments implicit features are not really necessary,
 				  // but may speed up the graph matching.
-			if (mol.isAromaticAtom(atom))
+			if (mol.isHeteroAromaticAtom(atom))
+				queryDefaults |= (Molecule.cAtomQFAromatic
+							  | Molecule.cAtomQFHeteroAromatic);
+			else if (mol.isAromaticAtom(atom))
 				queryDefaults |= Molecule.cAtomQFAromatic;
 
 			int ringBonds = mol.getAtomRingBondCount(atom);
@@ -1590,13 +1750,31 @@ System.out.println();
 				queryDefaults |= (Molecule.cAtomQFNeighbours & ~Molecule.cAtomQFNot4Neighbours);
 				break;
 				}
-			}
 
-		int piElectrons = mol.getAtomPi(atom);
-		if (piElectrons > 0)
-			queryDefaults |= Molecule.cAtomQFNot0PiElectrons;
-		if (piElectrons > 1)
-			queryDefaults |= Molecule.cAtomQFNot1PiElectron;
+			int eValue = mol.getAtomElectronegativeNeighbours(atom);
+			switch (eValue) {
+				case 0:
+					break;
+				case 1:
+					queryDefaults |= (Molecule.cAtomQFNot0ENeighbours);
+					break;
+				case 2:
+					queryDefaults |= (Molecule.cAtomQFNot0ENeighbours | Molecule.cAtomQFNot1ENeighbour);
+					break;
+				case 3:
+					queryDefaults |= (Molecule.cAtomQFNot0ENeighbours | Molecule.cAtomQFNot1ENeighbour | Molecule.cAtomQFNot2ENeighbours);
+					break;
+				default:
+					queryDefaults |= (Molecule.cAtomQFENeighbours & ~Molecule.cAtomQFNot4ENeighbours);
+					break;
+				}
+
+			int piElectrons = mol.getAtomPi(atom);
+			if (piElectrons > 0)
+				queryDefaults |= Molecule.cAtomQFNot0PiElectrons;
+			if (piElectrons > 1)
+				queryDefaults |= Molecule.cAtomQFNot1PiElectron;
+			}
 
 		return queryDefaults;
 		}
@@ -1627,6 +1805,12 @@ System.out.println();
 			case 3:
 				queryDefaults |= Molecule.cBondQFTriple;
 				break;
+			case 4:
+				queryDefaults |= Molecule.cBondQFQuadruple;
+				break;
+			case 5:
+				queryDefaults |= Molecule.cBondQFQuintuple;
+				break;
 			}
 
 		if (mol.isRingBond(bond))
@@ -1648,7 +1832,7 @@ System.out.println();
 		for (int bond=0; bond<mFragment.getBonds(); bond++) {
 			if (mFragment.isBondBridge(bond)) {
 				if (mBridgeBondList == null)
-					mBridgeBondList = new ArrayList<BridgeBond>();
+					mBridgeBondList = new ArrayList<>();
 				BridgeBond bridgeBond = new BridgeBond();
 				bridgeBond.atom1 = mFragment.getBondAtom(0, bond);
 				bridgeBond.atom2 = mFragment.getBondAtom(1, bond);
@@ -1668,4 +1852,10 @@ System.out.println();
 		System.arraycopy(original, 0, copy, 0, Math.min(original.length, newLength));
 		return copy;
 		}
+
+	private static boolean[] copyOf(boolean[] original, int newLength) {
+		boolean[] copy = new boolean[newLength];
+		System.arraycopy(original, 0, copy, 0, Math.min(original.length, newLength));
+		return copy;
+		}
 	}
