@@ -34,7 +34,7 @@
 
 package com.actelion.research.chem;
 
-import java.util.ArrayList;
+import com.actelion.research.chem.descriptor.AbstractDescriptorHandlerLongFP;
 
 // TODO purge mMoleculeIndexInt,mFragmentIndexInt and related methods from this class. Long versions were introduced Aug 3, 2018
 
@@ -563,11 +563,19 @@ public class SSSearcherWithIndex {
 	private int[]				mMoleculeIndexInt,mFragmentIndexInt;
 	private long[]				mMoleculeIndexLong,mFragmentIndexLong;
 	private byte[]				mMoleculeIDCode,mFragmentIDCode;
+	private AbstractDescriptorHandlerLongFP<StereoMolecule> mDescriptorHandler;
 
 	public static int getNoOfKeys() {
         return cKeyIDCode.length;
 	   	}
 
+	public SSSearcherWithIndex(AbstractDescriptorHandlerLongFP<StereoMolecule> dh) {
+		mSSSearcher = new SSSearcher();
+		mDescriptorHandler = dh;
+		init();
+		}
+
+
 	public SSSearcherWithIndex() {
 		mSSSearcher = new SSSearcher();
 		init();
@@ -862,19 +870,13 @@ public class SSSearcherWithIndex {
 
 
 	/**
-	 * If the match count mode is one of cCountModeFirstMatch, cCountModeOverlapping,
-	 * cCountModeRigorous then this method returns an arraylist of all counted matches,
-	 * i.e. int arrays mapping fragment atoms to molecule atoms. Atoms being part of a
-	 * matched bridge bond are naturally not covered by the mapping.<br>
-	 * Note: If some query fragment atoms are marked as exclude group, then the respective
-	 * matchlist values are -1.
-	 * @return list of distinct counted matches.
+	 * @return the SSSearcher employed for the graph matching by this index accelerated SS-searcher
 	 */
-	public ArrayList<int[]> getMatchList() {
-		return mSSSearcher.getMatchList();
+	public SSSearcher getGraphMatcher() {
+		return mSSSearcher;
 		}
 
-
+	@Deprecated // switch to long version; this version does not support the AllFragmentIndex
 	public int[] createIndex(StereoMolecule mol) {
 		if (mol == null)
 			return null;
@@ -896,6 +898,9 @@ public class SSSearcherWithIndex {
 		if (mol == null)
 			return null;
 
+		if (mDescriptorHandler != null)
+			return mDescriptorHandler.createDescriptor(mol);
+
 		long[] index = new long[(cKeyIDCode.length+63)/64];
 		mol = removeExcludeGroups(mol);
 		mSSSearcher.setMolecule(mol);
