@@ -14,7 +14,7 @@
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
-* 3. Neither the name of the the copyright holder nor the
+* 3. Neither the name of the copyright holder nor the
 *    names of its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
@@ -34,14 +34,13 @@
 package com.actelion.research.chem;
 
 import com.actelion.research.gui.generic.GenericPolygon;
+import com.actelion.research.gui.generic.GenericRectangle;
 
 import java.awt.*;
-import java.awt.geom.Rectangle2D;
 import java.awt.image.BufferedImage;
 import java.util.ArrayList;
 
-public class SVGDepictor extends AbstractDepictor
-{
+public class SVGDepictor extends AbstractDepictor {
     public static final int DEFAULT_ELEM_WIDTH = 8;
 
     private static final String FONTNAME = "Helvetica";
@@ -51,13 +50,14 @@ public class SVGDepictor extends AbstractDepictor
     private int textSize = 10;
     private int width = 400;
     private int height = 400;
+    private boolean legacyMode = true;
 
     private String currentColor = "black";
     private final java.util.List<String> bonds = new ArrayList<String>();
     private final java.util.List<String> atoms = new ArrayList<String>();
 
     private String id;
-    private StringBuffer buffer = new StringBuffer();
+    private StringBuilder buffer = new StringBuilder();
 
     private Font currentFont = new Font(FONTNAME, Font.PLAIN, 12);
     private Graphics2D graphics;
@@ -67,33 +67,37 @@ public class SVGDepictor extends AbstractDepictor
         this(mol, 0, id);
     }
 
-    public SVGDepictor(StereoMolecule mol, int displayMode, String id)
-    {
+    public SVGDepictor(StereoMolecule mol, int displayMode, String id) {
         super(mol, displayMode);
         this.id = id;
         instanceCnt++;
     }
 
-    public static final String makeColor(int r, int g, int b)
-    {
+    /**
+     * For legacy reasons the default is to include invisible event atoms and bonds.
+     * Call this method after contruction to skip these non-visible contributions to the SVG.
+     * @param b
+     */
+    public void setLegacyMode(boolean b) {
+        legacyMode = b;
+    }
+
+    public static final String makeColor(int r, int g, int b) {
         return "rgb(" + r + "," + g + "," + b + ")";
     }
 
-    public String getId()
-    {
+    public String getId() {
         return id != null ? id : ("mol" + instanceCnt);
     }
 
-    private void write(String s)
-    {
+    private void write(String s) {
         buffer.append("\t");
         buffer.append(s);
         buffer.append("\n");
     }
 
     @Override
-    protected void drawBlackLine(DepictorLine theLine)
-    {
+    protected void drawBlackLine(DepictorLine theLine) {
         int x1 = (int) theLine.x1;
         int x2 = (int) theLine.x2;
         int y1 = (int) theLine.y1;
@@ -103,32 +107,29 @@ public class SVGDepictor extends AbstractDepictor
                 "y1=\"" + y1 + "\" " +
                 "x2=\"" + x2 + "\" " +
                 "y2=\"" + y2 + "\" " +
-                "style=\"stroke:" + currentColor + ";" +
-                "stroke-width:" + (int) (lineWidth) + "\"/>";
+                "style=\"stroke:" + currentColor + "; stroke-width:" + lineWidth + "\"/>";
         write(s);
     }
 
     @Override
-    protected void drawDottedLine(DepictorLine theLine)
-    {
+    protected void drawDottedLine(DepictorLine theLine) {
         int x1 = (int) theLine.x1;
         int x2 = (int) theLine.x2;
         int y1 = (int) theLine.y1;
         int y2 = (int) theLine.y2;
-        String s = "<line stroke-dasharray=\"3, 3\" " +
+        int d = (int)(3*lineWidth);
+        String s = "<line stroke-dasharray=\""+d+","+d+"\" " +
                 "x1=\"" + x1 + "\" " +
                 "y1=\"" + y1 + "\" " +
                 "x2=\"" + x2 + "\" " +
                 "y2=\"" + y2 + "\" " +
-                "stroke=\"" + currentColor + "\" " +
-                "stroke-width:" + (int) (lineWidth) + "\"/>";
+                "style=\"stroke:" + currentColor + "; stroke-width:" + lineWidth + "\"/>";
 
         write(s);
     }
 
     @Override
-    protected void drawPolygon(GenericPolygon p)
-    {
+    protected void drawPolygon(GenericPolygon p) {
         StringBuilder s = new StringBuilder("<polygon points=\"");
         for (int i=0; i<p.getSize(); i++) {
             s.append(Math.round(p.getX(i)));
@@ -137,21 +138,18 @@ public class SVGDepictor extends AbstractDepictor
             s.append(" ");
         }
         s.append("\" " +
-                "style=\"fill:" + currentColor + ";" +
-                "stroke:" + currentColor + ";" +
-                "stroke-width:1\"/>");
+                "style=\"fill:" + currentColor + "; stroke:" + currentColor + "; stroke-width:"+lineWidth+"\"/>");
         write(s.toString());
     }
 
     @Override
-    protected void drawString(String theString, double x, double y)
-    {
-
+    protected void drawString(String theString, double x, double y) {
         double strWidth = getStringWidth(theString);
         String s = "<text " +
                 "x=\"" + (int) (x - strWidth / 2.0) + "\" " +
                 "y=\"" + (int) (y + textSize / 3) + "\" " +
-                "font-family=\" " + currentFont.getName() + "\" " +
+                "stroke=\"none\" " +
+//                "font-family=\" " + currentFont.getName() + "\" " +
                 "font-size=\"" + currentFont.getSize() + "\" " +
                 "fill=\"" + currentColor + "\">" + theString +
                 "</text>";
@@ -159,8 +157,7 @@ public class SVGDepictor extends AbstractDepictor
     }
 
     @Override
-    protected void fillCircle(double x, double y, double d)
-    {
+    protected void fillCircle(double x, double y, double d) {
         String s = "<circle " +
                 "cx=\"" + (int) (x+d/2) + "\" " +
                 "cy=\"" + (int) (y+d/2) + "\" " +
@@ -176,23 +173,18 @@ public class SVGDepictor extends AbstractDepictor
     }
 
     @Override
-    protected double getStringWidth(String theString)
-    {
+    protected double getStringWidth(String theString) {
         float ret =  (float)currentFont.getStringBounds(theString,graphics.getFontRenderContext()).getWidth();
         return ret;
-
-
     }
 
     @Override
-    protected int getTextSize()
-    {
+    protected int getTextSize() {
         return textSize;
     }
 
     @Override
-    protected void setTextSize(int theSize)
-    {
+    protected void setTextSize(int theSize) {
         if (textSize != theSize) {
             textSize = theSize;
             currentFont = new Font(FONTNAME, Font.PLAIN, theSize);
@@ -200,20 +192,17 @@ public class SVGDepictor extends AbstractDepictor
     }
 
     @Override
-    protected void setLineWidth(double width)
-    {
-        lineWidth = Math.max(width,1);
+    protected void setLineWidth(double width) {
+        lineWidth = Math.round(100 * Math.max(width, 1.0)) / 100;
     }
 
     @Override
-    protected void setColor(Color theColor)
-    {
-        currentColor = makeColor(theColor.getRed(), theColor.getGreen(), theColor.getBlue());
+    protected void setRGB(int rgb) {
+        currentColor = makeColor((rgb & 0x00FF0000) >> 16, (rgb & 0x0000FF00) >> 8, rgb & 0x000000FF);
     }
 
     @Override
-    protected void onDrawBond(int bond, double x1, double y1, double x2, double y2)
-    {
+    protected void onDrawBond(int bond, double x1, double y1, double x2, double y2) {
         String s = "<line " +
                 "id=\"" + getId() + ":Bond:" + bond + "\" " +
                 "class=\"event\" " +	// class to respond to the mouse event
@@ -228,8 +217,7 @@ public class SVGDepictor extends AbstractDepictor
     }
 
     @Override
-    protected void onDrawAtom(int atom, String symbol, double x, double y)
-    {
+    protected void onDrawAtom(int atom, String symbol, double x, double y) {
         int r = DEFAULT_ELEM_WIDTH;
         String s = "<circle " +
                 "id=\"" + getId() + ":Atom:" + atom + "\" " +
@@ -243,10 +231,8 @@ public class SVGDepictor extends AbstractDepictor
 
 
     @Override
-    public String toString()
-    {
-        String header =
-                "<svg " +
+    public String toString() {
+        String header = "<svg " +
                         "id=\"" + getId() + "\" " +
                         "xmlns=\"http://www.w3.org/2000/svg\" version=\"1.1\" " +
                         "width=\"" + width + "px\" " +
@@ -255,39 +241,44 @@ public class SVGDepictor extends AbstractDepictor
 
         String footer = "</svg>";
 
-        String style = "<style>" +
+        String style = legacyMode ?
+                "<style>" +
                 " #" + getId() +
                 " {pointer-events:none; } " +	// Disable Mouse events on the root element so they get passed to the childs
                 " #" + getId() + " .event " +
                 " { pointer-events:all;} " +	// Enable Mouse events for elements possessing the class "event"
-                " </style>\n";
-        String rect =
-                "<rect " +
-                        "x=\"" + 0 + "\" " +
-                        "y=\"" + 0 + "\" " +
-                        "width='" + width + "' " +
-                        "height='" + height + "' style='fill-opacity:0;stroke:red;stroke-width:3'/>\n";
+                " </style>\n"
+              : "<g style=\"font-size:"+getTextSize()+"px; fill-opacity:1; stroke-opacity:1; fill:black; stroke:black;"
+              + " font-weight:normal; text-rendering:optimizeLegibility; font-family:sans-serif;"
+              + " stroke-linejoin:round; stroke-linecap:round; stroke-dashoffset:0;\">";
+
+//        String rect = "<rect " +
+//                      "x=\"" + 0 + "\" " +
+//                      "y=\"" + 0 + "\" " +
+//                      "width='" + width + "' " +
+//                      "height='" + height + "' style='fill-opacity:0;stroke:red;stroke-width:3'/>\n";
         header += "\t";
         header += style;
 //        header += rect;
 
+        if (legacyMode) {
+            // Append the (invisible) bond lines
+            for (String b : bonds)
+                write(b);
 
-        // Append the (invisible) bond lines
-        for (String b : bonds) {
-            write(b);
-        }
-        // Append the (invisible) atom circles
-        for (String a : atoms) {
-            write(a);
+            // Append the (invisible) atom circles
+            for (String a : atoms)
+                write(a);
         }
 
+        if (!legacyMode)
+            write("</g>");
+
         return header + buffer.toString() + footer;
     }
 
     @Override
-    public DepictorTransformation simpleValidateView(Rectangle2D.Double viewRect, int mode)
-    {
-
+    public DepictorTransformation simpleValidateView(GenericRectangle viewRect, int mode) {
         width = (int) viewRect.getWidth();
         height = (int) viewRect.getHeight();
         BufferedImage img = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
@@ -295,6 +286,4 @@ public class SVGDepictor extends AbstractDepictor
 
         return super.simpleValidateView(viewRect, mode);
     }
-
-
 }
