@@ -52,6 +52,7 @@ public class SmilesParser {
 
 	public static final int MODE_SKIP_COORDINATE_TEMPLATES = 4;
 	public static final int MODE_MAKE_HYDROGEN_EXPLICIT = 8;
+	public static final int MODE_NO_CACTUS_SYNTAX = 16;  // if not set, then some CACTVS SMARTS extensions will be recognized and translated as close as possible
 
 	private static final int INITIAL_CONNECTIONS = 16;
 	private static final int MAX_CONNECTIONS = 100; // largest allowed one in SMILES is 99
@@ -68,7 +69,7 @@ public class SmilesParser {
 	private boolean[] mIsAromaticBond;
 	private int mAromaticAtoms,mAromaticBonds,mSmartsMode,mCoordinateMode;
 	private long mRandomSeed;
-	private boolean mCreateSmartsWarnings, mMakeHydrogenExplicit;
+	private boolean mCreateSmartsWarnings,mMakeHydrogenExplicit,mAllowCactvs;
 	private StringBuilder mSmartsWarningBuffer;
 
 	/**
@@ -93,6 +94,7 @@ public class SmilesParser {
 	 */
 	public SmilesParser(int mode, boolean createSmartsWarnings) {
 		mSmartsMode = mode & SMARTS_MODE_MASK;
+		mAllowCactvs = (mode & MODE_NO_CACTUS_SYNTAX) == 0;
 		mCreateSmartsWarnings = createSmartsWarnings;
 		mMakeHydrogenExplicit = ((mode & MODE_MAKE_HYDROGEN_EXPLICIT) != 0);
 		mCoordinateMode = CoordinateInventor.MODE_DEFAULT;
@@ -266,6 +268,7 @@ public class SmilesParser {
 		int bondType = Molecule.cBondTypeSingle;
 		int bondQueryFeatures = 0;
 		SortedList<Integer> atomList = new SortedList<>();
+		SmilesRange range = new SmilesRange(smiles);
 
 		while (smiles[position] <= 32)
 			position++;
@@ -273,6 +276,7 @@ public class SmilesParser {
 		while (position < endIndex) {
 			char theChar = (char)smiles[position++];
 
+			// if there is an atom symbol,
 			if (Character.isLetter(theChar)
 			 || theChar == '*'
 			 || theChar == '?'
@@ -287,12 +291,7 @@ public class SmilesParser {
 				boolean isClockwise = false;
 				int atomQueryFeatures = 0;      // translated from obvious SMARTS features
 				if (squareBracketOpen) {
-					if (theChar == 'R' && Character.isDigit(smiles[position])) {
-						int noOfDigits = Character.isDigit(smiles[position+1]) ? 2 : 1;
-						atomicNo = Molecule.getAtomicNoFromLabel(new String(smiles, position-1, 1+noOfDigits));
-						position += noOfDigits;
-						}
-					else if (theChar == '*') {
+					if (theChar == '*') {
 						atomicNo = 6;
 						atomQueryFeatures |= Molecule.cAtomQFAny;
 						}
@@ -318,47 +317,64 @@ public class SmilesParser {
 							position++;
 							}
 
-						int labelLength = Character.isLowerCase(smiles[position]) ? 2 : 1;
-						atomicNo = Molecule.getAtomicNoFromLabel(new String(smiles, position-1, labelLength));
-						position += labelLength-1;
-						explicitHydrogens = HYDROGEN_IMPLICIT_ZERO;
+						// Handle this before checking for atom symbols. R<n> (ring count) takes precendence to R1 - R16 (substituent pseudo label)
+						if (smiles[position-1] == 'R' && allowSmarts && (Character.isDigit(smiles[position]) || (mAllowCactvs && smiles[position] == '{'))) {
+							atomicNo = 6;
+							atomQueryFeatures |= Molecule.cAtomQFAny;
+							position--;
+							if (isNot)
+								position--;
+							}
+						else {
+							int labelLength = Character.isLowerCase(smiles[position]) ? 2 : 1;
+							atomicNo = Molecule.getAtomicNoFromLabel(new String(smiles, position-1, labelLength));
+							if (atomicNo == -1) {
+								atomicNo = 6;
+								atomQueryFeatures |= Molecule.cAtomQFAny;
+								position--;
+								}
+							else {
+								position += labelLength - 1;
+								explicitHydrogens = HYDROGEN_IMPLICIT_ZERO;
 
-						// If we have a comma after the first atom label, then we need to parse a list.
-						// In this case we also have to set aromaticity query features from upper and lower case symbols.
-						if (allowSmarts && (smiles[position] == ',' || isNot)) {
-							atomList.removeAll();
-							boolean upperCaseFound = false;
-							boolean lowerCaseFound = false;
-							int start = position - labelLength;
-							for (int p=start; p<smiles.length; p++) {
-								if (!Character.isLetter(smiles[p])) {
-									int no = Molecule.getAtomicNoFromLabel(new String(smiles, start, p - start));
-									if (no != 0) {
-										atomList.add(no);
-										if (Character.isUpperCase(smiles[start]))
-											upperCaseFound = true;
-										else
-											lowerCaseFound = true;
+								// If we have a comma after the first atom label, then we need to parse a list.
+								// In this case we also have to set aromaticity query features from upper and lower case symbols.
+								if (allowSmarts && (smiles[position] == ',' || isNot)) {
+									boolean upperCaseFound = false;
+									boolean lowerCaseFound = false;
+									int start = position - labelLength;
+									for (int p=start; p<smiles.length; p++) {
+										if (!Character.isLetter(smiles[p])) {
+											int no = Molecule.getAtomicNoFromLabel(new String(smiles, start, p - start));
+											if (no != 0) {
+												atomList.add(no);
+												if (Character.isUpperCase(smiles[start]))
+													upperCaseFound = true;
+												else
+													lowerCaseFound = true;
+												}
+											start = p+1;
+											if (smiles[p] != ',')
+												break;
+											if (smiles[p+1] == '!') {
+												if (!isNot)
+													throw new Exception("SmilesParser: inconsistent '!' in atom list.");
+												p++;
+												start++;
+												}
+											}
 										}
-									start = p+1;
-									if (smiles[p] != ',')
-										break;
-									if (smiles[p+1] == '!') {
-										if (!isNot)
-											throw new Exception("SmilesParser: inconsistent '!' in atom list.");
-										p++;
-										start++;
+									if (atomList.size() > 1) {
+										explicitHydrogens = HYDROGEN_ANY;   // don't use implicit zero with atom lists
+										if (!upperCaseFound)
+											atomQueryFeatures |= Molecule.cAtomQFAromatic;
+										else if (!lowerCaseFound)
+											atomQueryFeatures |= Molecule.cAtomQFNotAromatic;
 										}
+
+									position = start-1;
 									}
 								}
-							if (atomList.size() > 1) {
-								if (!upperCaseFound)
-									atomQueryFeatures |= Molecule.cAtomQFAromatic;
-								else if (!lowerCaseFound)
-									atomQueryFeatures |= Molecule.cAtomQFNotAromatic;
-								}
-
-							position = start-1;
 							}
 						}
 
@@ -431,41 +447,101 @@ public class SmilesParser {
 
 						if (smiles[position] == 'H') {
 							position++;
-							explicitHydrogens = 1;
-							if (Character.isDigit(smiles[position])) {
-								explicitHydrogens = smiles[position] - '0';
-								position++;
-								}
+							position += range.parse(smiles, position, 1, 1);
+							explicitHydrogens = range.min;
+							int flags = 0;
+							if (range.min <= 0 && range.max >= 0)
+								flags |= Molecule.cAtomQFNot0Hydrogen;
+							if (range.min <= 1 && range.max >= 1)
+								flags |= Molecule.cAtomQFNot1Hydrogen;
+							if (range.min <= 2 && range.max >= 2)
+								flags |= Molecule.cAtomQFNot2Hydrogen;
+							if (range.min <= 3 && range.max >= 3)
+								flags |= Molecule.cAtomQFNot3Hydrogen;
+
 							if (isNot) {
-								if (explicitHydrogens == 0)
-									atomQueryFeatures |= Molecule.cAtomQFNot0Hydrogen;
-								else if (explicitHydrogens == 1)
-									atomQueryFeatures |= Molecule.cAtomQFNot1Hydrogen;
-								else if (explicitHydrogens == 2)
-									atomQueryFeatures |= Molecule.cAtomQFNot2Hydrogen;
-								else if (explicitHydrogens == 3)
-									atomQueryFeatures |= Molecule.cAtomQFNot3Hydrogen;
+								atomQueryFeatures |= flags;
 								explicitHydrogens = HYDROGEN_ANY;
 								}
+							else {
+								if (range.isSingle()) {
+									explicitHydrogens = range.min;
+									}
+								else {
+									atomQueryFeatures |= (Molecule.cAtomQFHydrogen & ~flags);
+									explicitHydrogens = HYDROGEN_ANY;
+									}
+								}
 							continue;
 							}
 
 						if (smiles[position] == 'D') {   // non-H-neighbours
 							position++;
-							int neighbours = 1;
-							if (Character.isDigit(smiles[position])) {
-								neighbours = smiles[position] - '0';
-								position++;
-								}
-							int qf = (neighbours == 0) ? Molecule.cAtomQFNot0Neighbours
-								   : (neighbours == 1) ? Molecule.cAtomQFNot1Neighbour
-								   : (neighbours == 2) ? Molecule.cAtomQFNot2Neighbours
-								   : (neighbours == 3) ? Molecule.cAtomQFNot3Neighbours
-								   : (neighbours == 4) ? Molecule.cAtomQFNot4Neighbours : 0;
-							if (qf != 0) {
+							position += range.parse(smiles, position, 1, 1);
+							long flags = 0;
+							if (range.min <= 0 && range.max >= 0)
+								flags |= Molecule.cAtomQFNot0Neighbours;
+							if (range.min <= 1 && range.max >= 1)
+								flags |= Molecule.cAtomQFNot1Neighbour;
+							if (range.min <= 2 && range.max >= 2)
+								flags |= Molecule.cAtomQFNot2Neighbours;
+							if (range.min <= 3 && range.max >= 3)
+								flags |= Molecule.cAtomQFNot3Neighbours;
+							if (range.min <= 4 && range.max >= 4)
+								flags |= Molecule.cAtomQFNot4Neighbours;
+
+							if (flags != 0) {
 								if (!isNot)
-									qf = qf ^ Molecule.cAtomQFNeighbours;
-								atomQueryFeatures |= qf;
+									flags = flags ^ Molecule.cAtomQFNeighbours;
+								atomQueryFeatures |= flags;
+								}
+							continue;
+							}
+
+						if (smiles[position] == 'z' && mAllowCactvs) {   // electro-negative neighbour count (CACTVS extension)
+							position++;
+							position += range.parse(smiles, position, 1, 4);
+							long flags = 0;
+							if (range.min <= 0 && range.max >= 0)
+								flags |= Molecule.cAtomQFNot0ENeighbours;
+							if (range.min <= 1 && range.max >= 1)
+								flags |= Molecule.cAtomQFNot1ENeighbour;
+							if (range.min <= 2 && range.max >= 2)
+								flags |= Molecule.cAtomQFNot2ENeighbours;
+							if (range.min <= 3 && range.max >= 3)
+								flags |= Molecule.cAtomQFNot3ENeighbours;
+							if (range.min <= 4 && range.max >= 4)
+								flags |= Molecule.cAtomQFNot4ENeighbours;
+
+							if (flags != 0) {
+								if (!isNot)
+									flags = flags ^ Molecule.cAtomQFENeighbours;
+								atomQueryFeatures |= flags;
+								}
+							continue;
+							}
+
+						if (smiles[position] == 'X') {   // neighbour count including implicit hydrogens
+							position++;
+							position += range.parse(smiles, position, 1, 1);
+							byte[] valences = Molecule.cAtomValence[atomicNo];
+							if (valences == null)
+								continue;
+
+							int valence = valences[0];
+							long flags = 0;
+							// we convert into pi-electron count using standard valence
+							if (valence-range.min <= 0 && valence-range.max >= 0)
+								flags |= Molecule.cAtomQFNot0PiElectrons;
+							if (valence-range.min <= 1 && valence-range.max >= 1)
+								flags |= Molecule.cAtomQFNot1PiElectron;
+							if (valence-range.min <= 2 && valence-range.max >= 2)
+								flags |= Molecule.cAtomQFNot2PiElectrons;
+
+							if (flags != 0) {
+								if (!isNot)
+									flags = flags ^ Molecule.cAtomQFPiElectrons;
+								atomQueryFeatures |= flags;
 								}
 							continue;
 							}
@@ -478,73 +554,62 @@ public class SmilesParser {
 
 						if (smiles[position] == 'R') {
 							position++;
-							if (!Character.isDigit(smiles[position])) {
-								if (isNot)
-									atomQueryFeatures |= Molecule.cBondQFRingState & ~Molecule.cAtomQFNotChain;
-								else
-									atomQueryFeatures |= Molecule.cAtomQFNotChain;
-								continue;
-								}
-							int ringCount = smiles[position] - '0';
-							position++;
-							if (isNot) {
-								if (ringCount == 0)
-									atomQueryFeatures |= Molecule.cAtomQFNotChain;
-								else if (ringCount == 1)
-									atomQueryFeatures |= Molecule.cAtomQFNot2RingBonds;
-								else if (ringCount == 2)
-									atomQueryFeatures |= Molecule.cAtomQFNot3RingBonds;
-								else if (ringCount == 3)
-									atomQueryFeatures |= Molecule.cAtomQFNot4RingBonds;
-								else
-									smartsWarning("!R"+ringCount);
-								}
-							else {
-								if (ringCount >= 3)
-									ringCount = 3;
-								if (ringCount == 0)
-									atomQueryFeatures |= Molecule.cAtomQFRingState & ~Molecule.cAtomQFNotChain;
-								else if (ringCount == 1)
-									atomQueryFeatures |= Molecule.cAtomQFRingState & ~Molecule.cAtomQFNot2RingBonds;
-								else if (ringCount == 2)
-									atomQueryFeatures |= Molecule.cAtomQFRingState & ~Molecule.cAtomQFNot3RingBonds;
-								else if (ringCount == 3)
-									atomQueryFeatures |= Molecule.cAtomQFRingState & ~Molecule.cAtomQFNot4RingBonds;
-								else
-									smartsWarning("R"+ringCount);
+							position += range.parse(smiles, position, 1, 3);
+							long flags = 0;
+							if (range.min <= 0 && range.max >= 0)
+								flags |= Molecule.cAtomQFNotChain;
+							if (range.min <= 1 && range.max >= 1)
+								flags |= Molecule.cAtomQFNot2RingBonds;
+							if (range.min <= 2 && range.max >= 2)
+								flags |= Molecule.cAtomQFNot3RingBonds;
+							if (range.min <= 3 && range.max >= 3)
+								flags |= Molecule.cAtomQFNot4RingBonds;
+							if (range.max > 3)
+								smartsWarning((isNot?"!R":"R")+range.max);
+
+							if (flags != 0) {
+								if (!isNot)
+									flags = flags ^ Molecule.cAtomQFRingState;
+								atomQueryFeatures |= flags;
 								}
 							continue;
 							}
 
 						if (smiles[position] == 'r') {
 							position++;
-							if (!Character.isDigit(smiles[position])) {
+							position += range.parse(smiles, position, 1, 1);
+							if (range.isDefault) {
 								if (isNot)
 									atomQueryFeatures |= Molecule.cBondQFRingState & ~Molecule.cAtomQFNotChain;
 								else
 									atomQueryFeatures |= Molecule.cAtomQFNotChain;
 								continue;
 								}
-							int ringSize = smiles[position] - '0';
-							position++;
+
+							int ringSize = range.min;
+
+							if (range.isRange())
+								smartsWarning((isNot ? "!r" : "r") + range.toString());
+
 							if (!isNot && ringSize >= 3 && ringSize <= 7)
-								atomQueryFeatures |= (ringSize << Molecule.cAtomQFRingSizeShift);
-							else
+								atomQueryFeatures |= (ringSize << Molecule.cAtomQFSmallRingSizeShift);
+							else if (!range.isRange())
 								smartsWarning((isNot ? "!r" : "r") + ringSize);
 							continue;
 							}
 
 						if (smiles[position] == 'v') {
 							position++;
-							int valence = smiles[position] - '0';
-							position++;
-							if (Character.isDigit(smiles[position])) {
-								valence = 10 * valence + smiles[position] - '0';
-								position++;
-								}
+							position += range.parse(smiles, position, 1, 1);
+
+							int valence = range.min;
+
+							if (range.isRange())
+								smartsWarning((isNot ? "!v" : "v") + range.toString());
+
 							if (!isNot && valence <= 14)
 								abnormalValence = valence;
-							else
+							else if (!range.isRange())
 								smartsWarning((isNot ? "!v" : "v") + valence);
 							continue;
 							}
@@ -555,7 +620,7 @@ public class SmilesParser {
 							continue;
 							}
 
-						throw new Exception("SmilesParser: unexpected character inside brackets: '"+theChar+"'");
+						throw new Exception("SmilesParser: unexpected character inside brackets: '"+(char)smiles[position]+"'");
 						}
 					}
 				else if (theChar == '*') {
@@ -565,13 +630,14 @@ public class SmilesParser {
 				else if (theChar == '?') {
 					atomicNo = 0;
 					}
+				else if ((theChar == 'A' || theChar == 'a') && allowSmarts) {
+					atomicNo = 6;
+					atomQueryFeatures |= Molecule.cAtomQFAny;
+					atomQueryFeatures |= theChar == 'A' ? Molecule.cAtomQFNotAromatic : Molecule.cAtomQFAromatic;
+					smartsFeatureFound = true;
+					}
 				else {
 					switch (Character.toUpperCase(theChar)) {
-					case 'A':
-						atomicNo = 6;
-						atomQueryFeatures |= Molecule.cAtomQFAny;
-						atomQueryFeatures |= theChar == 'A' ? Molecule.cAtomQFNotAromatic : Molecule.cAtomQFAromatic;
-						break;
 					case 'B':
 						if (position < endIndex && smiles[position] == 'r') {
 							atomicNo = 35;
@@ -622,6 +688,14 @@ public class SmilesParser {
 				mMol.setAtomAbnormalValence(atom, abnormalValence);
 				if (atomQueryFeatures != 0) {
 					smartsFeatureFound = true;
+					if ((atomQueryFeatures & Molecule.cAtomQFAromatic) != 0) {
+						atomQueryFeatures &= ~Molecule.cAtomQFAromatic;
+						mMol.setAtomMarker(atom, true);
+						mAromaticAtoms++;
+						}
+					else {
+						mMol.setAtomMarker(atom, false);
+						}
 					mMol.setAtomQueryFeature(atom, atomQueryFeatures, true);
 					}
 				if (atomList.size() != 0) {
@@ -630,18 +704,19 @@ public class SmilesParser {
 					for (int i=0; i<atomList.size(); i++)
 						list[i] = atomList.get(i);
 					mMol.setAtomList(atom, list);
+					atomList.removeAll();
 					}
+				else {  // mark aromatic atoms
+					if (Character.isLowerCase(theChar)) {
+						if (atomicNo != 5 && atomicNo != 6 && atomicNo != 7 && atomicNo != 8 && atomicNo != 15 && atomicNo != 16 && atomicNo != 33 && atomicNo != 34)
+							throw new Exception("SmilesParser: atomicNo " + atomicNo + " must not be aromatic");
 
-				// mark aromatic atoms
-				if (Character.isLowerCase(theChar)) {
-					if (atomicNo != 5 && atomicNo != 6 && atomicNo != 7 && atomicNo != 8 && atomicNo != 15 &&atomicNo != 16 && atomicNo != 33  && atomicNo != 34)
-						throw new Exception("SmilesParser: atomicNo "+atomicNo+" must not be aromatic");
-
-					mMol.setAtomMarker(atom, true);
-					mAromaticAtoms++;
-					}
-				else {
-					mMol.setAtomMarker(atom, false);
+						mMol.setAtomMarker(atom, true);
+						mAromaticAtoms++;
+						}
+					else {
+						mMol.setAtomMarker(atom, false);
+						}
 					}
 
 				// put explicitHydrogen into atomCustomLabel to keep atom-relation when hydrogens move to end of atom list in handleHydrogen()
@@ -715,8 +790,12 @@ public class SmilesParser {
 							excludedBonds |= Molecule.cBondQFDouble;
 						else if (theChar == '#')
 							excludedBonds |= Molecule.cBondQFTriple;
+						else if (theChar == '$')
+							excludedBonds |= Molecule.cBondQFQuadruple;
 						else if (theChar == ':')
 							excludedBonds |= Molecule.cBondQFDelocalized;
+						else
+							throw new Exception("SmilesParser: bond symbol '"+theChar+"' not allowed after '!'.");
 						}
 					else {
 						if (theChar == '@')
@@ -725,8 +804,12 @@ public class SmilesParser {
 							bondType = Molecule.cBondTypeDouble;
 						else if (theChar == '#')
 							bondType = Molecule.cBondTypeTriple;
+						else if (theChar == '$')
+							bondType = Molecule.cBondTypeQuadruple;
 						else if (theChar == ':')
 							bondType = Molecule.cBondTypeDelocalized;
+						else if (theChar == '~')
+							bondQueryFeatures |= Molecule.cBondQFBondTypes;
 						else if (theChar == '/') {
 							if (readStereoFeatures)
 								bondType = Molecule.cBondTypeUp;    // encode slash temporarily in bondType
@@ -795,8 +878,10 @@ public class SmilesParser {
 										|| smiles[position-2] == '\\'
 										|| smiles[position-2] == '='
 										|| smiles[position-2] == '#'
+										|| smiles[position-2] == '$'
 										|| smiles[position-2] == ':'
-										|| smiles[position-2] == '>');
+										|| smiles[position-2] == '>'
+										|| smiles[position-2] == '~');
 					if (isDoubleDigit
 					 && position < endIndex
 					 && Character.isDigit(smiles[position])) {
@@ -923,30 +1008,36 @@ public class SmilesParser {
 			if (mMol.getAtomCustomLabel(atom) != null) {	// if we have the exact number of hydrogens
 				int explicitHydrogen = mMol.getAtomCustomLabelBytes(atom)[0];
 
-				if (mMakeHydrogenExplicit) {
-					for (int i=0; i<explicitHydrogen; i++)
-						mMol.addBond(atom, mMol.addAtom(1), 1);
-					}
-				else if (smartsFeatureFound || mSmartsMode == SMARTS_MODE_IS_SMARTS) {
-					if (explicitHydrogen == 0)
-						mMol.setAtomQueryFeature(atom, Molecule.cAtomQFHydrogen & ~Molecule.cAtomQFNot0Hydrogen, true);
-					if (explicitHydrogen == 1)
-						mMol.setAtomQueryFeature(atom, Molecule.cAtomQFHydrogen & ~Molecule.cAtomQFNot1Hydrogen, true);
-					if (explicitHydrogen == 2)
-						mMol.setAtomQueryFeature(atom, Molecule.cAtomQFHydrogen & ~Molecule.cAtomQFNot2Hydrogen, true);
-					if (explicitHydrogen == 3)
-						mMol.setAtomQueryFeature(atom, Molecule.cAtomQFHydrogen & ~Molecule.cAtomQFNot3Hydrogen, true);
+				if (smartsFeatureFound || mSmartsMode == SMARTS_MODE_IS_SMARTS) {
+					if (mMakeHydrogenExplicit) {
+						for (int i=0; i<explicitHydrogen; i++)
+							mMol.addBond(atom, mMol.addAtom(1), 1);
+						}
+					else {
+						if (explicitHydrogen == 0)
+							mMol.setAtomQueryFeature(atom, Molecule.cAtomQFHydrogen & ~Molecule.cAtomQFNot0Hydrogen, true);
+						if (explicitHydrogen == 1)
+							mMol.setAtomQueryFeature(atom, Molecule.cAtomQFHydrogen & ~Molecule.cAtomQFNot1Hydrogen, true);
+						if (explicitHydrogen == 2)
+							mMol.setAtomQueryFeature(atom, Molecule.cAtomQFHydrogen & ~Molecule.cAtomQFNot2Hydrogen, true);
+						if (explicitHydrogen == 3)
+							mMol.setAtomQueryFeature(atom, Molecule.cAtomQFHydrogen & ~Molecule.cAtomQFNot3Hydrogen, true);
+						}
 					}
 				else {
-					if (!mMol.isMarkedAtom(atom)) {
-						// We don't correct aromatic atoms, because for aromatic atoms the number of
+					if (!mMol.isMetalAtom(atom)
+					 && (!mMol.isMarkedAtom(atom)
+					  || (mMol.getAtomicNo(atom) == 6 && mMol.getAtomCharge(atom) == 0))) {
+						// We don't correct aromatic non-carbon atoms, because for these the number of
 						// explicit hydrogens encodes whether a pi-bond needs to be placed at the atom
-						// when resolving aromaticity.
+						// when resolving aromaticity. Same applies for charged carbon.
 						byte[] valences = Molecule.getAllowedValences(mMol.getAtomicNo(atom));
 						boolean compatibleValenceFound = false;
 						int usedValence = mMol.getOccupiedValence(atom);
 						usedValence -= mMol.getElectronValenceCorrection(atom, usedValence);
 						usedValence += explicitHydrogen;
+						if (mMol.isMarkedAtom(atom))    // later we will use a valence for the pi-bond
+							usedValence++;
 						for (byte valence:valences) {
 							if (usedValence <= valence) {
 								compatibleValenceFound = true;
@@ -963,11 +1054,9 @@ public class SmilesParser {
 							mMol.setAtomAbnormalValence(atom, usedValence);
 						}
 
-					if (!mMol.supportsImplicitHydrogen(atom)) {
-						// If implicit hydrogens are not supported, then add explicit ones.
+					if (mMakeHydrogenExplicit || !mMol.supportsImplicitHydrogen(atom))
 						for (int i=0; i<explicitHydrogen; i++)
 							mMol.addBond(atom, mMol.addAtom(1), 1);
-						}
 					}
 				}
 			else if (!mMakeHydrogenExplicit && (smartsFeatureFound || mSmartsMode == SMARTS_MODE_IS_SMARTS)) {
@@ -1046,10 +1135,12 @@ public class SmilesParser {
 		return theChar == '-'
 			|| theChar == '='
 			|| theChar == '#'
+			|| theChar == '$'
 			|| theChar == ':'
 			|| theChar == '/'
 			|| theChar == '\\'
 			|| theChar == '<'
+			|| theChar == '~'
 			|| theChar == '!'
 			|| theChar == '@';
 		}
@@ -1057,8 +1148,10 @@ public class SmilesParser {
 	private int bondSymbolToQueryFeature(char symbol) {
 		return symbol == '=' ? Molecule.cBondQFDouble
 			 : symbol == '#' ? Molecule.cBondQFTriple
+			 : symbol == '$' ? Molecule.cBondQFQuadruple
 			 : symbol == ':' ? Molecule.cBondQFDelocalized
-			 : symbol == '>' ? Molecule.cBondQFMetalLigand : Molecule.cBondQFSingle;
+			 : symbol == '>' ? Molecule.cBondQFMetalLigand
+			 : symbol == '~' ? Molecule.cBondQFBondTypes : Molecule.cBondQFSingle;
 		}
 
 	private void smartsWarning(String feature) {
@@ -1387,12 +1480,16 @@ public class SmilesParser {
 
 
 	private boolean qualifiesForPi(int atom) {
-		if (!RingCollection.qualifiesAsAromatic(mMol.getAtomicNo(atom)))
+		if (!RingCollection.qualifiesAsAromaticAtomicNo(mMol.getAtomicNo(atom)))
 			return false;
 
-		if ((mMol.getAtomicNo(atom) == 6 && mMol.getAtomCharge(atom) != 0)
-		 || !mMol.isMarkedAtom(atom))	// already marked as hetero-atom of another ring
-			return false;
+		if (mMol.getAtomicNo(atom) == 6) {
+			if (!mMol.isMarkedAtom(atom))	// already member of another aromatic ring
+				return false;
+
+			if (mMol.getAtomCharge(atom) > 0)
+				return false;
+			}
 
 		int explicitHydrogens = (mMol.getAtomCustomLabel(atom) == null) ?
 								0 : mMol.getAtomCustomLabelBytes(atom)[0];
@@ -1792,4 +1889,63 @@ System.out.println("parity:"+parity);
 				}
 			}
 		}
-	}
\ No newline at end of file
+	}
+
+class SmilesRange {
+	private byte[] smi;
+	private int pos;
+	public int min,max;
+	public boolean isDefault;
+
+	public SmilesRange(byte[] smiles) {
+		smi = smiles;
+		}
+
+	public int parse(byte[] smiles, int position, int defaultMin, int defaultMax) {
+		isDefault = false;
+		pos = position;
+
+		if (Character.isDigit(smiles[position])) {
+			min = max = parseInt();
+			return pos - position;
+			}
+
+		if (smiles[position] == '{'
+			&& Character.isDigit(smiles[position+1])) {
+			pos++;
+			min = parseInt();
+			if (smiles[pos++] != '-')
+				return 0;   // unexpected
+			if (!Character.isDigit(smiles[pos]))
+				return 0;   // unexpected
+			max = parseInt();
+			if (smiles[pos++] != '}')
+				return 0;   // unexpected
+			return pos - position;
+			}
+
+		min = defaultMin;
+		max = defaultMax;
+		isDefault = true;
+		return 0;
+	}
+
+	public boolean isSingle() {
+		return max == min;
+	}
+
+	public boolean isRange() {
+		return max > min;
+	}
+
+	public String toString() {
+		return "{"+min+"-"+max+"}";
+	}
+
+	private int parseInt() {
+		int num = smi[pos++] - '0';
+		if (Character.isDigit(smi[pos]))
+			num = 10 * num + (smi[pos++] - '0');
+		return num;
+	}
+}
