@@ -159,13 +159,14 @@ public class CoordinateInventor {
 	 * @parem ffp null or fragment fingerprint of the molecule, which is used (if available) for faster template location
 	 */
 	public void invent(StereoMolecule mol, long[] ffp) {
-		boolean paritiesPresent = ((mol.getHelperArrayStatus() & Molecule.cHelperParities) != 0);
+		boolean paritiesPresent = (mol.getHelperArrayStatus() & Molecule.cHelperParities) != 0;
+		int parityState = mol.getHelperArrayStatus() & Molecule.cHelperBitsStereo;
 
 		if (mRandom == null)
 			mRandom = new Random();
 
 		if ((mMode & MODE_REMOVE_HYDROGEN) != 0)
-			mol.removeExplicitHydrogens();
+			mol.removeExplicitHydrogens(false, false);
 
 		mMol = mol;
 		mMol.ensureHelperArrays(Molecule.cHelperRings);
@@ -227,8 +228,10 @@ public class CoordinateInventor {
 				}
 			}
 
-		if (paritiesPresent)
+		if (paritiesPresent) {
+			mMol.setParitiesValid(parityState);
 			mMol.setStereoBondsFromParity();
+			}
 
 		if (mAbsoluteOrientationTemplateFound)
 			mMol.removeAtomMarkers();
@@ -262,7 +265,7 @@ public class CoordinateInventor {
 			if (useFFP) {
 				searcherWithIndex.setFragment(templateMol, template.getFFP());
 				if (searcherWithIndex.findFragmentInMolecule(SSSearcher.cCountModeOverlapping, SSSearcher.cDefaultMatchMode) != 0)
-					matchList = searcherWithIndex.getMatchList();
+					matchList = searcherWithIndex.getGraphMatcher().getMatchList();
 				}
 			else {
 				searcher.setFragment(templateMol);
@@ -1140,13 +1143,18 @@ public class CoordinateInventor {
 				}
 			}
 
-		// if the bond is also a member of a small ring and if more than this one bond are shared
-		// then the first and the last shared bond are E and the ones between Z.
+		// If the bond is also a member of a small ring
 		if (mMol.isSmallRingBond(ringBond[index])) {
 			int sharedRing1 = mMol.getRingSet().getSharedRing(ringBond[lowerIndex], ringBond[index]);
 			int sharedRing2 = mMol.getRingSet().getSharedRing(ringBond[higherIndex], ringBond[index]);
+
+			// If more than one bond is shared by the large and the small ring,
+			// then the first and the last shared bond are E and the ones between Z.
 			if (sharedRing1 != -1 || sharedRing2 != -1)
 				return sharedRing1 == sharedRing2 ? Molecule.cBondParityZor2 : Molecule.cBondParityEor1;
+
+			// If only this one bond is shared, then we have a Z-configuration
+			return Molecule.cBondParityZor2;
 			}
 
 		return Molecule.cBondParityNone;
