@@ -45,40 +45,49 @@ import java.io.IOException;
 import java.util.ArrayList;
 
 public abstract class CompoundFileHelper {
-	public static final int cFileTypeMask = 0x0003FFFF;
+	public static final int cFileTypeMask = 0x007FFFFF;
 	public static final int cFileTypeDataWarrior = 0x00000001;
 	public static final int cFileTypeDataWarriorTemplate = 0x00000002;
 	public static final int cFileTypeDataWarriorQuery = 0x00000004;
 	public static final int cFileTypeDataWarriorMacro = 0x00000008;
 	public static final int cFileTypeTextTabDelimited = 0x00000010;
     public static final int cFileTypeTextCommaSeparated = 0x00000020;
-    public static final int cFileTypeText = cFileTypeTextTabDelimited | cFileTypeTextCommaSeparated;
-	public static final int cFileTypeSDV3 = 0x00000040;
-    public static final int cFileTypeSDV2 = 0x00000080;
+	public static final int cFileTypeTextSemicolonSeparated = 0x00000040;
+	public static final int cFileTypeTextVLineSeparated = 0x00000080;
+	public static final int cFileTypeTextAnyCSV = cFileTypeTextCommaSeparated | cFileTypeTextSemicolonSeparated | cFileTypeTextVLineSeparated;
+    public static final int cFileTypeTextAny = cFileTypeTextTabDelimited | cFileTypeTextAnyCSV;
+	public static final int cFileTypeSDV3 = 0x00000100;
+    public static final int cFileTypeSDV2 = 0x00000200;
     public static final int cFileTypeSD = cFileTypeSDV3 | cFileTypeSDV2;
-	public static final int cFileTypeRXN = 0x00000100;
-	public static final int cFileTypeSOM = 0x00000200;
-	public static final int cFileTypeJPG = 0x00000400;
-	public static final int cFileTypeGIF = 0x00000800;
-	public static final int cFileTypePNG = 0x00001000;
-	public static final int cFileTypeSVG = 0x00002000;
+	public static final int cFileTypeRXN = 0x00000400;
+	public static final int cFileTypeSOM = 0x00000800;
+	public static final int cFileTypeJPG = 0x00001000;
+	public static final int cFileTypeGIF = 0x00002000;
+	public static final int cFileTypePNG = 0x00004000;
+	public static final int cFileTypeSVG = 0x00008000;
 	public static final int cFileTypePictureFile = cFileTypeJPG | cFileTypeGIF | cFileTypePNG | cFileTypeSVG;
-    public static final int cFileTypeRDV3 = 0x00004000;
-    public static final int cFileTypeRDV2 = 0x00008000;
+    public static final int cFileTypeRDV3 = 0x00010000;
+    public static final int cFileTypeRDV2 = 0x00020000;
     public static final int cFileTypeRD = cFileTypeRDV3 | cFileTypeRDV2;
-	public static final int cFileTypeMOL = 0x00010000;
-	public static final int cFileTypeMOL2 = 0x00020000;
-	public static final int cFileTypePDB = 0x00040000;
-	public static final int cFileTypeMMTF = 0x00080000;
-	public static final int cFileTypeProtein = 0x000C0000;
-	public static final int cFileTypeSDGZ = 0x00100000;
+	public static final int cFileTypeMOL = 0x00040000;
+	public static final int cFileTypeMOL2 = 0x00080000;
+	public static final int cFileTypePDB = 0x00100000;
+	public static final int cFileTypeMMTF = 0x00200000;
+	public static final int cFileTypeProtein = cFileTypePDB | cFileTypeMMTF;
+	public static final int cFileTypeSDGZ = 0x00400000;
     public static final int cFileTypeUnknown = -1;
 	public static final int cFileTypeDirectory = -2;
 
+	public static final int cFileTypeCompoundFiles =
+			  CompoundFileHelper.cFileTypeMOL
+			| CompoundFileHelper.cFileTypeMOL2
+			| CompoundFileHelper.cFileTypeSD
+			| CompoundFileHelper.cFileTypeDataWarrior;
+
 	// explicitly supported compression format (SD-files only)
 	public static final String cGZipExtention = ".gz";
 
-	public static final int cFileTypeDataWarriorCompatibleData = cFileTypeDataWarrior | cFileTypeText | cFileTypeRD | cFileTypeSD | cFileTypeSDGZ;
+	public static final int cFileTypeDataWarriorCompatibleData = cFileTypeDataWarrior | cFileTypeTextAny | cFileTypeRD | cFileTypeSD | cFileTypeSDGZ;
 	public static final int cFileTypeDataWarriorTemplateContaining = cFileTypeDataWarrior | cFileTypeDataWarriorQuery | cFileTypeDataWarriorTemplate;
 
 	private static File sCurrentDirectory;
@@ -98,22 +107,12 @@ public abstract class CompoundFileHelper {
 		}
 
 	public ArrayList<StereoMolecule> readStructuresFromFile(boolean readIdentifier) {
-        File file = selectFileToOpen("Please select substance file",
-					           CompoundFileHelper.cFileTypeMOL
-					         | CompoundFileHelper.cFileTypeMOL2
-                             | CompoundFileHelper.cFileTypeSD
-                             | CompoundFileHelper.cFileTypeDataWarrior);
-
+        File file = selectFileToOpen("Please select a compound file", cFileTypeCompoundFiles);
         return readStructuresFromFile(file, readIdentifier);
 	    }
 
 	public ArrayList<String> readIDCodesFromFile() {
-        File file = selectFileToOpen("Please select substance file",
-		                CompoundFileHelper.cFileTypeMOL
-				             | CompoundFileHelper.cFileTypeMOL2
-					         | CompoundFileHelper.cFileTypeSD
-                             | CompoundFileHelper.cFileTypeDataWarrior);
-
+        File file = selectFileToOpen("Please select a compound file", cFileTypeCompoundFiles);
         return readIDCodesFromFile(file);
 	    }
 
@@ -129,7 +128,7 @@ public abstract class CompoundFileHelper {
 
 	/**
 	 * Reads all compounds as idcode list from the given file.
-	 * @param file SD- or DataWarrior file
+	 * @param file MOL-, mol2-, SD- or DataWarrior file
 	 * @return
 	 */
 	public ArrayList<String> readIDCodesFromFile(File file) {
@@ -321,12 +320,13 @@ public abstract class CompoundFileHelper {
 			filter.addDescription("DataWarrior macro files");
 			}
 		if ((filetypes & cFileTypeTextTabDelimited) != 0) {
+			filter.addExtension("tsv");
 			filter.addExtension("txt");
 			filter.addDescription("TAB delimited text files");
 			}
-        if ((filetypes & cFileTypeTextCommaSeparated) != 0) {
+        if ((filetypes & cFileTypeTextAnyCSV) != 0) {
             filter.addExtension("csv");
-            filter.addDescription("Comma separated text files");
+            filter.addDescription("Comma [,;|] separated text files");
             }
 		if ((filetypes & cFileTypeRXN) != 0) {
 			filter.addExtension("rxn");
@@ -453,6 +453,11 @@ public abstract class CompoundFileHelper {
 		return (i == -1) ? filePath : filePath.substring(0, i);
 		}
 
+	/**
+	 * Note: If
+	 * @param filename
+	 * @return one or multiple filtetypes that matching the extension of the given filename
+	 */
 	public static int getFileType(String filename) {
         int index = getExtensionIndex(filename);
 
@@ -470,10 +475,10 @@ public abstract class CompoundFileHelper {
             return cFileTypeSOM;
         if (extension.equals(".dwam"))
             return cFileTypeDataWarriorMacro;
-        if (extension.equals(".txt"))
+        if (extension.equals(".txt") || extension.equals(".tsv"))
             return cFileTypeTextTabDelimited;
         if (extension.equals(".csv"))
-            return cFileTypeTextCommaSeparated;
+            return cFileTypeTextAnyCSV;
         if (extension.equals(".sdf"))
             return cFileTypeSD;
 		if (extension.equals(".sdf.gz"))
@@ -510,11 +515,11 @@ public abstract class CompoundFileHelper {
     	ArrayList<String> list = new ArrayList<String>();
     	int type = 0x00000001;
     	while ((type & cFileTypeMask) != 0) {
-    		if ((type & fileTypes) != 0) {
-    			String extension = getExtension(type);
-    			if (extension.length() != 0 && !list.contains(extension))
-    				list.add(extension);
-    			}
+    		if ((type & fileTypes) != 0)
+    			for (String extension:getExtensions(type))
+    			    if (!list.contains(extension))
+    				    list.add(extension);
+
     		type <<= 1;
     		}
     	return list;
@@ -522,74 +527,88 @@ public abstract class CompoundFileHelper {
 
 	/**
 	 * @param filetype
-	 * @return file extension including the dot
+	 * @return preferred file extension including the dot
 	 */
-    public static String getExtension(int filetype) {
-		String extension = "";
+	public static String getExtension(int filetype) {
+    	String[] extensions = getExtensions(filetype);
+    	return extensions.length == 0 ? "" : extensions[0];
+		}
+
+	/**
+	 * @param filetype
+	 * @return file extensions including the dot
+	 */
+    public static String[] getExtensions(int filetype) {
+		ArrayList<String> extensions = new ArrayList<>();
 		switch (filetype) {
 		case cFileTypeDataWarrior:
-			extension = ".dwar";
+			extensions.add(".dwar");
 			break;
 		case cFileTypeDataWarriorQuery:
-			extension = ".dwaq";
+			extensions.add(".dwaq");
 			break;
 		case cFileTypeDataWarriorTemplate:
-			extension = ".dwat";
+			extensions.add(".dwat");
 			break;
 		case cFileTypeDataWarriorMacro:
-			extension = ".dwam";
+			extensions.add(".dwam");
 			break;
 		case cFileTypeTextTabDelimited:
-			extension = ".txt";
+			extensions.add(".txt");
+			extensions.add(".tsv");
 			break;
+		case cFileTypeTextAnyCSV:
         case cFileTypeTextCommaSeparated:
-            extension = ".csv";
+		case cFileTypeTextSemicolonSeparated:
+		case cFileTypeTextVLineSeparated:
+            extensions.add(".csv");
             break;
 		case cFileTypeSD:
         case cFileTypeSDV2:
         case cFileTypeSDV3:
-			extension = ".sdf";
+			extensions.add(".sdf");
 			break;
 		case cFileTypeRD:
         case cFileTypeRDV2:
         case cFileTypeRDV3:
-			extension = ".rdf";
+			extensions.add(".rdf");
 			break;
 		case cFileTypeRXN:
-			extension = ".rxn";
+			extensions.add(".rxn");
 			break;
 		case cFileTypeSOM:
-			extension = ".dwas";
+			extensions.add(".dwas");
 			break;
 		case cFileTypeJPG:
-			extension = ".jpeg";
+			extensions.add(".jpeg");
+			extensions.add(".jpg");
 			break;
 		case cFileTypeGIF:
-			extension = ".gif";
+			extensions.add(".gif");
 			break;
 		case cFileTypePNG:
-			extension = ".png";
+			extensions.add(".png");
 			break;
 		case cFileTypeSVG:
-			extension = ".svg";
+			extensions.add(".svg");
 			break;
 		case cFileTypeMOL:
-			extension = ".mol";
+			extensions.add(".mol");
 			break;
 		case cFileTypeMOL2:
-			extension = ".mol2";
+			extensions.add(".mol2");
 			break;
 		case cFileTypePDB:
-			extension = ".pdb";
+			extensions.add(".pdb");
 			break;
 		case cFileTypeMMTF:
-			extension = ".mmtf";
+			extensions.add(".mmtf");
 			break;
 		case cFileTypeSDGZ:
-			extension = ".sdf.gz";
+			extensions.add(".sdf.gz");
 			break;
 			}
-		return extension;
+		return extensions.toArray(new String[0]);
 		}
 
 	public void saveRXNFile(Reaction rxn) {
