@@ -1,9 +1,11 @@
 package com.actelion.research.chem.phesa;
 import com.actelion.research.chem.Canonizer;
 import com.actelion.research.chem.Coordinates;
+import com.actelion.research.chem.IDCodeParser;
 import com.actelion.research.chem.IDCodeParserWithoutCoordinateInvention;
 import com.actelion.research.chem.Molecule;
 import com.actelion.research.chem.StereoMolecule;
+import com.actelion.research.chem.alignment3d.PheSAAlignmentOptimizer.SimilarityMode;
 import com.actelion.research.chem.alignment3d.transformation.ExponentialMap;
 import com.actelion.research.chem.alignment3d.transformation.Quaternion;
 import com.actelion.research.chem.alignment3d.transformation.Transformation;
@@ -288,10 +290,14 @@ public class PheSAAlignment {
 	}
 	
 	public double[] findAlignment(double[][] initialTransforms, TransformationSequence optimizedTransform, boolean optimize) {
-		return findAlignment(initialTransforms,optimizedTransform,optimize,false);
+		return findAlignment(initialTransforms,optimizedTransform,optimize,SimilarityMode.TANIMOTO);
 	}
 	
-	public double[] findAlignment(double[][] initialTransforms, TransformationSequence optimizedTransform,boolean optimize, boolean tversky) {
+	public double[] findAlignment(double[][] initialTransforms, TransformationSequence optimizedTransform,boolean optimize, SimilarityMode simMode) {
+		boolean tversky = true;
+		if(simMode==SimilarityMode.TANIMOTO)
+			tversky=false;
+		double tverskyCoeff = simMode==SimilarityMode.TVERSKY ? TVERSKY_COEFFICIENT : 1.0-TVERSKY_COEFFICIENT;
 		double Oaa = getSelfAtomOverlapRef();
 		double Obb = getSelfAtomOverlapFit();
 		double ppOaa = getSelfPPOverlapRef();
@@ -322,7 +328,7 @@ public class PheSAAlignment {
 				ppSimilarity = 1.0;
 			else {
 				if(tversky)
-					ppSimilarity = ppOverlap/(TVERSKY_COEFFICIENT*ppObb+(1.0-TVERSKY_COEFFICIENT)*ppOaa);
+					ppSimilarity = ppOverlap/(tverskyCoeff*ppObb+(1.0-tverskyCoeff)*ppOaa);
 				else
 					ppSimilarity=(ppOverlap/(ppOaa+ppObb-ppOverlap));
 			}
@@ -334,7 +340,7 @@ public class PheSAAlignment {
 			atomOverlap = result[0];
 			double additionalVolOverlap = result[1];
 			if(tversky)
-				atomSimilarity = atomOverlap/(TVERSKY_COEFFICIENT*Obb+(1.0-TVERSKY_COEFFICIENT)*Oaa);
+				atomSimilarity = atomOverlap/(tverskyCoeff*Obb+(1.0-tverskyCoeff)*Oaa);
 			else
 				atomSimilarity = atomOverlap/(Oaa+Obb-atomOverlap);
 			if(!tversky && atomSimilarity>1.0) //can happen because of weights
@@ -356,7 +362,7 @@ public class PheSAAlignment {
 		transformation.addTransformation(translate);
 		for(Transformation trans : transformation.getTransformations())
 			optimizedTransform.addTransformation(trans);
-		if(maxSimilarity>1.0) // can happen because of manually placed inclusion spheres
+		if(!tversky && maxSimilarity>1.0) // can happen because of manually placed inclusion spheres
 			maxSimilarity = 1.0;
 		return Arrays.stream(new double[] {maxSimilarity,maxPPSimilarity,maxShapeSimilarity,maxVolSimilarity}).toArray();
 		}
@@ -405,17 +411,25 @@ public class PheSAAlignment {
 	public static class PheSAResult implements Comparable <PheSAResult>{
 		private StereoMolecule refMol;
 		private StereoMolecule fitMol;
+		private StereoMolecule fitInput;
+		
 		private double sim;
 		private double[] contributions;
 		private static final String DELIMITER = ";";
 		
-		public PheSAResult(StereoMolecule refMol, StereoMolecule fitMol, double sim) {
+		public PheSAResult(StereoMolecule refMol, StereoMolecule fitInput, StereoMolecule fitMol, double sim) {
 			this.refMol = refMol;
 			this.fitMol = fitMol;
 			this.sim = sim;
 			this.contributions = new double[4];
+			this.fitInput = fitInput;
 		}
 		
+		public void setFitInput(StereoMolecule fitInput) {
+			this.fitInput = fitInput;
+		}
+
+
 		public StereoMolecule getRefMol() {
 			return refMol;
 		}
@@ -456,6 +470,9 @@ public class PheSAAlignment {
 			sb.append(encoder.encodeToString(EncodeFunctions.doubleToByteArray(sim)));
 			sb.append(DELIMITER);
 			sb.append(encoder.encodeToString(EncodeFunctions.doubleArrayToByteArray(contributions)));
+			sb.append(DELIMITER);
+			sb.append(fitInput.getIDCode());
+			
 			return sb.toString();
 		}
 		
@@ -476,7 +493,9 @@ public class PheSAAlignment {
 			fitMol.ensureHelperArrays(Molecule.cHelperCIP);
 			double sim = EncodeFunctions.byteArrayToDouble(decoder.decode(s[4].getBytes()));
 			double[] contributions = EncodeFunctions.byteArrayToDoubleArray(decoder.decode(s[5].getBytes()));
-			PheSAResult pheSAResult = new PheSAResult(refMol,fitMol,sim);
+			StereoMolecule fitInput = new StereoMolecule();
+			new IDCodeParser().parse(fitInput,s[6]);
+			PheSAResult pheSAResult = new PheSAResult(refMol,fitInput,fitMol,sim);
 			pheSAResult.setContributions(contributions);
 			return pheSAResult;
 		}
