@@ -372,8 +372,8 @@ public class ReactionSearch {
 							break;
 
 						if (mSpecification.isSubreactionSearch()) {
-							long[] reactantFFP = (long[])mDataSource.getReactantDescriptor(row);
-							long[] productFFP = (long[])mDataSource.getProductDescriptor(row);
+							long[] reactantFFP = mDataSource.getMergedReactantDescriptor(row);
+							long[] productFFP = mDataSource.getMergedProductDescriptor(row);
 							mSRSearcher.setReaction(mDataSource.getReactionCode(row), mDataSource.getMapping(row), mDataSource.getCoordinates(row), reactantFFP, productFFP);
 
 							for (int i = 0; i<mQueryReaction.length; i++) {
@@ -387,7 +387,7 @@ public class ReactionSearch {
 						else {  // retron search
 							for (int i = 0; i<mQueryRetron.length; i++) {
 								if (!mDescriptorHandlerFFP512.calculationFailed(mQueryRetronDescriptor[i])) {
-									long[] productFFP = mDataSource.getProductDescriptor(row);
+									long[] productFFP = mDataSource.getMergedProductDescriptor(row);
 									mProductSearcher.setFragment(mQueryRetron[i], mQueryRetronDescriptor[i]);
 									mProductSearcher.setMolecule((StereoMolecule)null, productFFP);
 									if (mProductSearcher.isFragmentIndexInMoleculeIndex()) {
@@ -396,9 +396,9 @@ public class ReactionSearch {
 												mDataSource.getCoordinates(row),
 												mDataSource.getMapping(row), false, true));
 										mProductSearcher.setMolecule(product, productFFP);
-										int inProductCount = mProductSearcher.findFragmentInMoleculeWithoutIndex(SSSearcher.cCountModeOverlapping);
+										int inProductCount = mProductSearcher.findFragmentInMoleculeWithoutIndex(SSSearcher.cCountModeSeparated);
 										if (inProductCount != 0) {
-											long[] reactantFFP = mDataSource.getReactantDescriptor(row);
+											long[] reactantFFP = mDataSource.getMergedReactantDescriptor(row);
 											mReactantSearcher.setFragment(mQueryRetron[i], mQueryRetronDescriptor[i]);
 											mReactantSearcher.setMolecule((StereoMolecule)null, reactantFFP);
 											int inReactantCount = 0;
@@ -408,11 +408,11 @@ public class ReactionSearch {
 														mDataSource.getCoordinates(row),
 														mDataSource.getMapping(row), true, false));
 												mReactantSearcher.setMolecule(reactant, reactantFFP);
-												inReactantCount = mReactantSearcher.findFragmentInMoleculeWithoutIndex(SSSearcher.cCountModeOverlapping);
+												inReactantCount = mReactantSearcher.findFragmentInMoleculeWithoutIndex(SSSearcher.cCountModeSeparated);
 												if (inReactantCount != 0 && mDataSource.getMapping(row) != null) {
-													inProductCount -= countEquivalentMatches(product, mProductSearcher.getMatchList());
+													inProductCount -= countEquivalentMatches(product, mProductSearcher.getGraphMatcher().getMatchList());
 													if (inProductCount <= inReactantCount)
-														inReactantCount -= countEquivalentMatches(reactant, mReactantSearcher.getMatchList());
+														inReactantCount -= countEquivalentMatches(reactant, mReactantSearcher.getGraphMatcher().getMatchList());
 													}
 												}
 											// TODO check, whether we also have to take into account in catalyst occurences
