@@ -193,17 +193,25 @@ public class Reactor {
 		}
 
 	private int[] getSortedConnectedAtomicNos(StereoMolecule mol, int atom) {
-		int[] atomicNo = new int[mol.getConnAtoms(atom)];
-		for (int i=0; i<mol.getConnAtoms(atom); i++)
-			atomicNo[i] = mol.getAtomicNo(mol.getConnAtom(atom, i));
+		int[] atomicNo = new int[mol.getConnAtoms(atom) - mol.getExcludedNeighbourCount(atom)];
+		int index = 0;
+		for (int i=0; i<mol.getConnAtoms(atom); i++) {
+			int connAtom = mol.getConnAtom(atom, i);
+			if ((mol.getAtomQueryFeatures(connAtom) & Molecule.cAtomQFExcludeGroup) == 0)
+				atomicNo[index++] = mol.getAtomicNo(connAtom);
+			}
 		Arrays.sort(atomicNo);
 		return atomicNo;
 		}
 
 	private int[] getSortedConnectedMapNos(StereoMolecule mol, int atom) {
-		int[] mapNo = new int[mol.getConnAtoms(atom)];
-		for (int i=0; i<mol.getConnAtoms(atom); i++)
-			mapNo[i] = mol.getAtomMapNo(mol.getConnAtom(atom, i));
+		int[] mapNo = new int[mol.getConnAtoms(atom) - mol.getExcludedNeighbourCount(atom)];
+		int index = 0;
+		for (int i=0; i<mol.getConnAtoms(atom); i++) {
+			int connAtom = mol.getConnAtom(atom, i);
+			if ((mol.getAtomQueryFeatures(connAtom) & Molecule.cAtomQFExcludeGroup) == 0)
+				mapNo[index++] = mol.getAtomMapNo(connAtom);
+			}
 		Arrays.sort(mapNo);
 		return mapNo;
 		}
@@ -263,9 +271,10 @@ public class Reactor {
 	 * Matches are removed, if free valences are not compatible with the formation of new bonds.
 	 * @param no
 	 * @param reactant with correctly set parity flags
+	 * @return whether the generic reactant was found in this real world reactant
 	 */
 	@SuppressWarnings("unchecked")
-	public void setReactant(int no, StereoMolecule reactant) {
+	public boolean setReactant(int no, StereoMolecule reactant) {
 		mReactant[no] = reactant;
 		mProduct = null;
 		StereoMolecule genericReactant = mGenericReaction.getReactant(no);
@@ -275,7 +284,7 @@ public class Reactor {
 		if (mSSSearcher.findFragmentInMolecule(SSSearcher.cCountModeRigorous, matchMode) == 0) {
 			mMatchList[no] = new ArrayList<>();
 			mReactantMatchCombinationCount = 0;
-			return;
+			return false;
 			}
 
 		// eliminate matches where reaction would exceed an atom valence
@@ -307,6 +316,8 @@ public class Reactor {
 		mReactantMatchCombinationCount = 1;
 		for (int i=0; i<mReactant.length; i++)
 			mReactantMatchCombinationCount *= (mMatchList[i] == null) ? 0 : mMatchList[i].size();
+
+		return true;
 		}
 
 
@@ -553,7 +564,7 @@ public class Reactor {
 								product.setAtomRadical(newAtomNo[j], genericProduct.getAtomRadical(k));
 								if (mIsReactionCenter[genericProductNo][k]) {
 									int parity = mReactant[i].getAtomParity(j);
-									int productParityHint = genericProduct.getAtomQueryFeatures(k) & Molecule.cAtomQFRxnParityHint;
+									long productParityHint = genericProduct.getAtomQueryFeatures(k) & Molecule.cAtomQFRxnParityHint;
 									if (productParityHint == 0) {
 										product.setAtomParity(newAtomNo[j], Molecule.cAtomParityUnknown, false);
 										}
