@@ -39,9 +39,11 @@ import com.actelion.research.gui.clipboard.IClipboardHandler;
 import com.actelion.research.gui.dnd.MoleculeDragAdapter;
 import com.actelion.research.gui.dnd.MoleculeDropAdapter;
 import com.actelion.research.gui.dnd.MoleculeTransferable;
+import com.actelion.research.gui.editor.SwingEditorDialog;
+import com.actelion.research.gui.generic.GenericRectangle;
 import com.actelion.research.gui.hidpi.HiDPIHelper;
 import com.actelion.research.util.ColorHelper;
-import com.actelion.research.util.CursorHelper;
+import com.actelion.research.gui.swing.SwingCursorHelper;
 
 import javax.swing.*;
 import java.awt.*;
@@ -63,7 +65,7 @@ public class CompoundCollectionPane<T> extends JScrollPane
 			implements ActionListener,CompoundCollectionListener,MouseListener,MouseMotionListener,StructureListener {
 	private static final long serialVersionUID = 0x20060904;
 
-	private static final String[] MESSAGE = { "<to add compounds use popup menu,", "drag&drop, or paste structure or name(s)>" };
+	private static final String[] MESSAGE = { "<to add compounds use popup menu,", "drag&drop, or paste structure(s) or name(s)>" };
 
 	private static final String ADD = "Add...";
 	private static final String EDIT = "Edit...";
@@ -89,12 +91,14 @@ public class CompoundCollectionPane<T> extends JScrollPane
 	private CompoundCollectionModel<T> mModel;
 	private IClipboardHandler   mClipboardHandler;
 	private MoleculeFilter		mCompoundFilter;
-	private int					mDragActions,mDisplayMode,mSelectedIndex,mHighlightedIndex,
+	private int					mDisplayMode,mSelectedIndex,mHighlightedIndex,
 								mEditedIndex,mFileSupport,mStructureSize,mDragIndex,mDropIndex;
 	private Dimension		    mContentSize,mCellSize;
 	private JPanel			    mContentPanel;
 	private boolean			    mIsVertical,mIsEditable,mIsSelectable,mCreateFragments,
 								mIsEnabled,mShowValidationError,mInternalDragAndDropIsMove;
+	private String[]            mMessage;
+	private ArrayList<JMenuItem> mCustomPopupItemList;
 	private ScrollPaneAutoScrollerWhenDragging mScroller;
 
 
@@ -226,6 +230,29 @@ public class CompoundCollectionPane<T> extends JScrollPane
 		return mClipboardHandler;
 		}
 
+	public void addCustomPopupItem(JMenuItem customItem) {
+		if (mCustomPopupItemList == null)
+			mCustomPopupItemList = new ArrayList<>();
+
+		mCustomPopupItemList.add(customItem);
+		}
+
+	/**
+	 * @param msg null for default message or custom message to show if this pane is empty
+	 */
+	public void setMessage(String msg) {
+		if (msg == null) {
+			mMessage = null;
+			}
+		else {
+			mMessage = new String[1];
+			mMessage[0] = msg;
+			}
+		if (mModel.getSize() == 0)
+			repaint();
+		}
+
+	@Override
 	public void actionPerformed(ActionEvent e) {
 		if (e.getActionCommand().equals(COPY) && mHighlightedIndex != -1) {
 			mClipboardHandler.copyMolecule(mModel.getMolecule(mHighlightedIndex));
@@ -380,7 +407,7 @@ public class CompoundCollectionPane<T> extends JScrollPane
 			mol = mModel.getMolecule(mEditedIndex);
 			}
 		Component c = getParentFrame();
-		JDrawDialog theDialog = (c instanceof Frame) ? new JDrawDialog((Frame)c, mol) : new JDrawDialog((Dialog)c, mol);
+		SwingEditorDialog theDialog = (c instanceof Frame) ? new SwingEditorDialog((Frame)c, mol) : new SwingEditorDialog((Dialog)c, mol);
 		theDialog.addStructureListener(this);
 		theDialog.setVisible(true);
 		}
@@ -446,7 +473,7 @@ public class CompoundCollectionPane<T> extends JScrollPane
 
 						Depictor2D d = new Depictor2D(compound, mDisplayMode);
 						d.validateView((Graphics2D)g,
-									   new Rectangle2D.Double(bounds.x, bounds.y, bounds.width, bounds.height),
+									   new GenericRectangle(bounds.x, bounds.y, bounds.width, bounds.height),
 									   AbstractDepictor.cModeInflateToMaxAVBL);
 
 						d.setForegroundColor(foreground, background);
@@ -466,10 +493,11 @@ public class CompoundCollectionPane<T> extends JScrollPane
 					g.setColor(foreground);
 					g.setFont(g.getFont().deriveFont(Font.PLAIN, HiDPIHelper.scale(12)));
 					FontMetrics m = g.getFontMetrics();
-					for (int i=0; i<MESSAGE.length; i++) {
-						Rectangle2D b = m.getStringBounds(MESSAGE[i], g);
-						g.drawString(MESSAGE[i], bounds.x + (bounds.width - (int)b.getWidth()) / 2,
-								bounds.y + i*m.getHeight() + (bounds.height - MESSAGE.length * m.getHeight()) / 2 + m.getAscent());
+					String[] message = (mMessage == null) ? MESSAGE : mMessage;
+					for (int i=0; i<message.length; i++) {
+						Rectangle2D b = m.getStringBounds(message[i], g);
+						g.drawString(message[i], bounds.x + (bounds.width - (int)b.getWidth()) / 2,
+								bounds.y + i*m.getHeight() + (bounds.height - message.length * m.getHeight()) / 2 + m.getAscent());
 						}
 					}
 
@@ -556,7 +584,7 @@ public class CompoundCollectionPane<T> extends JScrollPane
 			int index = getMoleculeIndex(e.getX(), e.getY());
 			if (mHighlightedIndex != index) {
 				mHighlightedIndex = index;
-				setCursor(CursorHelper.getCursor(index == -1 ? CursorHelper.cPointerCursor : CursorHelper.cHandCursor));
+				setCursor(SwingCursorHelper.getCursor(index == -1 ? SwingCursorHelper.cPointerCursor : SwingCursorHelper.cHandCursor));
 				repaint();
 				}
 			}
@@ -615,6 +643,12 @@ public class CompoundCollectionPane<T> extends JScrollPane
 				}
 			}
 
+		if (mCustomPopupItemList != null) {
+			popup.addSeparator();
+			for (JMenuItem customItem:mCustomPopupItemList)
+				popup.add(customItem);
+			}
+
 		popup.show(this, e.getX(), e.getY());
 		}
 
@@ -676,13 +710,12 @@ public class CompoundCollectionPane<T> extends JScrollPane
 		}
 
 	private void initializeDragAndDrop(int dragAction, int dropAction) {
-		mDragActions = dragAction;
 		if (dragAction != DnDConstants.ACTION_NONE) {
 			new MoleculeDragAdapter(this) {
 				public Transferable getTransferable(Point p) {
 					if (mHighlightedIndex == -1)
 						return null;
-					setCursor(CursorHelper.getCursor(CursorHelper.cFistCursor));
+					setCursor(SwingCursorHelper.getCursor(SwingCursorHelper.cFistCursor));
 					mDragIndex = mHighlightedIndex;
 					return new MoleculeTransferable(mModel.getMolecule(mHighlightedIndex));
 					}
