@@ -38,16 +38,19 @@ import com.actelion.research.chem.io.CompoundFileHelper;
 import com.actelion.research.chem.io.RDFileParser;
 import com.actelion.research.chem.io.RXNFileParser;
 import com.actelion.research.chem.reaction.Reaction;
+import com.actelion.research.chem.reaction.ReactionEncoder;
 import com.actelion.research.gui.clipboard.ClipboardHandler;
 import com.actelion.research.gui.dnd.MoleculeDropAdapter;
 import com.actelion.research.gui.dnd.MoleculeTransferable;
 import com.actelion.research.gui.dnd.ReactionDropAdapter;
 import com.actelion.research.gui.dnd.ReactionTransferable;
+import com.actelion.research.gui.editor.GenericEditorArea;
 import com.actelion.research.gui.generic.GenericDrawContext;
+import com.actelion.research.gui.generic.GenericRectangle;
 import com.actelion.research.gui.hidpi.HiDPIHelper;
+import com.actelion.research.gui.swing.SwingCursorHelper;
 import com.actelion.research.gui.swing.SwingDrawContext;
 import com.actelion.research.util.ColorHelper;
-import com.actelion.research.util.CursorHelper;
 
 import javax.swing.*;
 import java.awt.*;
@@ -69,6 +72,7 @@ public class JChemistryView extends JComponent
 
 	private static final String ITEM_COPY_RXN = "Copy Reaction";
 	private static final String ITEM_PASTE_RXN = "Paste Reaction";
+	private static final String ITEM_USE_TEMPLATE = "Use Template";
 	private static final String ITEM_OPEN_RXN = "Open Reaction File...";
 	private static final String ITEM_SAVE_RXN = "Save Reaction File...";
 	private static final String ITEM_COPY_MOLS = "Copy Molecules";
@@ -94,8 +98,8 @@ public class JChemistryView extends JComponent
 	private ArrayList<StructureListener> mListener;
 	private Dimension           mSize;
 	private int					mChemistryType,mUpdateMode,mDisplayMode,mDragType,mCopyOrDragActions,mPasteOrDropActions,mPasteAndDropOptions;
-	private boolean				mIsDragging,mAllowDropOrPasteWhenDisabled,mIsEditable,mShowBorder,mOpaqueBackground;
-	private Color				mFragmentNoColor;
+	private boolean				mIsDragging,mAllowDropOrPasteWhenDisabled,mIsEditable,mShowBorder;
+	private int 				mFragmentNoColor;
 	private MoleculeDropAdapter mMoleculeDropAdapter = null;
 	private ReactionDropAdapter mReactionDropAdapter = null;
 	private String              mWarningMessage;
@@ -180,10 +184,6 @@ public class JChemistryView extends JComponent
 		repaint();
 		}
 
-	public void setOpaqueBackground(boolean b) {
-		mOpaqueBackground = b;
-		}
-
 	/**
 	 * fragment status change on drop is allowed then dropping a fragment (molecule)
 	 * on a molecule (fragment) inverts the status of the view's chemical object.
@@ -208,9 +208,10 @@ public class JChemistryView extends JComponent
 			if (mReactionDropAdapter != null)
 				mReactionDropAdapter.setActive(enable);
 			if (enable)
-				mDepictor.setOverruleColor(null, null);
+				mDepictor.setOverruleColor(0, 0);
 			else
-				mDepictor.setOverruleColor(ColorHelper.getContrastColor(Color.GRAY, getBackground()), getBackground());
+				mDepictor.setOverruleColor(0xFF000000 | ColorHelper.getContrastColor(0xFF808080,
+						getBackground().getRGB()), 0xFF000000 | getBackground().getRGB());
 			repaint();
 			}
 		}
@@ -224,11 +225,14 @@ public class JChemistryView extends JComponent
 			mIsEditable = b;
 		}
 
-	public void setFragmentNoColor(Color c) {
-	        // use setFragmentNoColor(null) if you don't want fragment numbers to be shown
-		mFragmentNoColor = c;
+	/**
+	 * Use setFragmentNoColor(0) if you don't want fragment numbers to be shown
+	 * @param argb
+	 */
+	public void setFragmentNoColor(int argb) {
+		mFragmentNoColor = argb;
 		if (mDepictor != null)
-			mDepictor.setFragmentNoColor(c);
+			mDepictor.setFragmentNoColor(argb);
 		}
 
 	public void setDisplayMode(int displayMode) {
@@ -257,11 +261,11 @@ public class JChemistryView extends JComponent
 		 || mSize.width != theSize.width
 		 || mSize.height != theSize.height
 		 || mUpdateMode == UPDATE_SCALE_COORDS) {
-			mDepictor.validateView(context, new Rectangle2D.Double(insets.left, insets.top, theSize.width, theSize.height),
+			mDepictor.validateView(context, new GenericRectangle(insets.left, insets.top, theSize.width, theSize.height),
 									AbstractDepictor.cModeInflateToMaxAVBL);
 			}
 		else if (mUpdateMode == UPDATE_CHECK_COORDS) {
-			mDepictor.validateView(context, new Rectangle2D.Double(insets.left, insets.top, theSize.width, theSize.height), 0);
+			mDepictor.validateView(context, new GenericRectangle(insets.left, insets.top, theSize.width, theSize.height), 0);
 			}
 
 		mSize = theSize;
@@ -272,17 +276,23 @@ public class JChemistryView extends JComponent
 		g2.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS, RenderingHints.VALUE_FRACTIONALMETRICS_ON);
 
 		Color fg = g2.getColor();
-		Color bg = UIManager.getColor(isEditable() && isEnabled() ? "TextField.background" : "TextField.inactiveBackground");
+		Color bg = null;
+		if (!isEditable() || !isEnabled())
+			bg = UIManager.getColor("TextField.inactiveBackground");
+		if (bg == null)
+			bg = UIManager.getColor("TextField.background");
+		if (bg == null)
+			bg = Color.WHITE;
 		g2.setColor(bg);
 		g2.fill(new Rectangle(insets.left, insets.top, theSize.width, theSize.height));
 		g2.setColor(fg);
 
-		mDepictor.setForegroundColor(getForeground(), getBackground());
+		mDepictor.setForegroundColor(getForeground().getRGB(), getBackground().getRGB());
 
 		if (mShowBorder && mDragType != DRAG_TYPE_NONE) {
 			Color color = ColorHelper.perceivedBrightness(bg) < 0.5f ? ColorHelper.brighter(bg, 0.85f) : ColorHelper.darker(bg, 0.85f);
 			g.setColor(color);
-			Rectangle2D.Double rect = (mDragType == DRAG_TYPE_REACTION) ? getChemistryBounds() : getMoleculeBounds(mDragType);
+			GenericRectangle rect = (mDragType == DRAG_TYPE_REACTION) ? getChemistryBounds() : getMoleculeBounds(mDragType);
 			int arc = (int)Math.min(rect.height/4, Math.min(rect.width/4, HiDPIHelper.scale(10)));
 			g.fillRoundRect((int)rect.x, (int)rect.y, (int)rect.width, (int)rect.height, arc, arc);
 			if (mDragType == DRAG_TYPE_REACTION) {
@@ -320,23 +330,23 @@ public class JChemistryView extends JComponent
 	 * @param i
 	 * @return bounds or null, if i is out of range
 	 */
-	public Rectangle2D.Double getMoleculeBounds(int i) {
+	public GenericRectangle getMoleculeBounds(int i) {
 		return mDepictor == null || i >= mDepictor.getMoleculeCount() ?
 				null : mDepictor.getMoleculeDepictor(i).getBoundingRect();
 		}
 
-	public Rectangle2D.Double getChemistryBounds() {
+	public GenericRectangle getChemistryBounds() {
 		if (mDepictor == null || mDepictor.getMoleculeCount() == 0)
 			return null;
 
-		Rectangle2D.Double rect = null;
+		GenericRectangle rect = null;
 		for (int i=0; i<mDepictor.getMoleculeCount(); i++) {
-			Rectangle2D.Double mrect = mDepictor.getMoleculeDepictor(i).getBoundingRect();
+			GenericRectangle mrect = mDepictor.getMoleculeDepictor(i).getBoundingRect();
 			if (mrect != null) {
 				if (rect == null)
 					rect = mrect;
 				else
-					rect = (Rectangle2D.Double)rect.createUnion(mrect);
+					rect = rect.union(mrect);
 				}
 			}
 		return rect;
@@ -396,6 +406,14 @@ public class JChemistryView extends JComponent
 				showWarningMessage("No reaction on clipboard!");
 			}
 
+		if (e.getActionCommand().startsWith(ITEM_USE_TEMPLATE) && mIsEditable) {
+			Reaction rxn = ReactionEncoder.decode(e.getActionCommand().substring(ITEM_USE_TEMPLATE.length()), true);
+			if (rxn != null)
+				pasteOrDropReaction(rxn);
+			else
+				showWarningMessage("Could not decode template!");
+			}
+
 		if (e.getActionCommand().equals(ITEM_OPEN_RXN) && mIsEditable) {
 			File rxnFile = FileHelper.getFile(this, "Please select a reaction file",
 					FileHelper.cFileTypeRXN | CompoundFileHelper.cFileTypeRD);
@@ -429,8 +447,8 @@ public class JChemistryView extends JComponent
 	public void mousePressed(MouseEvent e) {
 		handlePopupTrigger(e);
 
-		setCursor(CursorHelper.getCursor((mDragType == DRAG_TYPE_NONE) ?
-				CursorHelper.cPointerCursor : CursorHelper.cFistCursor));
+		setCursor(SwingCursorHelper.getCursor((mDragType == DRAG_TYPE_NONE) ?
+				SwingCursorHelper.cPointerCursor : SwingCursorHelper.cFistCursor));
 		}
 
 	@Override
@@ -468,6 +486,28 @@ public class JChemistryView extends JComponent
 						item.addActionListener(this);
 						popup.add(item);
 						}
+					if (GenericEditorArea.getReactionQueryTemplates() != null && mDepictor.getReaction() != null && mDepictor.getReaction().isFragment()) {
+						popup.addSeparator();
+						JMenu templateMenu = null;
+						for (String[] template:GenericEditorArea.getReactionQueryTemplates()) {
+							if (GenericEditorArea.TEMPLATE_SECTION_KEY.equals(template[0])) {
+								if (templateMenu != null)
+									popup.add(templateMenu);
+
+								templateMenu = new JMenu("Use "+template[1]+" Template");
+								continue;
+								}
+
+							if (templateMenu == null)
+								templateMenu = new JMenu("Use Template");
+
+							JMenuItem item = new JMenuItem(template[0]);
+							item.setActionCommand(ITEM_USE_TEMPLATE + template[1]);
+							item.addActionListener(this);
+							templateMenu.add(item);
+							}
+						popup.add(templateMenu);
+						}
 
 					if (popup.getComponentCount() != 0)
 						popup.addSeparator();
@@ -540,11 +580,11 @@ public class JChemistryView extends JComponent
 			}
 
 		updateBorder(dragType != DRAG_TYPE_NONE);
-		setCursor(CursorHelper.getCursor((mDragType == DRAG_TYPE_NONE) ?
-				CursorHelper.cPointerCursor : CursorHelper.cHandCursor));
+		setCursor(SwingCursorHelper.getCursor((mDragType == DRAG_TYPE_NONE) ?
+				SwingCursorHelper.cPointerCursor : SwingCursorHelper.cHandCursor));
 		}
 
-	private Rectangle shrink(Rectangle2D.Double rect) {
+	private Rectangle shrink(GenericRectangle rect) {
 		if (rect == null)
 			return null;
 
@@ -723,7 +763,7 @@ public class JChemistryView extends JComponent
 					getReactionTransferable() : getMoleculeTransferable();
 			if (transferable != null) {
 				try {
-					e.startDrag(CursorHelper.getCursor(CursorHelper.cFistCursor), transferable, this);
+					e.startDrag(SwingCursorHelper.getCursor(SwingCursorHelper.cFistCursor), transferable, this);
 					mIsDragging = true;
 //					e.startDrag(DragSource.DefaultCopyNoDrop, transferable, this);
 					}
