@@ -41,10 +41,11 @@ import com.actelion.research.gui.dnd.MoleculeDragAdapter;
 import com.actelion.research.gui.dnd.MoleculeDropAdapter;
 import com.actelion.research.gui.dnd.MoleculeTransferable;
 import com.actelion.research.gui.generic.GenericDepictor;
+import com.actelion.research.gui.generic.GenericRectangle;
 import com.actelion.research.gui.hidpi.HiDPIHelper;
 import com.actelion.research.gui.swing.SwingDrawContext;
 import com.actelion.research.util.ColorHelper;
-import com.actelion.research.util.CursorHelper;
+import com.actelion.research.gui.swing.SwingCursorHelper;
 
 import javax.swing.*;
 import java.awt.*;
@@ -82,6 +83,7 @@ public class JStructureView extends JComponent implements ActionListener,MouseLi
 	private String mWarningMessage;
 	private int[] mAtomHiliteColor;
 	private float[] mAtomHiliteRadius;
+	private double mTextSizeFactor;
 
 	public JStructureView() {
         this(null);
@@ -151,6 +153,7 @@ public class JStructureView extends JComponent implements ActionListener,MouseLi
 		mMol = (mol == null) ? new StereoMolecule() : new StereoMolecule(mol);
 		mDisplayMol = (displayMol == null) ? mMol : displayMol;
 		mDisplayMode = AbstractDepictor.cDModeHiliteAllQueryFeatures;
+		mTextSizeFactor = 1.0;
 		mIsEditable = false;
 		updateBackground();
 		addMouseListener(this);
@@ -192,6 +195,14 @@ public class JStructureView extends JComponent implements ActionListener,MouseLi
 			}
 	    }
 
+	/**
+	 * Sets a multiplication factor to the text size of all labels. The default is 1.0.
+	 * @param factor text size factor
+	 */
+	public void setFactorTextSize(double factor) {
+		mTextSizeFactor = factor;
+		}
+
 	public void setDisableBorder(boolean b) {
 		mDisableBorder = b;
 		}
@@ -278,7 +289,7 @@ public class JStructureView extends JComponent implements ActionListener,MouseLi
 		g2.fill(new Rectangle(insets.left, insets.top, theSize.width, theSize.height));
 
 		if (mShowBorder && !mDisableBorder) {
-			Rectangle2D.Double rect = mDepictor.getBoundingRect();
+			GenericRectangle rect = mDepictor.getBoundingRect();
 			if (rect != null) {
 				g.setColor(ColorHelper.perceivedBrightness(bg) < 0.5f ? ColorHelper.brighter(bg, 0.85f) : ColorHelper.darker(bg, 0.85f));
 				int arc = (int)Math.min(rect.height/4, Math.min(rect.width/4, HiDPIHelper.scale(10)));
@@ -291,17 +302,19 @@ public class JStructureView extends JComponent implements ActionListener,MouseLi
 		if (mDisplayMol != null && mDisplayMol.getAllAtoms() != 0) {
 			mDepictor = new GenericDepictor(mDisplayMol);
             mDepictor.setDisplayMode(mDisplayMode);
+            mDepictor.setFactorTextSize(mTextSizeFactor);
             mDepictor.setAtomText(mAtomText);
             mDepictor.setAtomHighlightColors(mAtomHiliteColor, mAtomHiliteRadius);
 
+            int bgRGB = bg.getRGB();
 			if (!isEnabled())
-                mDepictor.setOverruleColor(ColorHelper.getContrastColor(Color.GRAY, bg), bg);
+                mDepictor.setOverruleColor(ColorHelper.getContrastColor(0x808080, bgRGB), bgRGB);
 			else
-				mDepictor.setForegroundColor(getForeground(), bg);
+				mDepictor.setForegroundColor(getForeground().getRGB(), bgRGB);
 
 			int avbl = HiDPIHelper.scale(AbstractDepictor.cOptAvBondLen);
 			SwingDrawContext context = new SwingDrawContext((Graphics2D)g);
-			mDepictor.validateView(context, new Rectangle2D.Double(insets.left, insets.top, theSize.width,theSize.height),
+			mDepictor.validateView(context, new GenericRectangle(insets.left, insets.top, theSize.width,theSize.height),
 								   AbstractDepictor.cModeInflateToMaxAVBL | mChiralTextPosition | avbl);
             mDepictor.paint(context);
 			}
@@ -436,20 +449,20 @@ public class JStructureView extends JComponent implements ActionListener,MouseLi
 		int y = e.getY();
 		boolean isInRect = false;
 		if (mDepictor != null && (mAllowedDragAction & DnDConstants.ACTION_COPY) != 0) {
-			Rectangle bounds = shrink(mDepictor.getBoundingRect());
+			GenericRectangle bounds = shrink(mDepictor.getBoundingRect());
 			if (bounds != null && bounds.contains(x, y))
 				isInRect = true;
 			}
 
 		updateBorder(isInRect);
-		setCursor(CursorHelper.getCursor(isInRect ? CursorHelper.cHandCursor : CursorHelper.cPointerCursor));
+		setCursor(SwingCursorHelper.getCursor(isInRect ? SwingCursorHelper.cHandCursor : SwingCursorHelper.cPointerCursor));
 		}
 
-	private Rectangle shrink(Rectangle2D.Double rect) {
+	private GenericRectangle shrink(GenericRectangle rect) {
 		int margin = HiDPIHelper.scale(DRAG_MARGIN);
 		int marginX = Math.min(margin, (int)rect.width / 6);
 		int marginY = Math.min(margin, (int)rect.height / 6);
-		return new Rectangle((int)rect.x+marginX, (int)rect.y+marginY, (int)rect.width-2*marginX, (int)rect.height-2*marginY);
+		return new GenericRectangle((int)rect.x+marginX, (int)rect.y+marginY, (int)rect.width-2*marginX, (int)rect.height-2*marginY);
 	}
 
 	@Override public void mouseDragged(MouseEvent e) {}
