@@ -43,6 +43,7 @@ import com.actelion.research.chem.reaction.ReactionEncoder;
 import com.actelion.research.gui.clipboard.external.ChemDrawCDX;
 import com.actelion.research.gui.dnd.MoleculeTransferable;
 import com.actelion.research.gui.dnd.ReactionTransferable;
+import com.actelion.research.gui.generic.GenericRectangle;
 import com.actelion.research.gui.wmf.WMF;
 import com.actelion.research.gui.wmf.WMFGraphics2D;
 import com.actelion.research.util.Platform;
@@ -51,7 +52,6 @@ import com.actelion.research.util.Sketch;
 import java.awt.*;
 import java.awt.datatransfer.DataFlavor;
 import java.awt.datatransfer.Transferable;
-import java.awt.geom.Rectangle2D;
 import java.io.*;
 import java.util.ArrayList;
 
@@ -142,13 +142,33 @@ public class ClipboardHandler implements IClipboardHandler
 				ArrayList<String> unresolvedNameList = null;
 
 				if (molList.size() == 0) {
+					boolean isFirstLine = true;
+					int column = -1;    // in case we have a TAB-delimited table with '[idcode]' tags, use first tagged column
 					BufferedReader reader = new BufferedReader(new StringReader(text));
 					try {
 						String line = reader.readLine();
 						while (line != null) {
 							line = line.trim();
+							if (isFirstLine) {
+								String[] header = line.split("\\t");
+								for (int i=0; i<header.length; i++) {
+									if (header[i].endsWith("[idcode]")) {
+										column = i;
+										break;
+									}
+								}
+								isFirstLine = false;
+								if (column != -1)
+									continue;
+							}
 							try {
-								mol = new IDCodeParser(prefer2D).getCompactMolecule(line);
+								String idcode = line;
+								if (column != -1) {
+									String[] entry = line.split("\\t");
+									if (column < entry.length)
+										idcode = entry[column];
+								}
+								mol = new IDCodeParser(prefer2D).getCompactMolecule(idcode);
 							} catch (Exception e) {
 								mol = null;
 							}
@@ -284,9 +304,7 @@ public class ClipboardHandler implements IClipboardHandler
 		try {
 			Transferable t = Toolkit.getDefaultToolkit().getSystemClipboard().getContents(null);
 			return (StereoMolecule)t.getTransferData(ChemistryFlavors.DF_SERIALIZED_MOLECULE);
-		} catch (Exception e) {
-			System.err.println("error getting clipboard data "+ e);
-		}
+		} catch (Exception e) {}
 	return null;
 	}
 
@@ -378,9 +396,7 @@ public class ClipboardHandler implements IClipboardHandler
 		try {
 			Transferable t = Toolkit.getDefaultToolkit().getSystemClipboard().getContents(null);
 			return (Reaction)t.getTransferData(ChemistryFlavors.DF_SERIALIZED_REACTION);
-		} catch (Exception e) {
-			System.err.println("error getting clipboard data "+ e);
-		}
+		} catch (Exception e) {}
 		return null;
 	}
 
@@ -536,8 +552,8 @@ public class ClipboardHandler implements IClipboardHandler
 		WMF wmf = new WMF();
 		WMFGraphics2D g = new WMFGraphics2D(wmf, w, h, Color.black, Color.white);
 
-		Depictor d = new Depictor(m);
-		d.updateCoords(g, new Rectangle2D.Double(0, 0, w, h), AbstractDepictor.cModeInflateToMaxAVBL);
+		Depictor2D d = new Depictor2D(m);
+		d.updateCoords(g, new GenericRectangle(0, 0, w, h), AbstractDepictor.cModeInflateToMaxAVBL);
 		d.paint(g);
 
 		if (sketch != null) {
