@@ -2,14 +2,14 @@ package com.actelion.research.gui.dock;
 
 import javax.swing.*;
 import java.awt.*;
-import java.awt.event.AWTEventListener;
+import java.awt.datatransfer.Transferable;
+import java.awt.dnd.*;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
-import java.awt.event.MouseEvent;
 import java.util.List;
 import java.util.*;
 
-public class JDockingPanel extends JPanel implements ActionListener,AWTEventListener {
+public class JDockingPanel extends JPanel implements ActionListener {
 	private static final long serialVersionUID = 0x20070720;
 
 	public  static final int DOCK_CENTER = 0;
@@ -18,15 +18,17 @@ public class JDockingPanel extends JPanel implements ActionListener,AWTEventList
 	public  static final int DOCK_BOTTOM = 3;
 	public  static final int DOCK_RIGHT = 4;
 
+	public static final int ALLOWED_DRAG_DROP_ACTIONS = DnDConstants.ACTION_MOVE;   // currently no copy
+
 	private TreeMap<String,Dockable> mDockableMap;
 	private TreeMap<String,TreeLeaf> mLeafMap;
-	private Dockable mDraggedDockable,mPreviousTargetDockable;
+	private Dockable mPreviousTargetDockable;
 	private TreeRoot mTreeRoot;
 	private TreeLeaf mTargetLeaf;
 	private int mTargetPosition,mPreviousTargetPosition;
 	private GhostPreview mPreview;
 	private Dockable mMaximizedView;
-//	private ArrayList<ActionListener> mActionListenerList;
+	private Vector<DividerChangeListener> mDividerChangeListeners;
 
 	/**
 	 * Creates a docking panel to which any Dockables may be added by
@@ -36,31 +38,70 @@ public class JDockingPanel extends JPanel implements ActionListener,AWTEventList
 		mDockableMap = new TreeMap<>();
 		mLeafMap = new TreeMap<>();
 		mTargetPosition = -1;
-		try {   // applets throw a security exception which must be caught
-			Toolkit.getDefaultToolkit().addAWTEventListener(this, AWTEvent.MOUSE_EVENT_MASK
-																| AWTEvent.MOUSE_MOTION_EVENT_MASK);
-			}
-		catch (java.security.AccessControlException e) {}
 		mPreview = new GhostPreview();
 
+		// Unfortunately, this cannot be used to catch drag events of default JTable with active DropTarget
+//		Toolkit.getDefaultToolkit().addAWTEventListener(this, AWTEvent.MOUSE_MOTION_EVENT_MASK);
+
 		setLayout(new OverlayLayout(this));
+
+		new DropTarget(this, ALLOWED_DRAG_DROP_ACTIONS, new DropTargetAdapter() {
+			@Override
+			public void dragOver(DropTargetDragEvent dtde) {
+				Dockable dockable = getDraggedDockable(dtde);
+				if (dockable != null) {
+					Point p = dtde.getLocation();
+					Rectangle b = dockable.getHeader().getBounds();
+					p.translate(b.x, b.y);  // make p relative to Dockable
+					updatePreview(p, dockable);
+					}
+				}
+
+			@Override
+			public void drop(DropTargetDropEvent dtde) {
+				Transferable transferable = dtde.getTransferable();
+				if (mTargetPosition == -1
+				 || !transferable.isDataFlavorSupported(TransferableDockable.DF_DOCKABLE_DEF)) {
+// TODO do we need this?				fireDockableSelected(mDraggedDockable);
+					dtde.rejectDrop();
+					}
+				else {
+					dtde.acceptDrop(ALLOWED_DRAG_DROP_ACTIONS);
+					try {
+						String draggedTitle = (String)transferable.getTransferData(TransferableDockable.DF_DOCKABLE_DEF);
+						for (int i=0; i<mTargetLeaf.getDockableCount(); i++) {
+							if (!draggedTitle.equals(mTargetLeaf.getDockable(i).getTitle())) {
+								String targetTitle = mTargetLeaf.getDockable(i).getTitle();
+								relocateView(draggedTitle, targetTitle, mTargetPosition, 0.5f);
+								break;
+								}
+							}
+						dtde.dropComplete(true);
+						}
+					catch (Exception e) {}
+					}
+
+				mTargetPosition = -1;
+				}
+			}, true);
 		}
 
-/*	public void addActionListener(ActionListener l) {
-		if (l != null) {
-			if (mActionListenerList == null)
-				mActionListenerList = new ArrayList<ActionListener>();
-			mActionListenerList.add(l);
-			}
+	public void addDividerChangeLister(DividerChangeListener l) {
+		if (mDividerChangeListeners == null)
+			mDividerChangeListeners = new Vector<>();
+
+		mDividerChangeListeners.add(l);
+		if (mTreeRoot != null)
+			mTreeRoot.setDividerChangeListeners(mDividerChangeListeners);
 		}
 
-	public void removeActionListener(ActionListener l) {
-		if (l != null && mActionListenerList != null) {
-			mActionListenerList.remove(l);
-			if (mActionListenerList.size() == 0)
-				mActionListenerList = null;
+	public void removeDividerChangeLister(DividerChangeListener l) {
+		if (mDividerChangeListeners != null) {
+			mDividerChangeListeners.remove(l);
+			if (mTreeRoot != null)
+				mTreeRoot.setDividerChangeListeners(mDividerChangeListeners);
 			}
-		}*/
+		}
 
 	public void actionPerformed(ActionEvent e) {
 		if (e.getActionCommand().startsWith("close_")) {
@@ -69,18 +110,18 @@ public class JDockingPanel extends JPanel implements ActionListener,AWTEventList
 			}
 		else if (e.getActionCommand().startsWith("max_")) {
 			String title = e.getActionCommand().substring(4);
-			maximize(title);
+			maximize(title, null);
 			}
-
-/*		if (mActionListenerList != null)
-			for (ActionListener l:mActionListenerList)
-				l.actionPerformed(e);	*/
 		}
 
 	/**
 	 * Toggle the maximization state of the dockable in the docking panel.
+	 * If it is maximizing, then a toolbar may be provided that is shown in the maximized state.
+	 * Note that a null toolbar causes the maximized state to use the default toolbar with
+	 * maximize and close buttons only.
+	 * @param maximizeToolBar null for default or toolbar that shall be shown in maximized state
 	 */
-	public void maximize(String title) {
+	public void maximize(String title, JToolBar maximizeToolBar) {
 		Dockable dockable = mDockableMap.get(title);
 		if (mMaximizedView != null) {
 			remove(mMaximizedView);
@@ -95,7 +136,7 @@ public class JDockingPanel extends JPanel implements ActionListener,AWTEventList
 			JComponent content = dockable.borrowContent();
 
 			// create dummy dockable to hold the maximized view
-			mMaximizedView = new Dockable(this, content, title, this, false, dockable.hasMenuButton());
+			mMaximizedView = new Dockable(this, content, title, maximizeToolBar);
 			mMaximizedView.setPopupProvider(dockable.getPopupProvider());
 			mMaximizedView.setVisible(true);
 			add(mMaximizedView);
@@ -104,46 +145,6 @@ public class JDockingPanel extends JPanel implements ActionListener,AWTEventList
 			}
 		}
 
-	public void eventDispatched(AWTEvent event) {
-		if (!(event.getSource() instanceof DockableHeader)
-		 || !mDockableMap.containsValue(((DockableHeader)event.getSource()).getDockable())
-		 || mMaximizedView != null)
-			return;
-
-		if (event.getID() == MouseEvent.MOUSE_RELEASED) {
-			if (mTargetPosition == -1) {
-				fireDockableSelected(mDraggedDockable);
-				}
-			else {
-				String draggedTitle = mDraggedDockable.getTitle();
-				String targetTitle = null;
-				for (int i=0; i<mTargetLeaf.getDockableCount(); i++) {
-					if (!draggedTitle.equals(mTargetLeaf.getDockable(i).getTitle())) {
-						targetTitle = mTargetLeaf.getDockable(i).getTitle();
-						}
-					}
-				if (targetTitle != null)
-					relocateView(draggedTitle, targetTitle, mTargetPosition, 0.5f);
-				}
-			mTargetPosition = -1;
-			mDraggedDockable = null;
-			return;
-			}
-
-		if (event.getID() == MouseEvent.MOUSE_PRESSED) {
-			mDraggedDockable = ((DockableHeader)event.getSource()).getDockable();
-			selectDockable(mDraggedDockable);
-			return;
-			}
-
-		if (event.getID() == MouseEvent.MOUSE_DRAGGED) {
-			Point p = ((MouseEvent)event).getPoint();
-			Rectangle b = ((DockableHeader)event.getSource()).getBounds();
-			p.translate(b.x, b.y);  // make p relative to Dockable
-			updatePreview(p);
-			}
-		}
-
 	/**
 	 * Sends ActionEvent to listeners when the user interactively selected another dockable
 	 * @param dockable
@@ -152,12 +153,10 @@ public class JDockingPanel extends JPanel implements ActionListener,AWTEventList
 		actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "selected_"+dockable.getTitle()));
 		}
 
-	private void updatePreview(Point p) {
+	private void updatePreview(Point p, Dockable draggedDockable) {
 		mTargetPosition = -1;
 		Dockable targetDockable = null;
 
-		Rectangle b = getAbsoluteBounds(mDraggedDockable);
-		p.translate(b.x, b.y);  // make p relative to JDockingPanel
 		for (Dockable d:mDockableMap.values()) {
 			if (d.isVisible()) {
 				Rectangle bounds = getAbsoluteBounds(d.getContent());
@@ -169,7 +168,7 @@ public class JDockingPanel extends JPanel implements ActionListener,AWTEventList
 				}
 			}
 
-		updatePreview(targetDockable);
+		updatePreview(draggedDockable, targetDockable);
 		}
 
 	/**
@@ -204,12 +203,12 @@ public class JDockingPanel extends JPanel implements ActionListener,AWTEventList
 		return isTopOrLeft ? DOCK_LEFT : DOCK_BOTTOM;
 		}
 
-	private void updatePreview(Dockable targetDockable) {
+	private void updatePreview(Dockable draggedDockable, Dockable targetDockable) {
 		if (targetDockable == null) {
 			mTargetPosition = -1;
 			}
-		else if (targetDockable == mDraggedDockable) {
-			TreeLeaf sourceLeaf = mLeafMap.get(mDraggedDockable.getTitle());
+		else if (targetDockable == draggedDockable) {
+			TreeLeaf sourceLeaf = mLeafMap.get(draggedDockable.getTitle());
 			if (mTargetPosition == DOCK_CENTER
 			 || sourceLeaf.getDockableCount() == 1)
 				mTargetPosition = -1;
@@ -223,7 +222,7 @@ public class JDockingPanel extends JPanel implements ActionListener,AWTEventList
 		if (mPreviousTargetDockable != targetDockable
 		 || mPreviousTargetPosition != mTargetPosition) {
 			if (mTargetPosition != -1)
-				mPreview.createPreview(mDraggedDockable, targetDockable, mTargetPosition, this);
+				mPreview.createPreview(draggedDockable, targetDockable, mTargetPosition, this);
 			mPreviousTargetDockable = targetDockable;
 			mPreviousTargetPosition = mTargetPosition;
 			repaint();
@@ -328,7 +327,7 @@ public class JDockingPanel extends JPanel implements ActionListener,AWTEventList
 		else {
 			TreeLeaf newLeaf = new TreeLeaf(dockable, this, isDragging);
 			TreeContainer parent = treeLeaf.getParent();
-			TreeFork treeFork = new TreeFork(treeLeaf, newLeaf, position, dividerPosition);
+			TreeFork treeFork = new TreeFork(treeLeaf, newLeaf, position, dividerPosition, mDividerChangeListeners);
 			parent.replaceChildElement(treeLeaf, treeFork);
 			mLeafMap.put(dockable.getTitle(), newLeaf);
 			}
@@ -449,6 +448,34 @@ public class JDockingPanel extends JPanel implements ActionListener,AWTEventList
 			}
 		}
 
+	/**
+	 * Checks, whether a dockable with the given title exists, and whether it is visible (selected if in a JTabbedPane)
+	 * and whether it is in the left or right branch (depending on parameter left) of a JSplitPane.
+	 * @param title
+	 * @param left if true, then
+	 * @return true the respective dockable is visible and in the left part of a JSplitPane
+	 */
+	public boolean isVisibleInSplitPane(String title, boolean left) {
+		Component view = mDockableMap.get(title);
+		if (view == null)
+			return false;
+
+		Component pane = view.getParent();
+		if (pane instanceof JTabbedPane) {
+			if (view != ((JTabbedPane)pane).getSelectedComponent())
+				return false;
+
+			view = pane;
+			pane = pane.getParent();
+			}
+
+		if (pane instanceof JSplitPane)
+			return (left && view == ((JSplitPane)pane).getLeftComponent())
+				|| (!left && view == ((JSplitPane)pane).getRightComponent());
+
+		return false;
+		}
+
 	/**
 	 * changes a title of a docked Dockable
 	 * @param oldTitle existing title
@@ -503,6 +530,17 @@ public class JDockingPanel extends JPanel implements ActionListener,AWTEventList
 		return null;
 		}
 
+	public Dockable getDraggedDockable(DropTargetDragEvent dtde) {
+		Transferable transferable = dtde.getTransferable();
+		if (transferable.isDataFlavorSupported(TransferableDockable.DF_DOCKABLE_DEF)) {
+			try {
+				return getDockable((String)transferable.getTransferData(TransferableDockable.DF_DOCKABLE_DEF));
+				}
+			catch (Exception e) {}
+			}
+		return null;
+		}
+
 	/**
 	 * This is called when a Dockable's visibility changes, i.e.<br>
 	 * - when a new Dockable is docked<br>
@@ -604,7 +642,7 @@ public class JDockingPanel extends JPanel implements ActionListener,AWTEventList
 			Dockable dockable = dockables.get(size2);
 			bottomLeft = new TreeLeaf(dockable, this, false);
 			TreeContainer parent = topLeft.getParent();
-			TreeFork treeFork = new TreeFork(topLeft, bottomLeft, DOCK_BOTTOM, .5);	
+			TreeFork treeFork = new TreeFork(topLeft, bottomLeft, DOCK_BOTTOM, .5, mDividerChangeListeners);
 			parent.replaceChildElement(topLeft, treeFork);
 			mLeafMap.put(dockable.getTitle(), bottomLeft);
 			mDockableMap.put(dockable.getTitle(), dockable);
@@ -614,7 +652,7 @@ public class JDockingPanel extends JPanel implements ActionListener,AWTEventList
 			Dockable dockable = dockables.get(size1);
 			topRight = new TreeLeaf(dockable, this, false);
 			TreeContainer parent = topLeft.getParent();
-			TreeFork treeFork = new TreeFork(topLeft, topRight, DOCK_RIGHT, .5);	
+			TreeFork treeFork = new TreeFork(topLeft, topRight, DOCK_RIGHT, .5, mDividerChangeListeners);
 			parent.replaceChildElement(topLeft, treeFork);
 			mLeafMap.put(dockable.getTitle(), topRight);
 			mDockableMap.put(dockable.getTitle(), dockable);
@@ -624,7 +662,7 @@ public class JDockingPanel extends JPanel implements ActionListener,AWTEventList
 			Dockable dockable = dockables.get(size3);
 			bottomRight = new TreeLeaf(dockable, this, false);
 			TreeContainer parent = bottomLeft.getParent();
-			TreeFork treeFork = new TreeFork(bottomLeft, bottomRight, DOCK_RIGHT, .5);	
+			TreeFork treeFork = new TreeFork(bottomLeft, bottomRight, DOCK_RIGHT, .5, mDividerChangeListeners);
 			parent.replaceChildElement(bottomLeft, treeFork);
 			mLeafMap.put(dockable.getTitle(), bottomRight);
 			mDockableMap.put(dockable.getTitle(), dockable);
