@@ -2,7 +2,8 @@ package com.actelion.research.gui.generic;
 
 public interface GenericCanvas {
 	void repaint();
-	int getWidth();
-	int getHeight();
+	double getCanvasWidth();
+	double getCanvasHeight();
+	int getBackgroundRGB();
 	GenericDrawContext getDrawContext();
 }
