@@ -1,10 +1,8 @@
 package com.actelion.research.gui.generic;
 
-import java.awt.*;
-
 public interface GenericPopupMenu extends GenericComponent {
 	void addItem(String text, String command, boolean enabled);
-	void addRadioButtonItem(String text, String command, Color color, boolean isSelected);
+	void addRadioButtonItem(String text, String command, int rgb, boolean isSelected);
 	void startSubMenu(String text);
 	void endSubMenu();
 	void addSeparator();
