@@ -5,9 +5,10 @@ import com.actelion.research.chem.Depictor2D;
 import com.actelion.research.chem.ExtendedDepictor;
 import com.actelion.research.chem.StereoMolecule;
 import com.actelion.research.chem.reaction.Reaction;
+import com.actelion.research.gui.LookAndFeelHelper;
 import com.actelion.research.gui.generic.GenericDepictor;
 import com.actelion.research.gui.generic.GenericDrawContext;
-import com.actelion.research.gui.LookAndFeelHelper;
+import com.actelion.research.gui.generic.GenericRectangle;
 import com.actelion.research.gui.hidpi.HiDPIHelper;
 import com.actelion.research.gui.swing.SwingDrawContext;
 import com.actelion.research.util.ColorHelper;
@@ -15,15 +16,21 @@ import com.actelion.research.util.ColorHelper;
 import javax.swing.*;
 import javax.swing.border.EmptyBorder;
 import java.awt.*;
-import java.awt.geom.Rectangle2D;
 
 public class ChemistryRenderPanel extends JPanel {
     static final long serialVersionUID = 0x20070312;
 
     private Object  mChemistry;
-    private Color mOverruleForeground;
+    private int mDisplayMode,mOverruleForegroundARGB;
+    private double mTextSizeFactor;
     private boolean mAlternateBackground;
 
+    public ChemistryRenderPanel() {
+        mOverruleForegroundARGB = 0;
+        mDisplayMode = 0;
+        mTextSizeFactor = 1.0;
+        }
+
     public void setChemistry(Object chemistry) {
         mChemistry = chemistry;
         repaint();
@@ -52,14 +59,27 @@ public class ChemistryRenderPanel extends JPanel {
             setBorder(new EmptyBorder(1, 1, 1, 1));
         }
 
+    @Deprecated
     public void setOverruleForeground(Color fg) {
-		mOverruleForeground = fg;
+        mOverruleForegroundARGB = fg == null ? 0 : fg.getRGB();
     	}
 
+    public void setOverruleForeground(int argb) {
+        mOverruleForegroundARGB = argb;
+    }
+
     public void setAlternateBackground(boolean b) {
         mAlternateBackground = b;
         }
 
+    public void setDisplayMode(int mode) {
+        mDisplayMode = mode;
+        }
+
+    public void setTextSizeFactor(double factor) {
+        mTextSizeFactor = factor;
+        }
+
     public void paintComponent(Graphics g) {
         super.paintComponent(g);
         ((Graphics2D)g).setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
@@ -83,24 +103,27 @@ public class ChemistryRenderPanel extends JPanel {
 
         if (mChemistry != null && r.width > 0 && r.height > 0) {
             if (mChemistry instanceof StereoMolecule) {
-                GenericDepictor d = new GenericDepictor((StereoMolecule)mChemistry, Depictor2D.cDModeSuppressChiralText);
-                d.setForegroundColor(getForeground(), getBackground());
-                if (mOverruleForeground != null)
-                	d.setOverruleColor(mOverruleForeground, getBackground());
+                GenericDepictor d = new GenericDepictor((StereoMolecule)mChemistry, mDisplayMode | Depictor2D.cDModeSuppressChiralText);
+                d.setFactorTextSize(mTextSizeFactor);
+                d.setForegroundColor(getForeground().getRGB(), getBackground().getRGB());
+                if (mOverruleForegroundARGB != 0)
+                	d.setOverruleColor(mOverruleForegroundARGB, getBackground().getRGB());
                 int avbl = HiDPIHelper.scale(AbstractDepictor.cOptAvBondLen);
                 SwingDrawContext context = new SwingDrawContext((Graphics2D)g);
-                d.validateView(context, new Rectangle2D.Double(r.x, r.y, r.width, r.height), AbstractDepictor.cModeInflateToMaxAVBL | avbl);
+                d.validateView(context, new GenericRectangle(r.x, r.y, r.width, r.height), AbstractDepictor.cModeInflateToMaxAVBL | avbl);
                 d.paint(context);
                 }
             if (mChemistry instanceof Reaction) {
             	Reaction rxn = (Reaction)mChemistry;
                 ExtendedDepictor d = new ExtendedDepictor(rxn, rxn.getDrawingObjects(), rxn.isReactionLayoutRequired());
-                d.setForegroundColor(getForeground(), getBackground());
-                if (mOverruleForeground != null)
-                	d.setOverruleColor(mOverruleForeground, getBackground());
+                d.setFactorTextSize(mTextSizeFactor);
+                d.setDisplayMode(mDisplayMode | Depictor2D.cDModeSuppressChiralText);
+                d.setForegroundColor(getForeground().getRGB(), getBackground().getRGB());
+                if (mOverruleForegroundARGB != 0)
+                	d.setOverruleColor(mOverruleForegroundARGB, getBackground().getRGB());
                 int avbl = HiDPIHelper.scale(AbstractDepictor.cOptAvBondLen);
                 GenericDrawContext context = new SwingDrawContext((Graphics2D)g);
-                d.validateView(context, new Rectangle2D.Double(r.x, r.y, r.width, r.height), AbstractDepictor.cModeInflateToMaxAVBL | avbl);
+                d.validateView(context, new GenericRectangle(r.x, r.y, r.width, r.height), AbstractDepictor.cModeInflateToMaxAVBL | avbl);
                 d.paint(context);
                 }
             }
