@@ -209,10 +209,10 @@ public class JFXCanvasDepictor extends AbstractDepictor
 
 
     @Override
-    protected void setColor(java.awt.Color theColor)
+    protected void setRGB(int rgb)
     {
         if (ctx != null) {
-            Color color = new Color(theColor.getRed() / 255.0, theColor.getGreen() / 255.0, theColor.getBlue() / 255.0, 1.0);
+            Color color = new Color(((rgb & 0x00FF0000) >> 16) / 255.0, ((rgb & 0x0000FF00) >> 8) / 255.0, (rgb & 0x000000FF) / 255.0, 1.0);
             ctx.setStroke(color);
             ctx.setFill(color);
         }
